@@ -2,13 +2,12 @@
 
   main.c
 
-  ODBC (unixODBC) driver
   (c) 2004-2007 Andrea Bortolan <andrea_bortolan@yahoo.it>
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,11 +17,13 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
+
 ***************************************************************************/
 
 #define __MAIN_C
+
 //#define ODBC_TYPE
 //#define ODBC_DEBUG
 //#define DEBUG_ODBC
@@ -108,12 +109,11 @@ ODBC_TABLES;
 static void *my_malloc(size_t size)
 {
 	void *ptr;
-	GB.Alloc(&ptr, size);
-
+	GB.Alloc(&ptr, (int)size);
 	return ptr;
 }
 
-static void my_free(_ptr)
+static void my_free(void *_ptr)
 {
 	GB.Free(POINTER(&_ptr));
 }
@@ -325,59 +325,44 @@ static void conv_data(char *data, GB_VARIANT_VALUE * val, int type)
 
 	switch (type)
 	{
-
-
 		//case SQL_NUMERIC:
 		case SQL_DECIMAL:
 		case SQL_INTEGER:
 		case SQL_SMALLINT:
 	
-			GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);
 			val->type = GB_T_INTEGER;
-			val->value._integer = ((GB_INTEGER *) & conv)->value;
-			if (strlen(data) == 0)
+			
+			if (GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv))
 				val->value._integer = 0;
+			else
+				val->value._integer = conv._integer.value;
+
 			break;
 			
 		case SQL_NUMERIC:
 		case SQL_FLOAT:
 		case SQL_REAL:
-			
-			if (strlen(data) == 0) 
-			{
-				val->type = GB_T_FLOAT;
-				val->value._float = 0;
-			}
-			else 
-			{
-				GB.NumberFromString(GB_NB_READ_FLOAT, data, strlen(data), &conv);
-				val->type = GB_T_FLOAT;
-				val->value._float = ((GB_FLOAT *) & conv)->value;
-			}
-			break;
-
 		case SQL_DOUBLE:
 			
-			if (strlen(data) == 0) 
-			{
-				val->type = GB_T_FLOAT;
+			val->type = GB_T_FLOAT;
+			
+			if (GB.NumberFromString(GB_NB_READ_FLOAT, data, strlen(data), &conv))
 				val->value._float = 0;
-			}
-			else 
-			{
-				GB.NumberFromString(GB_NB_READ_FLOAT, data, strlen(data), &conv);
-				val->type = GB_T_FLOAT;
-				val->value._float = ((GB_FLOAT *) & conv)->value;
-			}
+			else
+				val->value._float = conv._float.value;
+			
 			break;
-
+			
+			
 // Data type bigint 64 bits
 		case SQL_BIGINT:
 
-			GB.NumberFromString(GB_NB_READ_LONG, data, strlen(data), &conv);
-
 			val->type = GB_T_LONG;
-			val->value._long = ((GB_LONG *) & conv)->value;
+			
+			if (GB.NumberFromString(GB_NB_READ_LONG, data, strlen(data), &conv))
+				val->value._long = 0;
+			else
+				val->value._long = conv._long.value;
 
 			break;
 
@@ -385,8 +370,9 @@ static void conv_data(char *data, GB_VARIANT_VALUE * val, int type)
    	case SQL_LONGVARCHAR:
 		case SQL_VARBINARY:
 		case SQL_LONGVARBINARY:
-    		// The BLOB are read by the blob_read() driver function
-    		// You must set NULL there.
+			
+			// The BLOB are read by the blob_read() driver function
+			// You must set NULL there.
 			val->type = GB_T_NULL;
 			break;
 
@@ -419,19 +405,14 @@ static void conv_data(char *data, GB_VARIANT_VALUE * val, int type)
 				GB.MakeDate(&date, (GB_DATE *) & conv);
 
 				val->type = GB_T_DATE;
-				val->value._date.date = ((GB_DATE *) & conv)->value.date;
-				val->value._date.time = ((GB_DATE *) & conv)->value.time;
+				val->value._date.date = conv._date.value.date;
+				val->value._date.time = conv._date.value.time;
 				break;
 			}
 
 		case SQL_CHAR:
-		//case SQL_NUMERIC:
-		
-		
 		default:
 
-			//val->_string.type = GB_T_STRING;
-			//GB.NewString(&val->_string.value, data, 0);
 			val->type = GB_T_CSTRING;
 			val->value._string = data;
 
@@ -543,6 +524,7 @@ fflush(stderr);
 
 	}
 
+	SQLSetConnectAttr(odbc->odbcHandle, SQL_ATTR_LOGIN_TIMEOUT, (SQLPOINTER)(intptr_t)db->timeout, 0);
 	/* Connect to Database (Data Source Name) */
 	retcode =SQLConnect(odbc->odbcHandle, (SQLCHAR *) desc->host, SQL_NTS, (SQLCHAR *) desc->user, SQL_NTS, (SQLCHAR *) desc->password, SQL_NTS);
 	if ((retcode != SQL_SUCCESS) && (retcode != SQL_SUCCESS_WITH_INFO))
@@ -594,7 +576,7 @@ fflush(stderr);
 
 static void close_database(DB_DATABASE *db)
 {
-  SQLRETURN retcode;
+  //SQLRETURN retcode;
 #ifdef ODBC_DEBUG_HEADER
 fprintf(stderr,"[ODBC][%s][%d]\n",__FILE__,__LINE__);
 fprintf(stderr,"\tclose_database\n");
@@ -603,37 +585,38 @@ fflush(stderr);
 	
 	ODBC_CONN *conn = (ODBC_CONN *)db->handle;
 
-	if (conn->odbcHandle != NULL){
-	retcode=SQLDisconnect(conn->odbcHandle);
-	}else 
-	{
-	GB.Error("ODBC module internal error");
-	}
+	if (conn->odbcHandle)
+		SQLDisconnect(conn->odbcHandle);
+	else
+		GB.Error("ODBC module internal error");
 
-
-	if (conn->odbcHandle != NULL){
-	retcode=SQLFreeHandle(SQL_HANDLE_DBC, conn->odbcHandle);
-	conn->odbcHandle=NULL;
-	}else 
+	if (conn->odbcHandle)
 	{
-	GB.Error("ODBC module internal error");
+		SQLFreeHandle(SQL_HANDLE_DBC, conn->odbcHandle);
+		conn->odbcHandle = NULL;
 	}
+	else 
+		GB.Error("ODBC module internal error");
 	
-	if (conn->odbcEnvHandle != NULL){
-	retcode=SQLFreeHandle(SQL_HANDLE_ENV, conn->odbcEnvHandle);
-	conn->odbcEnvHandle=NULL;
-	}else 
+	if (conn->odbcEnvHandle)
 	{
-	GB.Error("ODBC module internal error");
+		SQLFreeHandle(SQL_HANDLE_ENV, conn->odbcEnvHandle);
+		conn->odbcEnvHandle = NULL;
 	}
+	else 
+		GB.Error("ODBC module internal error");
 
-	if (conn->dsn_name != NULL)free(conn->dsn_name);
-	if (conn->user_name != NULL)free(conn->user_name);
-	if (conn != NULL){
-			SQL_Handle_free(conn);
-			db->handle=NULL;
+	if (conn->dsn_name)
+		free(conn->dsn_name);
+	
+	if (conn->user_name)
+		free(conn->user_name);
+	
+	if (conn)
+	{
+		SQL_Handle_free(conn);
+		db->handle = NULL;
 	}
-
 }
 
 
@@ -932,7 +915,7 @@ fflush(stderr);
 	SQLULEN precision;
 	SQLSMALLINT scale;
 	SQLINTEGER i;
-	SQLINTEGER displaysize;
+	SQLLEN displaysize;
 	ODBC_FIELDS *field, *current;
 	SQLINTEGER collen;
 	int nresultcols;
@@ -1101,16 +1084,14 @@ fflush(stderr);
 
 static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VALUE * buffer, 	int next)
 {
-
-
 	ODBC_RESULT *res = (ODBC_RESULT *) result;
 	GB_VARIANT value;
 	SQLRETURN retcode2;
 	SQLINTEGER i;
 	ODBC_FIELDS *current;
-	SQLRETURN retcode;
+	//SQLRETURN retcode;
 	int nresultcols;
-	SQLLEN displaysize;
+	SQLINTEGER displaysize;
 	//int V_OD_erg=0;
 	
 #ifdef ODBC_DEBUG_HEADER
@@ -1119,16 +1100,13 @@ fprintf(stderr,"\tquery_fill result %p,result->odbcStatHandle %p, pos %d\n",res,
 fflush(stderr);
 #endif
 	
-	
 	nresultcols = get_num_columns(res);
 
 	current = res->fields;
 	for (i = 0; i < nresultcols; i++)
 	{
-			
-		if(current->next != NULL)current = (ODBC_FIELDS *) current->next;
-		
-		
+		if(current->next != NULL)
+			current = (ODBC_FIELDS *) current->next;
 	}
 
 
@@ -1142,7 +1120,6 @@ fflush(stderr);
 		{
 			retcode2 = SQLFetchScroll(res->odbcStatHandle, SQL_FETCH_NEXT, pos + 1);
 		}
-
 	}
 	else
 	{
@@ -1176,7 +1153,7 @@ fflush(stderr);
 				
 			SQLDescribeCol(res->odbcStatHandle, i+1 , namebuff, sizeof(namebuff), &colnamelen,&type,&precision, &scale,NULL);
 				
-			SQLColAttribute(res->odbcStatHandle, i+1 , SQL_DESC_LENGTH, "",0, NULL, &displaysize);
+			SQLColAttribute(res->odbcStatHandle, i+1 , SQL_DESC_LENGTH, "",0, NULL, (SQLPOINTER)&displaysize);
 				
 			read=0;
 
@@ -1195,7 +1172,7 @@ if (displaysize>0)
 	if (type != SQL_LONGVARCHAR && type != SQL_VARBINARY && type != SQL_LONGVARBINARY)
 	{
 		fieldata=malloc(sizeof(char)*(displaysize));
-		retcode=SQLGetData(res->odbcStatHandle,i+1,SQL_C_CHAR , fieldata,displaysize,&read); 
+		SQLGetData(res->odbcStatHandle,i+1,SQL_C_CHAR , fieldata,displaysize,&read); 
 	
 	} else
 	{
@@ -1249,27 +1226,23 @@ return FALSE;
   <blob> points at a DB_BLOB structure that will receive a pointer to the
   data and its length.
 
-  NOTE: this function is always called after query_fill() with the same
-  value of <pos>.
-
 *****************************************************************************/
 
 static void blob_read(DB_RESULT result, int pos, int field, DB_BLOB *blob)
 {
-
-
-int i;
-//int outlen;
-//int precision;
-//int scale;
-//int displaysize;
-//char * pointer;
-ODBC_RESULT * res= (ODBC_RESULT *) result;
-ODBC_FIELDS * cfield ;
-SQLLEN strlen;
-SQLRETURN retcode;
-i=0;
-cfield=res->fields;
+	int i;
+	//int outlen;
+	//int precision;
+	//int scale;
+	//int displaysize;
+	//char * pointer;
+	ODBC_RESULT * res= (ODBC_RESULT *) result;
+	ODBC_FIELDS * cfield ;
+	SQLLEN strlen;
+	SQLRETURN retcode;
+	
+	i = 0;
+	cfield = res->fields;
 
 #ifdef ODBC_DEBUG_HEADER
 fprintf(stderr,"[ODBC][%s][%d]\n",__FILE__,__LINE__);
@@ -1277,49 +1250,71 @@ fprintf(stderr,"\tblob_read DB_RESULT %p, dbresult->stathandle %p, pos %d , fiel
 fflush(stderr);
 #endif
 
-	while (i < field ){
-		
-		if (cfield->next== NULL){
+	while (i < field )
+	{
+		if (cfield->next== NULL)
+		{
 			GB.Error("ODBC module :Internal error1"); 
+			return;
 		}
-		cfield=(ODBC_FIELDS *) cfield->next;
-		if (cfield== NULL){
-			GB.Error("ODBC module :Internal error2"); 
-		}
-		i=i+1;
-	}
-	if(i>field) GB.Error("ODBC module : Internal error");
 		
-		blob->data=NULL;
-		if (cfield->outlen > 0) {
-		blob->data=malloc( sizeof(char)*cfield->outlen);
+		cfield=(ODBC_FIELDS *) cfield->next;
+		
+		if (cfield== NULL)
+		{
+			GB.Error("ODBC module :Internal error2"); 
+			return;
+		}
+		
+		i++;
+	}
+	
+	if (i > field)
+	{
+		GB.Error("ODBC module : Internal error");
+		return;
+	}
+		
+	blob->data=NULL;
+	if (cfield->outlen > 0) 
+	{
+		blob->data = malloc( sizeof(char)*cfield->outlen);
 		blob->length = cfield->outlen;
 
 		DB.Query.Init();
 
-		retcode=SQLGetData(res->odbcStatHandle,field+1,SQL_C_BINARY , blob->data,blob->length, &strlen);
-		}else
+		retcode = SQLGetData(res->odbcStatHandle,field+1,SQL_C_BINARY , blob->data,blob->length, &strlen);
+	
+		if (retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO)
 		{
-			blob->data=NULL; // 
-			blob->length=0;
+			GB.Error("Unable to retrieve blob data");
+			free(blob->data);
+			blob->length = 0;
+			blob->data = NULL;
+			return;
 		}
+	}
+	else
+	{
+		blob->data = NULL; // 
+		blob->length = 0;
+		return;
+	}
 		
-
 	char *data;
 	int len;
 
-  	if (!unquote_blob(blob->data, blob->length, DB.Query.AddLength))
-  	{
-  		len = DB.Query.Length();
-  		data = DB.Query.GetNew();
+	if (!unquote_blob(blob->data, blob->length, DB.Query.AddLength))
+	{
+		len = DB.Query.Length();
+		data = DB.Query.GetNew();
 	}
 	else
 		blob->constant = TRUE;
+	
 	free(blob->data);//091107
-	blob->data=data;
-	blob->length=len;
-	
-	
+	blob->data = data;
+	blob->length = len;
 }
 
 
@@ -1434,9 +1429,13 @@ fflush(stderr);
 	retcode=SQLDescribeCol(res->odbcStatHandle, field + 1, colname, sizeof(colname),
 								 &colnamelen, &coltype, &precision, &scale, NULL);
 
-
-	return conv_type(coltype);
-
+	if (retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO)
+	{
+		GB.Error("Unable to retrieve field type");
+		return GB_T_NULL;
+	}
+	else
+		return conv_type(coltype);
 }
 
 
@@ -1569,7 +1568,7 @@ fflush(stderr);
 	ODBC_CONN *han = (ODBC_CONN *)db->handle;
 
 
-	GB.NewString(&info->table, table, 0);
+	info->table = GB.NewZeroString(table);
 
 	retcode =SQLAllocHandle(SQL_HANDLE_STMT, (ODBC_CONN *) han->odbcHandle, &statHandle);
 
@@ -1623,7 +1622,7 @@ fflush(stderr);
 	{
 		fieldstr = current;
 		f = &info->field[i];
-		GB.NewString(&f->name, (char *)current->fieldname, 0);
+		f->name = GB.NewZeroString((char *)current->fieldname);
 
 		f->type = conv_type(current->type);
 
@@ -1862,7 +1861,12 @@ fflush(stderr);
 	SQLLEN nIndicatorRemarks;
 	int compare = -1;
 	ODBC_CONN *han = (ODBC_CONN *)db->handle;
+	int len;
 
+	len = strlen(table);
+	if (len == 0)
+		return FALSE;
+	
 	retcode = SQLAllocHandle(SQL_HANDLE_STMT, han->odbcHandle, &statHandle);
 
 	if ((retcode != SQL_SUCCESS) && (retcode != SQL_SUCCESS_WITH_INFO))
@@ -1870,12 +1874,9 @@ fflush(stderr);
 		return FALSE; //V_OD_erg;
 	}
 
-
-
 	// EXECUTE OUR SQL/CALL
 	if (SQL_SUCCESS != (nReturn = SQLTables(statHandle, 0, 0, 0, 0, 0, 0, 0, 0)))
 	{
-
 		return FALSE; //nReturn;
 	}
 
@@ -1890,9 +1891,8 @@ fflush(stderr);
 	while ((nReturn == SQL_SUCCESS || nReturn == SQL_SUCCESS_WITH_INFO)
 				 && compare != 0)
 	{
-
 		//printf("le tabelle in comparazione %s : %s\n",szTableName,table);
-		compare = strncmp((char *)szTableName, table, sizeof(table));
+		compare = strncmp((char *)szTableName, table, len);
 		szTableName[0] = '\0';
 		szTableType[0] = '\0';
 		szTableRemarks[0] = '\0';
@@ -2003,13 +2003,10 @@ fflush(stderr);
 
 	GB.NewArray(tables, sizeof(char *), tablenum);
 
-
-
 	curtable = &tablelist;
 	for (i = 0; i < tablenum; i++)
 	{
-
-		GB.NewString(&(*tables)[i], curtable->tablename, 0);
+		(*tables)[i] = GB.NewZeroString(curtable->tablename);
 		free(curtable->tablename);
 		curtable = (ODBC_TABLES *) curtable->next;
 
@@ -2086,10 +2083,13 @@ fflush(stderr);
 	}
 
 
-	if (!SQL_SUCCEEDED
-			(nReturn = SQLPrimaryKeys(statHandle, 0, 0, 0, SQL_NTS, (SQLCHAR *)table, SQL_NTS)))
+	/*if (!SQL_SUCCEEDED
+			(nReturn = SQLPrimaryKeys(statHandle, 0, 0, 0, SQL_NTS, (SQLCHAR *)table, SQL_NTS)))*/
+	if (!SQL_SUCCEEDED(nReturn = SQLPrimaryKeys(statHandle, (SQLCHAR *)"", 0, (SQLCHAR *)"", 0, (SQLCHAR *)table, SQL_NTS)))
 	{
 		free(res);
+		fprintf(stderr, "return %d\n", nReturn);
+		GB.Error("Unable to get primary key: &1", table);
 		return TRUE;
 	}
 	// GET RESULTS
@@ -2114,7 +2114,7 @@ fflush(stderr);
 				 (statHandle, 6, SQL_C_CHAR, &szKeyName[0], sizeof(szKeyName), 0)))
 			strcpy((char *) szKeyName, "Unknown");
 
-		GB.NewString((char **) GB.Add(primary), (char *)szColumnName, 0);
+		*(char **)GB.Add(primary) = GB.NewZeroString((char *)szColumnName);
 		i++;
 	}
 
@@ -2466,9 +2466,7 @@ fflush(stderr);
 
 	for (i = 0; i < colsNum; i++)
 	{
-		GB.NewString(&((*fields)[i]), (char *) current->fieldname, 0);
-
-
+		(*fields)[i] = GB.NewZeroString((char *) current->fieldname);
 
 		current = (ODBC_FIELDS *) current->next;
 		free(fieldstr);
@@ -2501,7 +2499,7 @@ fflush(stderr);
 
 *****************************************************************************/
 
-static int field_info(DB_DATABASE *db, char *table, char *field,DB_FIELD * info)
+static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD * info)
 {
 #ifdef ODBC_DEBUG_HEADER
 fprintf(stderr,"[ODBC][%s][%d]\n",__FILE__,__LINE__);
@@ -2516,12 +2514,13 @@ fflush(stderr);
 	SQLHSTMT statHandle1;
 	SQLRETURN retcode;
 	//SQLRETURN V_OD_erg;
-	SQLINTEGER auton=SQL_FALSE;
+	SQLLEN auton=SQL_FALSE;
 	int i;
 	
 	ODBC_CONN *han = (ODBC_CONN *)db->handle;
 	ODBC_CONN *han1 = (ODBC_CONN *)db->handle;
 
+	*precision = 0;
 
 	strncpy((char *)&query[0], "SELECT ",7);
 	strncpy((char *)&query[7], field,strlen(field));
@@ -2555,7 +2554,6 @@ fflush(stderr);
 	}
 	
 	retcode=SQLColAttribute (statHandle1,1,SQL_DESC_AUTO_UNIQUE_VALUE,NULL,0,NULL,&auton);
-     
 
 	SQLFreeHandle(SQL_HANDLE_STMT, statHandle1);
 
@@ -2583,28 +2581,24 @@ fflush(stderr);
 
 	info->name = NULL;
 	info->type = conv_type(atol((char *)coltype));
-	info->length = atol((char *) precision);
+	info->length = 0;
+	if (*precision)
+		info->length = atol((char *) precision);
 
 	if (info->type == GB_T_STRING)
 	{
-
-
-		info->length = atol((char *)precision);
 		if (info->length < 0)
 			info->length = 0;
-
 	}
-	else
-		info->length = atol((char *) precision);
-if (auton==SQL_TRUE) info->type = DB_T_SERIAL;
+	
+	if (auton == SQL_TRUE)
+		info->type = DB_T_SERIAL;
 
 	info->def.type = GB_T_NULL;
 
 	SQLFreeHandle(SQL_HANDLE_STMT, statHandle);
 
-
 	return FALSE;
-
 }
 
 
@@ -2772,7 +2766,7 @@ static int database_list(DB_DATABASE *db, char ***databases)
 
 //GB.Error("ODBC does not implement this function");
 	GB.NewArray(databases, sizeof(char *), 1);
-	GB.NewString(&(*databases)[0], han->dsn_name, 0);
+	(*databases)[0] = GB.NewZeroString(han->dsn_name);
 
 	return (1);
 }
@@ -2905,7 +2899,7 @@ static int user_list(DB_DATABASE *db, char ***users)
 {
 	ODBC_CONN *han = (ODBC_CONN *)db->handle;
 	GB.NewArray(users, sizeof(char *), 1);
-	GB.NewString(&(*users)[0], han->user_name, 0);
+	(*users)[0] = GB.NewZeroString(han->user_name);
 
 	//GB.Error("ODBC does not implement this function");
 	return (1);
