@@ -2,13 +2,12 @@
 
   main.c
 
-  PostgreSQL driver
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2011-2012 Bruce Bruen <bbruen@paddys-hill.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +17,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -131,7 +131,7 @@ static char *get_quote_string(const char *str, int len, char quote)
 			len_res++;
 	}
 	
-	GB.TempString(&res, NULL, len_res);
+	res = GB.TempString(NULL, len_res);
 	
 	p = res;
 	for (i = 0; i < len; i++)
@@ -155,6 +155,62 @@ static char *get_quote_string(const char *str, int len, char quote)
 	return res;
 }
 
+/* Quote a table name so that schema work, and return a temporary string */
+
+static char *get_quoted_table(const char *table)
+{
+	int len;
+	char *point;
+	char *res;
+	
+	if (!table || !*table)
+		return "";
+	
+	len = strlen(table);
+	point = index(table, '.');
+	
+	if (!point)
+	{
+		res = GB.TempString(NULL, len + 2);
+		sprintf(res, "\"%s\"", table);
+	}
+	else
+	{
+		res = GB.TempString(NULL, len + 2);
+		sprintf(res, "%.*s.\"%s\"", (int)(point - table), table, point + 1);
+	}
+	
+	return res;
+}
+
+static bool get_table_schema(const char **table, char **schema)
+{
+	char *point;
+	
+	//fprintf(stderr, "get_table_schema: %s\n", *table);
+	
+	*schema = NULL;
+	
+	if (!*table || !**table)
+	{
+		//fprintf(stderr, "get_table_schema: -> NULL\n");
+		return TRUE;
+	}
+	
+	point = strchr(*table, '.');
+	if (!point)
+	{
+		//fprintf(stderr, "get_table_schema: -> No point\n");
+		*schema = "public";
+		return FALSE;
+	}
+	
+	*schema = GB.TempString(*table, point - *table);
+	*table = point + 1;
+	//fprintf(stderr, "get_table_schema: -> %s / %s\n", *schema, *table);
+	return FALSE;
+}
+
 /* internal function to quote a value stored as a blob */
 
 static void quote_blob(const char *data, int len, DB_FORMAT_CALLBACK add)
@@ -314,7 +370,7 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 		case BOOLOID:
 
 			val->type = GB_T_BOOLEAN;
-			val->value._boolean = conv_boolean(data);
+			val->value._boolean = conv_boolean(data) ? -1 : 0;
 			break;
 
 		case INT2OID:
@@ -323,7 +379,7 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 			GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);
 
 			val->type = GB_T_INTEGER;
-			val->value._integer = ((GB_INTEGER *)&conv)->value;
+			val->value._integer = conv._integer.value;
 
 			break;
 
@@ -332,7 +388,7 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 			GB.NumberFromString(GB_NB_READ_LONG, data, strlen(data), &conv);
 
 			val->type = GB_T_LONG;
-			val->value._long = ((GB_LONG *)&conv)->value;
+			val->value._long = conv._long.value;
 
 			break;
 
@@ -343,7 +399,7 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 			GB.NumberFromString(GB_NB_READ_FLOAT, data, strlen(data), &conv);
 
 			val->type = GB_T_FLOAT;
-			val->value._float = ((GB_FLOAT *)&conv)->value;
+			val->value._float = conv._float.value;
 
 			break;
 
@@ -402,8 +458,8 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 			GB.MakeDate(&date, (GB_DATE *)&conv);
 
 			val->type = GB_T_DATE;
-			val->value._date.date = ((GB_DATE *)&conv)->value.date;
-			val->value._date.time = ((GB_DATE *)&conv)->value.time;
+			val->value._date.date = conv._date.value.date;
+			val->value._date.time = conv._date.value.time;
 
 			break;
 
@@ -516,6 +572,77 @@ static int db_version(DB_DATABASE *db)
 	return dbversion;
 }
 
+/* Internal function that fills field information from a schema request.
+
+   The result columns must be ordered that way (from the specified column index):
+     - atttypid::int
+     - atttypmod
+     - attnotnull
+     - adsrc
+     - atthasdef
+*/
+
+static void fill_field_info(DB_FIELD *info, PGresult *res, int row, int col)
+{
+	char *val;
+	Oid type;
+	GB_VARIANT def;
+	
+	info->name = NULL;
+
+	type = atoi(PQgetvalue(res, row, col));
+	info->type = conv_type(type);
+
+	info->length = 0;
+	if (info->type == GB_T_STRING)
+	{
+		info->length = atoi(PQgetvalue(res, row, col + 1));
+		if (info->length < 0)
+			info->length = 0;
+		else
+			info->length -= 4;
+	}
+
+	info->def.type = GB_T_NULL;
+
+	if (conv_boolean(PQgetvalue(res, row, col + 4)) && conv_boolean(PQgetvalue(res, row, col + 2)))
+	{
+		def.type = GB_T_VARIANT;
+		def.value.type = GB_T_NULL;
+
+		val = PQgetvalue(res, row, col + 3);
+		if (val && *val)
+		{
+			if (strncmp(val, "nextval(", 8) == 0)
+			{
+				if (info->type == GB_T_LONG)
+					info->type = DB_T_SERIAL;
+			}
+			else
+			{
+				switch(info->type)
+				{
+					case GB_T_BOOLEAN:
+						def.value.type = GB_T_BOOLEAN;
+						def.value.value._boolean = (val[1] == 't');
+						break;
+
+					default:
+
+						DB.Query.Init();
+						if (!unquote_string(val, PQgetlength(res, row, col + 3), DB.Query.AddLength))
+							val = DB.Query.Get();
+
+						conv_data(val, -1, &def.value, type);
+				}
+
+				GB.StoreVariant(&def, &info->def);
+			}
+		}
+	}	
+}
+
+
 
 /*****************************************************************************
 
@@ -554,13 +681,22 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 	PGresult *res;
 	int status;
 	char *name;
+	char dbname[512];
 
 	if (desc->name)
 		name = desc->name;
 	else
 		name = "template1";
 
-	conn = PQsetdbLogin(desc->host, desc->port, NULL, NULL, name, desc->user, desc->password);
+	if (snprintf(dbname, sizeof(dbname), "dbname='%s' connect_timeout=%d", get_quote_string(name, strlen(name), '\''), db->timeout) >= sizeof(dbname))
+	{
+		GB.Error("Cannot open database: database name too long");
+		return TRUE;
+	}
+	
+	//fprintf(stderr, "gb.db.postgresql: host = `%s` port = `%s` dbnname = `%s` user = `%s` password = `%s`\n", desc->host, desc->port, dbname, desc->user, desc->password);
+	
+	conn = PQsetdbLogin(desc->host, desc->port, NULL, NULL, dbname, desc->user, desc->password);
 
 	if (!conn)
 	{
@@ -595,6 +731,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
 	db->flags.no_table_type = TRUE;
 	db->flags.no_nest = TRUE;
+	db->flags.schema = TRUE;
 
 	/* encoding */
 
@@ -603,7 +740,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
 	if (!do_query(db, NULL, &res, query, 0))
 	{
-		GB.NewString(&db->charset, PQgetvalue(res, 0, 0), 0);
+		db->charset = GB.NewZeroString(PQgetvalue(res, 0, 0));
 		PQclear(res);
 	}
 	else
@@ -837,9 +974,6 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 	<blob> points at a DB_BLOB structure that will receive a pointer to the
 	data and its length.
 
-	NOTE: this function is always called after query_fill() with the same
-	value of <pos>.
-
 *****************************************************************************/
 
 static void blob_read(DB_RESULT result, int pos, int field, DB_BLOB *blob)
@@ -900,28 +1034,30 @@ static int field_index(DB_RESULT Result, const char *name, DB_DATABASE *db)
 
 	char *fld;
 	int index;
-	char *table;
+	char *table = NULL;
 	int numfields, oid;
 	char *qfield =
 		"select oid from pg_class where relname = '&1' "
-		"and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
+		"and ((relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')))";
 
-	fld = strchr(name, (int)FLD_SEP);
+	fld = strrchr(name, (int)FLD_SEP);
 
-	if (fld){
+	if (fld)
+	{
 		//Includes table identity.
 		//This feature is only available for 7.4 onwards
 		/* check version */
 
-		if (db->version > 70399){ // version 7.4.?
-				fld[0] = '.';
-				GB.NewString(&table, name, fld - name);
-				fld = fld + 1;
+		if (db->version > 70399)
+		{ // version 7.4.?
+			fld[0] = '.';
+			table = GB.NewString(name, fld - name);
+			fld = fld + 1;
 
-				/* Need to find the OID for the table */
-				PGresult *oidres;
+			/* Need to find the OID for the table */
+			PGresult *oidres;
 
-				if (do_query(db, "Unable to get OID for table &1", &oidres, qfield, 1, table)){
+			if (do_query(db, "Unable to get OID for table &1", &oidres, qfield, 1, table)){
 			GB.FreeString(&table);
 			return -1;
 				}
@@ -1101,26 +1237,45 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 
 static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 {
-	char *qfield =
-		"select pg_attribute.attname, pg_attribute.atttypid::int,pg_attribute.atttypmod "
-			"from pg_class, pg_attribute "
-			"where pg_class.relname = '&1' "
-				"and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
-				"and pg_attribute.attnum > 0 "
-				"and pg_attribute.attrelid = pg_class.oid ";
-
+	const char *qfield_all=
+			"SELECT col.attname, col.atttypid::int, col.atttypmod, "
+					"col.attnotnull, def.adsrc, col.atthasdef "
+			"FROM pg_catalog.pg_class tbl, pg_catalog.pg_attribute col "
+							"LEFT JOIN pg_catalog.pg_attrdef def ON (def.adnum = col.attnum AND def.adrelid = col.attrelid) "
+			"WHERE tbl.relname = '&1' AND "
+					"col.attrelid = tbl.oid AND "
+					"col.attnum > 0 AND "
+					"not col.attisdropped "
+			"ORDER BY col.attnum ASC;";
+        
+	char *qfield_schema_all =
+		"select pg_attribute.attname, pg_attribute.atttypid::int, pg_attribute.atttypmod, "
+						"pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef "
+				"from pg_class, pg_attribute "
+						"LEFT JOIN pg_catalog.pg_attrdef  ON (pg_attrdef.adnum = pg_attribute.attnum AND pg_attrdef.adrelid = pg_attribute.attrelid) "
+				"where pg_class.relname = '&1' "
+						"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+						"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
+						"and pg_attribute.attrelid = pg_class.oid ";
+                
 	PGresult *res;
 	int i, n;
 	DB_FIELD *f;
+	char *schema;
 
-	/* Nom de la table */
-
-	GB.NewString(&info->table, table, 0);
-
-	/* Liste des champs */
-
-	if (do_query(db, "Unable to get table fields: &1", &res, qfield, 1, table))
-		return TRUE;
+	// Table name
+	info->table = GB.NewZeroString(table);
+	
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db,"Unable to get table fields: &1", &res, qfield_all, 1, table))
+			return TRUE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to get table fields: &1", &res, qfield_schema_all, 2, table, schema))
+			return TRUE;
+	}
 
 	info->nfield = n = PQntuples(res);
 	if (n == 0)
@@ -1134,25 +1289,10 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 	for (i = 0; i < n; i++)
 	{
 		f = &info->field[i];
-
-		if (field_info(db, table, PQgetvalue(res, i, 0), f))
-		{
-			PQclear(res);
-			return TRUE;
-		}
-
-		GB.NewString(&f->name, PQgetvalue(res, i, 0), 0);
-
-		/*f->type = conv_type(atol(PQgetvalue(res, i, 1)));
-		f->length = 0;
-		if (f->type == GB_T_STRING)
-		{
-			f->length = atoi(PQgetvalue(res, i, 2));
-			if (f->length < 0)
-				f->length = 0;
-			else
-				f->length -= 4;
-		}*/
+		
+		fill_field_info(f, res, i, 1);
+		
+		f->name = GB.NewZeroString(PQgetvalue(res, i, 0));
 	}
 
 	PQclear(res);
@@ -1185,9 +1325,12 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 static int table_index(DB_DATABASE *db, const char *table, DB_INFO *info)
 {
 	const char *qindex;
+	const char *qindex_schema;
 
 	PGresult *res;
 	int i, j, n;
+	const char *fulltable = table;
+	char *schema;
 
 	/* Index primaire */
 
@@ -1201,28 +1344,53 @@ static int table_index(DB_DATABASE *db, const char *table, DB_INFO *info)
 					"and pg_att1.attrelid = pg_ind.indrelid "
 					"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
 				"order by pg_att2.attnum";
-	}
-	else
-	{
-		qindex = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
-		"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind, pg_class pg_table "
-			"where pg_table.relname = '&1' AND pg_table.oid = pg_att1.attrelid AND pg_cl.oid = pg_ind.indexrelid "
-					"and (pg_cl.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
-					"and pg_ind.indisprimary "
+		qindex_schema = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
+			"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
+				"where pg_cl.relname = '&1_pkey' AND pg_cl.oid = pg_ind.indexrelid "
+					"and (pg_cl.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
 					"and pg_att2.attrelid = pg_ind.indexrelid "
 					"and pg_att1.attrelid = pg_ind.indrelid "
 					"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
 				"order by pg_att2.attnum";
 	}
+	else
+	{
+		qindex = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
+			"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind, pg_class pg_table "
+				"where pg_table.relname = '&1' AND pg_table.oid = pg_att1.attrelid AND pg_cl.oid = pg_ind.indexrelid "
+						"and (pg_cl.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
+						"and pg_ind.indisprimary "
+						"and pg_att2.attrelid = pg_ind.indexrelid "
+						"and pg_att1.attrelid = pg_ind.indrelid "
+						"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
+					"order by pg_att2.attnum";
+		qindex_schema = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
+			"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind, pg_class pg_table "
+				"where pg_table.relname = '&1' AND pg_table.oid = pg_att1.attrelid AND pg_cl.oid = pg_ind.indexrelid "
+						"and (pg_cl.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+						"and pg_ind.indisprimary "
+						"and pg_att2.attrelid = pg_ind.indexrelid "
+						"and pg_att1.attrelid = pg_ind.indrelid "
+						"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
+					"order by pg_att2.attnum";
+	}
 
-	if (do_query(db, "Unable to get primary index: &1", &res, qindex, 1, table))
-		return TRUE;
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to get primary index: &1", &res, qindex, 1, table))
+			return TRUE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to get primary index: &1", &res, qindex_schema, 2, table, schema))
+			return TRUE;
+	}
 
 	n = info->nindex = PQntuples(res);
 
 	if (n <= 0)
 	{
-		GB.Error("Table '&1' has no primary index", table);
+		GB.Error("Table '&1' has no primary index", fulltable);
 		PQclear(res);
 		return TRUE;
 	}
@@ -1280,37 +1448,36 @@ static void table_release(DB_DATABASE *db, DB_INFO *info)
 
 static int table_exist(DB_DATABASE *db, const char *table)
 {
-	const char *query73 =
+	const char *query =
 		"select relname from pg_class where (relkind = 'r' or relkind = 'v') "
 		"and (relname = '&1') "
 		"and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
-	/*const char *query74_1 =
-		"select tablename from pg_tables where schemaname <> 'information_schema' and tablename = lower('&1')";
+	const char *query_schema =
+		"select relname from pg_class where (relkind = 'r' or relkind = 'v') "
+		"and (relname = '&1') "
+		"and (relnamespace in (select oid from pg_namespace where nspname = '&2'))";
 
-	const char *query74_2 =
-		"select viewname from pg_views where schemaname <> 'information_schema' and viewname = lower('&1')";*/
+		/*"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind = 'r' or pg_class.relkind = 'v') "
+		"and (pg_namespace.oid = pg_class.relnamespace) "
+		"and (pg_class.relname = '&1') "
+		"and (pg_namespace.nspname = '&2') "
+		"and (pg_namespace.oid not in (select oid from pg_namespace where nspname = 'information_schema'))";*/
 
 	PGresult *res;
 	int exist;
-
-	//if (1) //version < 70400)
-	//{
-		if (do_query(db, "Unable to check table: &1", &res, query73, 1, table))
+	char *schema;
+	
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to check table: &1", &res, query, 1, table))
 			return FALSE;
-	/*}
+	}
 	else
 	{
-		if (do_query(conn, "Unable to check table: &1", &res, query74_1, 1, table))
-			return TRUE;
-
-		if (PQntuples(res) == 0)
-		{
-			PQclear(res);
-			if (do_query(conn, "Unable to check table: &1", &res, query74_2, 1, table))
-				return TRUE;
-		}
-	}*/
+		if (do_query(db, "Unable to check table: &1", &res, query_schema, 2, table, schema))
+			return FALSE;
+	}
 
 	exist = PQntuples(res) == 1;
 
@@ -1340,12 +1507,14 @@ static int table_exist(DB_DATABASE *db, const char *table)
 static int table_list_73(DB_DATABASE *db, char ***tables)
 {
 	const char *query =
-		"select relname from pg_class where (relkind = 'r' or relkind = 'v') "
-		"and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
+		"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind = 'r' or pg_class.relkind = 'v') "
+		"and (pg_namespace.oid = pg_class.relnamespace) "
+		"and (pg_namespace.oid not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
 	PGresult *res;
 	int i;
 	int count;
+	char *schema;
 
 	if (do_query(db, "Unable to get tables: &1", &res, query, 0))
 		return -1;
@@ -1355,7 +1524,17 @@ static int table_list_73(DB_DATABASE *db, char ***tables)
 		GB.NewArray(tables, sizeof(char *), PQntuples(res));
 
 		for (i = 0; i < PQntuples(res); i++)
-			GB.NewString(&((*tables)[i]), PQgetvalue(res, i, 0), 0);
+		{
+			schema = PQgetvalue(res, i, 1);
+			if (!strcmp(schema, "public")) // || !strcmp(schema, "pg_catalog"))
+				(*tables)[i] = GB.NewZeroString(PQgetvalue(res, i, 0));
+			else
+			{
+				(*tables)[i] = GB.NewZeroString(schema);
+				(*tables)[i] = GB.AddChar((*tables)[i], '.');
+				(*tables)[i] = GB.AddString((*tables)[i], PQgetvalue(res, i, 0), 0);
+			}
+		}
 	}
 
 	count = PQntuples(res);
@@ -1435,8 +1614,10 @@ static int table_list(DB_DATABASE *db, char ***tables)
 static int table_primary_key(DB_DATABASE *db, const char *table, char ***primary)
 {
 	const char *query;
+	const char *query_schema;
 	PGresult *res;
 	int i;
+	char *schema;
 
 	if (db->version < 80200)
 	{
@@ -1448,6 +1629,14 @@ static int table_primary_key(DB_DATABASE *db, const char *table, char ***primary
 					"and pg_att1.attrelid = pg_ind.indrelid "
 					"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
 				"order by pg_att2.attnum";
+		query_schema = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
+			"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
+				"where pg_cl.relname = '&1_pkey' AND pg_cl.oid = pg_ind.indexrelid "
+					"and (pg_cl.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+					"and pg_att2.attrelid = pg_ind.indexrelid "
+					"and pg_att1.attrelid = pg_ind.indrelid "
+					"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
+				"order by pg_att2.attnum";
 	}
 	else
 	{
@@ -1460,15 +1649,32 @@ static int table_primary_key(DB_DATABASE *db, const char *table, char ***primary
 					"and pg_att1.attrelid = pg_ind.indrelid "
 					"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
 				"order by pg_att2.attnum";
+		query_schema = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
+			"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind, pg_class pg_table "
+				"where pg_table.relname = '&1' AND pg_table.oid = pg_att1.attrelid AND pg_cl.oid = pg_ind.indexrelid "
+					"and (pg_cl.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+					"and pg_ind.indisprimary "
+					"and pg_att2.attrelid = pg_ind.indexrelid "
+					"and pg_att1.attrelid = pg_ind.indrelid "
+					"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
+				"order by pg_att2.attnum";
 	}
 
-	if (do_query(db, "Unable to get primary key: &1", &res, query, 1, table))
-		return TRUE;
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to get primary key: &1", &res, query, 1, table))
+			return TRUE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to get primary key: &1", &res, query_schema, 2, table, schema))
+			return TRUE;
+	}
 
 	GB.NewArray(primary, sizeof(char *), PQntuples(res));
 
 	for (i = 0; i < PQntuples(res); i++)
-		GB.NewString(&((*primary)[i]), PQgetvalue(res, i, 0), 0);
+		(*primary)[i] = GB.NewZeroString(PQgetvalue(res, i, 0));
 
 	PQclear(res);
 
@@ -1495,44 +1701,42 @@ static int table_is_system(DB_DATABASE *db, const char *table)
 	const char *query =
 		"select 1 from pg_class where (relkind = 'r' or relkind = 'v') "
 		"and (relname = '&1') "
-		"and (relnamespace not in (select oid from pg_namespace where nspname <> 'public'))";
+		"and (relnamespace in (select oid from pg_namespace where nspname = 'pg_catalog'))";
 
 	//const char *query1 =
 	//  "select 1 from pg_tables where tablename = lower('&1') and schemaname = 'public'";
 
 	const char *query2 =
-		"select 1 from pg_views where viewname = '&1' and schemaname = 'public'";
+		"select 1 from pg_views where viewname = '&1' and schemaname = 'pg_catalog'";
 
 	PGresult *res;
 	int exist;
+	char *schema;
 
-	/* check version */
-	/*if (version < 70400){ // version 7.3.?
-		if (table[0] == 'p' && table[1] == 'g' && table[2] == '_')
-				return TRUE;
-		else
-				return FALSE;
-	}*/
+	get_table_schema(&table, &schema);
+	
+	if (schema)
+		return !strcmp(schema, "pg_catalog");
 
 	if (do_query(db, "Unable to check table: &1", &res, query, 1, table))
-		return FALSE;
+		return TRUE;
 
 	exist = PQntuples(res) == 1;
 	PQclear(res);
 
 	if (exist)
-		return FALSE;
+		return TRUE;
 
 	if (do_query(db, "Unable to check table: &1", &res, query2, 1, table))
-		return FALSE;
+		return TRUE;
 
 	exist = PQntuples(res) == 1;
 	PQclear(res);
 
 	if (exist)
-		return FALSE;
+		return TRUE;
 
-	return TRUE;
+	return FALSE;
 }
 
 
@@ -1554,7 +1758,7 @@ static int table_delete(DB_DATABASE *db, const char *table)
 {
 	return
 		do_query(db, "Unable to delete table: &1", NULL,
-			"drop table \"&1\"", 1, table);
+			"drop table &1", 1, get_quoted_table(table));
 }
 
 
@@ -1583,9 +1787,9 @@ static int table_create(DB_DATABASE *db, const char *table, DB_FIELD *fields, ch
 
 	DB.Query.Init();
 
-	DB.Query.Add("CREATE TABLE \"");
-	DB.Query.Add(table);
-	DB.Query.Add("\" ( ");
+	DB.Query.Add("CREATE TABLE ");
+	DB.Query.Add(get_quoted_table(table));
+	DB.Query.Add(" ( ");
 
 	comma = FALSE;
 	for (fp = fields; fp; fp = fp->next)
@@ -1687,14 +1891,30 @@ static int field_exist(DB_DATABASE *db, const char *table, const char *field)
 		"where pg_class.relname = '&1' "
 		"and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
 		"and pg_attribute.attname = '&2' "
-		"and pg_attribute.attnum > 0 "
+		"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
+		"and pg_attribute.attrelid = pg_class.oid ";
+
+	const char *query_schema = "select pg_attribute.attname from pg_class, pg_attribute "
+		"where pg_class.relname = '&1' "
+		"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
+		"and pg_attribute.attname = '&2' "
+		"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 		"and pg_attribute.attrelid = pg_class.oid ";
 
 	PGresult *res;
 	int exist;
+	char *schema;
 
-	if (do_query(db, "Unable to check field: &1", &res, query, 2, table, field))
-		return FALSE;
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to check field: &1", &res, query, 2, table, field))
+			return FALSE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to check field: &1", &res, query_schema, 3, table, field, schema))
+			return FALSE;
+	}
 
 	exist = PQntuples(res) == 1;
 
@@ -1726,22 +1946,37 @@ static int field_list(DB_DATABASE *db, const char *table, char ***fields)
 	const char *query = "select pg_attribute.attname from pg_class, pg_attribute "
 		"where pg_class.relname = '&1' "
 		"and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
-		"and pg_attribute.attnum > 0 "
+		"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
+		"and pg_attribute.attrelid = pg_class.oid";
+
+	const char *query_schema = "select pg_attribute.attname from pg_class, pg_attribute "
+		"where pg_class.relname = '&1' "
+		"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+		"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 		"and pg_attribute.attrelid = pg_class.oid";
 
 	PGresult *res;
 	int i;
 	int count;
+	char *schema;
 
-	if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
-		return -1;
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
+			return -1;
+	}
+	else
+	{
+		if (do_query(db, "Unable to get fields: &1", &res, query_schema, 2, table, schema))
+			return -1;
+	}
 
 	if (fields)
 	{
 		GB.NewArray(fields, sizeof(char *), PQntuples(res));
 
 		for (i = 0; i < PQntuples(res); i++)
-			GB.NewString(&((*fields)[i]), PQgetvalue(res, i, 0), 0);
+			(*fields)[i] = GB.NewZeroString(PQgetvalue(res, i, 0));
 	}
 
 	count = PQntuples(res);
@@ -1771,82 +2006,48 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 {
 	const char *query =
 		"select pg_attribute.attname, pg_attribute.atttypid::int, "
-		"pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc "
+		"pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef "
 		"from pg_class, pg_attribute "
 		"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
 		"where pg_class.relname = '&1' "
 		"and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
 		"and pg_attribute.attname = '&2' "
-		"and pg_attribute.attnum > 0 "
+		"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
+		"and pg_attribute.attrelid = pg_class.oid";
+
+	const char *query_schema =
+		"select pg_attribute.attname, pg_attribute.atttypid::int, "
+		"pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc, pg_attribute.atthasdef "
+		"from pg_class, pg_attribute "
+		"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
+		"where pg_class.relname = '&1' "
+		"and (pg_class.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
+		"and pg_attribute.attname = '&2' "
+		"and pg_attribute.attnum > 0 and not pg_attribute.attisdropped "
 		"and pg_attribute.attrelid = pg_class.oid";
 
 	PGresult *res;
-	Oid type;
-	GB_VARIANT def;
-	char *val;
-
-	if (do_query(db, "Unable to get field info: &1", &res, query, 2, table, field))
-		return TRUE;
+	char *schema;
+	const char *fulltable = table;
+	
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to get field info: &1", &res, query, 2, table, field))
+			return TRUE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to get field info: &1", &res, query_schema, 3, table, field, schema))
+			return TRUE;
+	}
 
 	if (PQntuples(res) != 1)
 	{
-		GB.Error("Unable to find field &1.&2", table, field);
+		GB.Error("Unable to find field &1.&2", fulltable, field);
 		return TRUE;
 	}
 
-	info->name = NULL;
-
-	type = atoi(PQgetvalue(res, 0, 1));
-	info->type = conv_type(type);
-
-	info->length = 0;
-	if (info->type == GB_T_STRING)
-	{
-		info->length = atoi(PQgetvalue(res, 0, 2));
-		if (info->length < 0)
-			info->length = 0;
-		else
-			info->length -= 4;
-	}
-
-	info->def.type = GB_T_NULL;
-
-	if (conv_boolean(PQgetvalue(res, 0, 3)))
-	{
-		def.type = GB_T_VARIANT;
-		def.value.type = GB_T_NULL;
-
-		val = PQgetvalue(res, 0, 4);
-		if (val && *val)
-		{
-			if (strncmp(val, "nextval(", 8) == 0)
-			{
-				if (info->type == GB_T_LONG)
-					info->type = DB_T_SERIAL;
-			}
-			else
-			{
-				switch(info->type)
-				{
-					case GB_T_BOOLEAN:
-						def.value.type = GB_T_BOOLEAN;
-						def.value.value._boolean = (val[1] == 't');
-						break;
-
-					default:
-
-						DB.Query.Init();
-						if (!unquote_string(val, PQgetlength(res, 0, 4), DB.Query.AddLength))
-							val = DB.Query.Get();
-
-						conv_data(val, -1, &def.value, type);
-				}
-
-				GB.StoreVariant(&def, &info->def);
-			}
-
-		}
-	}
+	fill_field_info(info, res, 0, 1);
 
 	PQclear(res);
 	return FALSE;
@@ -1877,15 +2078,32 @@ static int index_exist(DB_DATABASE *db, const char *table, const char *index)
 		"and pg_index.indexrelid = pg_class.oid "
 		"and pg_class.relname = '&2'";
 
+	const char *query_schema =
+		"select pg_class.relname from pg_class, pg_index, pg_class pg_class2 "
+		"where pg_class2.relname = '&1' "
+		"and (pg_class2.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
+		"and pg_index.indrelid = pg_class2.oid "
+		"and pg_index.indexrelid = pg_class.oid "
+		"and pg_class.relname = '&2'";
+
 	/*const char *query = "select relname from pg_class, pg_index "
 		"where pg_class.relname = '&1' "
 		"and pg_index.indexrelid = pg_class.oid ";*/
 
 	PGresult *res;
 	int exist;
-
-	if (do_query(db, "Unable to check index: &1", &res, query, 2, table, index))
-		return FALSE;
+	char *schema;
+	
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to check index: &1", &res, query, 2, table, index))
+			return FALSE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to check index: &1", &res, query_schema, 3, table, index, schema))
+			return FALSE;
+	}
 
 	exist = PQntuples(res) == 1;
 
@@ -1920,19 +2138,34 @@ static int index_list(DB_DATABASE *db, const char *table, char ***indexes)
 		"and pg_index.indrelid = pg_class2.oid "
 		"and pg_index.indexrelid = pg_class.oid ";
 
+	const char *query_schema = "select pg_class.relname from pg_class, pg_index, pg_class pg_class2 "
+		"where pg_class2.relname = '&1' "
+		"and (pg_class2.relnamespace in (select oid from pg_namespace where nspname = '&2')) "
+		"and pg_index.indrelid = pg_class2.oid "
+		"and pg_index.indexrelid = pg_class.oid ";
+
 	PGresult *res;
 	int i;
 	int count;
-
-	if (do_query(db, "Unable to get indexes: &1", &res, query, 1, table))
-		return TRUE;
+	char *schema;
+	
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to get indexes: &1", &res, query, 1, table))
+			return TRUE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to get indexes: &1", &res, query_schema, 2, table, schema))
+			return TRUE;
+	}
 
 	if (indexes)
 	{
 		GB.NewArray(indexes, sizeof(char *), PQntuples(res));
 
 		for (i = 0; i < PQntuples(res); i++)
-			GB.NewString(&((*indexes)[i]), PQgetvalue(res, i, 0), 0);
+			(*indexes)[i] = GB.NewZeroString(PQgetvalue(res, i, 0));
 	}
 
 	count = PQntuples(res);
@@ -1968,6 +2201,14 @@ static int index_info(DB_DATABASE *db, const char *table, const char *index, DB_
 		"and pg_index.indexrelid = pg_class.oid "
 		"and pg_class.relname = '&2'";
 
+	const char *query_schema =
+		"select indisunique, indisprimary, indexrelid from pg_class, pg_index, pg_class pg_class2 "
+		"where pg_class2.relname = '&1' "
+		"and (pg_class2.relnamespace in (select oid from pg_namespace where nspname = '&3')) "
+		"and pg_index.indrelid = pg_class2.oid "
+		"and pg_index.indexrelid = pg_class.oid "
+		"and pg_class.relname = '&2'";
+
 	const char *query_field =
 		"select pg_att1.attname "
 			"from pg_attribute pg_att1, pg_attribute pg_att2, pg_index pg_ind "
@@ -1980,13 +2221,23 @@ static int index_info(DB_DATABASE *db, const char *table, const char *index, DB_
 	PGresult *res;
 	char indexrelid[16];
 	int i;
+	char *schema;
+	const char *fulltable = table;
 
-	if (do_query(db, "Unable to get index info: &1", &res, query, 2, table, index))
-		return TRUE;
+	if (get_table_schema(&table, &schema))
+	{
+		if (do_query(db, "Unable to get index info: &1", &res, query, 2, table, index))
+			return TRUE;
+	}
+	else
+	{
+		if (do_query(db, "Unable to get index info: &1", &res, query_schema, 3, table, index, schema))
+			return TRUE;
+	}
 
 	if (PQntuples(res) != 1)
 	{
-		GB.Error("Unable to find index &1.&2", table, index);
+		GB.Error("Unable to find index &1.&2", fulltable, index);
 		return TRUE;
 	}
 
@@ -2037,7 +2288,7 @@ static int index_delete(DB_DATABASE *db, const char *table, const char *index)
 {
 	return
 		do_query(db, "Unable to delete index: &1", NULL,
-			"drop index \"&1\"", 1, index);
+			"drop index &1", 1, get_quoted_table(index));
 }
 
 
@@ -2067,7 +2318,7 @@ static int index_create(DB_DATABASE *db, const char *table, const char *index, D
 	DB.Query.Add("INDEX \"");
 	DB.Query.Add(index);
 	DB.Query.Add("\" ON ");
-	DB.Query.Add(table);
+	DB.Query.Add(get_quoted_table(table));
 	DB.Query.Add(" ( ");
 	DB.Query.Add(info->fields);
 	DB.Query.Add(" )");
@@ -2143,7 +2394,7 @@ static int database_list(DB_DATABASE *db, char ***databases)
 		GB.NewArray(databases, sizeof(char *), PQntuples(res));
 
 		for (i = 0; i < PQntuples(res); i++)
-			GB.NewString(&((*databases)[i]), PQgetvalue(res, i, 0), 0);
+			(*databases)[i] = GB.NewZeroString(PQgetvalue(res, i, 0));
 	}
 
 	count = PQntuples(res);
@@ -2320,7 +2571,7 @@ static int user_list(DB_DATABASE *db, char ***users)
 		GB.NewArray(users, sizeof(char *), PQntuples(res));
 
 		for (i = 0; i < PQntuples(res); i++)
-			GB.NewString(&((*users)[i]), PQgetvalue(res, i, 0), 0);
+			(*users)[i] = GB.NewZeroString(PQgetvalue(res, i, 0));
 	}
 
 	count = PQntuples(res);
@@ -2375,7 +2626,7 @@ static int user_info(DB_DATABASE *db, const char *name, DB_USER *info)
 	if (!do_query(db, NULL, &res, query_passwd, 1, name))
 	{
 		if (*PQgetvalue(res, 0, 0))
-			GB.NewString(&info->password, "***", 0);
+			info->password = GB.NewString("***", 3);
 	}
 
 	return FALSE;
@@ -2494,7 +2745,7 @@ int EXPORT GB_INIT(void)
 	GB.GetInterface("gb.db", DB_INTERFACE_VERSION, &DB);
 	DB.Register(&_driver);
 
-	return FALSE;
+	return 0;
 }
 
 void EXPORT GB_EXIT()
