@@ -1,15 +1,12 @@
 /***************************************************************************
 
-  main.c
+  main.cpp
 
-  SQLite driver
-
-  iHacked by N.Gerrard from code originally provided by
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -36,6 +34,7 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "sqlitedataset.h"
 
@@ -110,15 +109,10 @@ static void conv_data(const char *data, GB_VARIANT_VALUE *val, fType type)
    case ft_Boolean:
 
       val->type = GB_T_BOOLEAN;
-      /*GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);*/
       if (data[0] == 't' || data[0] == 'T')
-			{
-				val->value._boolean = 1;
-      }
-      else 
-			{
-				val->value._boolean = atoi(data);// != 0;
-      }
+				val->value._boolean = -1;
+      else
+				val->value._boolean = atoi(data) ? -1 : 0;
       break;
 
     case ft_Short:
@@ -129,7 +123,7 @@ static void conv_data(const char *data, GB_VARIANT_VALUE *val, fType type)
       GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);
 
       val->type = GB_T_INTEGER;
-      val->value._integer = ((GB_INTEGER *)&conv)->value;
+      val->value._integer = conv._integer.value;
 
       break;
 
@@ -138,7 +132,7 @@ static void conv_data(const char *data, GB_VARIANT_VALUE *val, fType type)
       GB.NumberFromString(GB_NB_READ_LONG, data, strlen(data), &conv);
 
       val->type = GB_T_LONG;
-      val->value._long = ((GB_LONG *)&conv)->value;
+      val->value._long = conv._long.value;
 
       break;
 
@@ -148,7 +142,7 @@ static void conv_data(const char *data, GB_VARIANT_VALUE *val, fType type)
       GB.NumberFromString(GB_NB_READ_FLOAT, data, strlen(data), &conv);
 
       val->type = GB_T_FLOAT;
-      val->value._float = ((GB_FLOAT *)&conv)->value;
+      val->value._float = conv._float.value;
 
       break;
 
@@ -221,8 +215,8 @@ static void conv_data(const char *data, GB_VARIANT_VALUE *val, fType type)
 			GB.MakeDate(&date, (GB_DATE *) & conv);
 
 			val->type = GB_T_DATE;
-			val->value._date.date = ((GB_DATE *) & conv)->value.date;
-			val->value._date.time = ((GB_DATE *) & conv)->value.time;
+			val->value._date.date = conv._date.value.date;
+			val->value._date.time = conv._date.value.time;
 
 			break;
 
@@ -231,9 +225,9 @@ static void conv_data(const char *data, GB_VARIANT_VALUE *val, fType type)
     case ft_Char:
     case ft_WChar:
     default:
+			
       val->type = GB_T_CSTRING;
       val->value._string = (char *)data;
-      /*GB.NewString(&val->_string.value, data, strlen(data));*/
 
       break;
   }
@@ -369,15 +363,15 @@ static char *FindDatabase (const char *name, const char *hostName)
   if (*name == '/')
   {
     if (IsDatabaseFile(name))
-      GB.NewString(&fullpath, name, 0);
+      fullpath = GB.NewZeroString(name);
 
     return fullpath;
   }
 
   /* Hostname contains home area */
-  GB.NewString(&fullpath, hostName, 0);
-  GB.AddString(&fullpath, "/", 0);
-  GB.AddString(&fullpath, name, 0);
+  fullpath = GB.NewZeroString(hostName);
+  fullpath = GB.AddChar(fullpath, '/');
+  fullpath = GB.AddString(fullpath, name, 0);
   if (IsDatabaseFile(fullpath))
      return fullpath;
   
@@ -387,9 +381,9 @@ static char *FindDatabase (const char *name, const char *hostName)
   dbhome = getenv("GAMBAS_SQLITE_DBHOME");
   if (dbhome)
   {
-		GB.NewString(&fullpath, dbhome, 0);
-		GB.AddString(&fullpath, "/", 0);
-		GB.AddString(&fullpath, name, 0);
+		fullpath = GB.NewZeroString(dbhome);
+		fullpath = GB.AddChar(fullpath, '/');
+		fullpath = GB.AddString(fullpath, name, 0);
 
 		if (IsDatabaseFile(fullpath))
 				return fullpath;
@@ -403,9 +397,9 @@ static char *FindDatabase (const char *name, const char *hostName)
   }
   #endif
 
-  GB.NewString(&fullpath, GB.GetTempDir(), 0);
-  GB.AddString(&fullpath, "/sqlite/", 0);
-  GB.AddString(&fullpath, name, 0);
+  fullpath = GB.NewZeroString(GB.TempDir());
+  fullpath = GB.AddString(fullpath, "/sqlite/", 0);
+  fullpath = GB.AddString(fullpath, name, 0);
 
   if (IsDatabaseFile(fullpath))
      return fullpath;
@@ -439,7 +433,7 @@ static char *GetDatabaseHome()
      }
      */
 
-     sprintf(dbhome, "%s/sqlite", GB.GetTempDir());
+     sprintf(dbhome, "%s/sqlite", GB.TempDir());
   }
   else {
      strcpy(dbhome, env);
@@ -448,68 +442,50 @@ static char *GetDatabaseHome()
   return dbhome;
 }
 
-// BM: not used anymore
-#if 0
-/* Return Fullpath for database */
-char *FullPath( char *name )
+
+// Internal function to walk a directory and list files
+// Used by database_list
+
+static int WalkDirectory(const char *dir, char ***databases)
 {
-  char *db_fullpath = NULL;
-  char *dbhome = NULL;
+	DIR *dp;
+	struct dirent *entry;
+	struct stat statbuf;
+	char cwd[MAX_PATH];
 
-  dbhome = GetDatabaseHome();
+	if ((dp = opendir(dir)) == NULL)
+		return -1;
 
-  if (!dbhome)
-     return NULL;
+	if (getcwd(cwd, MAX_PATH) == NULL)
+	{
+		fprintf(stderr, "gb.db.sqlite2: warning: getcwd: %s\n", strerror(errno));
+		return -1;
+	}
 
-  GB.Alloc ((void **)&db_fullpath, strlen(name)+strlen(dbhome)+2); /* leave room
-							     for \0 and / */
-  /* start with an empty string */
-  db_fullpath[0] = '\0';
+	if (chdir(dir))
+	{
+		fprintf(stderr, "gb.db.sqlite2: warning: chdir: %s\n", strerror(errno));
+		return -1;
+	}
 
-  strcpy(db_fullpath, dbhome);
-  if (db_fullpath[strlen(db_fullpath)-1] != '/') {
-      strcat(db_fullpath, "/");
-  }
+	while ((entry = readdir(dp)) != NULL) 
+	{
+		stat(entry->d_name, &statbuf);
 
-  strcat(db_fullpath, name);
+		if (S_ISREG(statbuf.st_mode)) 
+		{
+			if (IsDatabaseFile(entry->d_name))
+				*(char **)GB.Add(databases) = GB.NewZeroString(entry->d_name);
+		}
+	}
 
-  GB.Free((void **)&dbhome);
-  return db_fullpath;
-
-}
-#endif
-
-/* Internal function to walk a dirctory and list files */
-/* Used by database_list                               */
-static int WalkDirectory(const char *dir, char ***databases ){
-
-  DIR *dp;
-  struct dirent *entry;
-  struct stat statbuf;
-  char cwd[MAX_PATH];
-
-  if ((dp = opendir(dir)) == NULL) {
-      return -1;
-  }
-
-  getcwd(cwd, MAX_PATH);
-
-  chdir(dir);
-
-  while ((entry = readdir(dp)) != NULL) {
-	 stat(entry->d_name, &statbuf);
-
-	 if (S_ISREG(statbuf.st_mode)) {
-            if (IsDatabaseFile(entry->d_name)){
-                GB.NewString((char **)GB.Add(databases), entry->d_name, 0);
-	    }
-	 }
-  }
-
-  chdir(cwd);
-  // BM: you must call closedir()
-  closedir(dp);
-  return GB.Count(databases);
+	// BM: you must call closedir()
+	closedir(dp);
+	
+	if (chdir(cwd))
+		fprintf(stderr, "gb.db.sqlite2: warning: chdir: %s\n", strerror(errno));
+	
+	return GB.Count(databases);
 }
 
 /* Internal function to check database version number */
@@ -563,12 +539,12 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
   if (desc->name)
   {
-    GB.NewString(&name, desc->name, 0);
+    name = GB.NewZeroString(desc->name);
     memory = false;
   }
   else
   {
-    GB.NewString(&name, ":memory:", 0);
+    name = GB.NewZeroString(":memory:");
     memory = true;
   }
 
@@ -586,6 +562,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
   else
   {
     GB.Error("Unable to locate database: &1", name);
+		GB.FreeString(&name);
     delete conn;
     return TRUE;
   }
@@ -603,7 +580,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
   /* Character set cannot be set for sqlite. A sqlite re-compile
    * is required.                                             */
-  GB.NewString(&db->charset, strcmp(sqlite_encoding, "iso8859") == 0 ? "ISO-8859-1" : "UTF-8", 0);
+  db->charset = GB.NewZeroString(strcmp(sqlite_encoding, "iso8859") == 0 ? "ISO-8859-1" : "UTF-8");
 
   /* set dbversion */
   db->version = db_version();
@@ -825,7 +802,7 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 
   for ( i=0; i < res->fieldCount(); i++)
   {
-       GB.NewString( &data, res->fv(res->fieldName(i)).get_asString().data(),0);
+       GB.NewZeroString( &data, res->fv(res->fieldName(i)).get_asString().data());
 
         value.type = GB_T_VARIANT;
         value.value._object.type = GB_T_NULL;
@@ -857,7 +834,6 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 
 	for (i = 0; i < res->fieldCount(); i++)
 	{
-		//GB.NewString( &data, res->fv(res->fieldName(i)).get_asString().data(),0);
 		if (res->fv(i).get_isNull())
 			data = NULL;
 		else
@@ -1077,7 +1053,7 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 
   /* Nom de la table */
 
-  GB.NewString(&info->table, table, 0);
+  info->table = GB.NewZeroString(table);
 
   /* Liste des champs */
 
@@ -1106,7 +1082,7 @@ static int table_init(DB_DATABASE *db, const char *table, DB_INFO *info)
 			return TRUE;
 		}
 
-    GB.NewString(&f->name, field, 0);
+    f->name = GB.NewZeroString(field);
     /*f->length = 0;
     f->type = conv_type(GetFieldType((char *) r->records[i][2].get_asString().data(), (unsigned int *) &f->length));*/
   }
@@ -1165,7 +1141,7 @@ static int table_index(DB_DATABASE *db, const char *table, DB_INFO *info)
   for (int i = 0; i < n; i++)
   {
     if (strstr(r->records[i][1].get_asString().data(), "autoindex") != NULL){
-       GB.NewString( &sql, r->records[i][1].get_asString().data(), 0);
+       sql = GB.NewZeroString(r->records[i][1].get_asString().data());
        res->close();
 
        if (do_query(db, "Unable to get information on primary index: &1", &res, qindex2, 1, sql)){
@@ -1279,7 +1255,7 @@ static int table_list(DB_DATABASE *db, char ***tables)
   						//added to the list
 
   while ( !res->eof()){
-    GB.NewString(&(*tables)[i], res->fv("tbl_name").get_asString().data(), 0);
+    (*tables)[i] = GB.NewZeroString(res->fv("tbl_name").get_asString().data());
     res->next();
     i++;
   }
@@ -1287,8 +1263,8 @@ static int table_list(DB_DATABASE *db, char ***tables)
 
   res->close();
 
-  GB.NewString(&(*tables)[i], "sqlite_master", 0);
-  GB.NewString(&(*tables)[++i], "sqlite_temp_master", 0);
+  (*tables)[i] = GB.NewZeroString("sqlite_master");
+  (*tables)[++i] = GB.NewZeroString("sqlite_temp_master");
   return rows;
 }
 
@@ -1332,7 +1308,7 @@ static int table_primary_key(DB_DATABASE *db, const char *table, char ***primary
   for (i = 0; i < n; i++)
   {
     if (strstr(r->records[i][1].get_asString().data(), "autoindex")){
-       GB.NewString( &sql, r->records[i][1].get_asString().data(), 0);
+       sql = GB.NewZeroString(r->records[i][1].get_asString().data());
        res->close();
 
        if (do_query(db, "Unable to get primary key: &1", &res, qindex2, 1, sql)){
@@ -1349,10 +1325,9 @@ static int table_primary_key(DB_DATABASE *db, const char *table, char ***primary
 	         return TRUE;
        }
 
-       for (i = 0; i < n; i++){
-           GB.NewString((char **)GB.Add(primary),
-			   r->records[i][2].get_asString().data(),0);
-       }
+       for (i = 0; i < n; i++)
+				*(char **)GB.Add(primary) = GB.NewZeroString(r->records[i][2].get_asString().data());
+			 
        break;
     }
   }
@@ -1603,7 +1578,7 @@ static int field_list(DB_DATABASE *db, const char *table, char ***fields)
     GB.NewArray(fields, sizeof(char *), n);
 
     for (i = 0; i < n; i++){
-      GB.NewString(&(*fields)[i], r->records[i][1].get_asString().data(), 0);
+      (*fields)[i] = GB.NewZeroString(r->records[i][1].get_asString().data());
     }
   }
 
@@ -1767,7 +1742,7 @@ static int index_list(DB_DATABASE *db, const char *table, char ***indexes)
 
   while ( !res->eof()){
                     //(res->fv("tbl_name").get_asString().data());
-    GB.NewString(&(*indexes)[i], res->fv(res->fieldName(0)).get_asString().data(), 0);
+    (*indexes)[i] = GB.NewZeroString(res->fv(res->fieldName(0)).get_asString().data());
     res->next();
     i++;
   }
@@ -2080,7 +2055,7 @@ static int database_create(DB_DATABASE *db, const char *name)
   /* BM: do not use basename(), it can modify its argument */
   if (name && name[0] == '/')
   {
-    GB.NewString(&fullpath, name, 0);
+    fullpath = GB.NewZeroString(name);
     goto _CREATE_DATABASE;
   }
 
@@ -2090,20 +2065,20 @@ static int database_create(DB_DATABASE *db, const char *name)
   host = conn->getHostName();
   if (host && host[0])
   {
-    GB.NewString(&fullpath, host, 0);
+    fullpath = GB.NewZeroString(host);
   }
   else
   {
     dir = GetDatabaseHome();
     mkdir(dir, S_IRWXU);
-    GB.NewString(&fullpath, dir, 0);
+    fullpath = GB.NewZeroString(dir);
     GB.Free(POINTER(&dir));
   }
 
   if (fullpath[strlen(fullpath) - 1] != '/')
-    GB.AddString(&fullpath, "/", 0);
+    fullpath = GB.AddChar(fullpath, '/');
 
-  GB.AddString(&fullpath, name, 0);
+  fullpath = GB.AddString(fullpath, name, 0);
 
 _CREATE_DATABASE:
 
@@ -2298,7 +2273,7 @@ static int user_list(DB_DATABASE *db, char ***users)
 
        while (( user = getpwent()) != NULL ){
              if (users){
-                GB.NewString((char **)GB.Add(users), user->pw_name, 0);
+                *(char **)GB.Add(users) = GB.NewZeroString(user->pw_name);
              }
              else {
                 Count++;
@@ -2320,7 +2295,7 @@ static int user_list(DB_DATABASE *db, char ***users)
           Member = Group->gr_mem;
           while (Member && *Member){
                if (users){
-                  GB.NewString((char **)GB.Add(users), *Member, 0);
+                  *(char **)GB.Add(users) = GB.NewZeroString(*Member);
                }
                else {
                   Count++;
@@ -2335,7 +2310,7 @@ static int user_list(DB_DATABASE *db, char ***users)
   if (( buf.st_mode & S_IRUSR ) || (buf.st_mode & S_IWUSR)){
      if (( user = getpwuid(buf.st_uid)) != NULL){
         if (users){
-           GB.NewString((char **)GB.Add(users), user->pw_name, 0);
+           *(char **)GB.Add(users) = GB.NewZeroString(user->pw_name);
 	}
 	else {
 	   Count++;
@@ -2486,7 +2461,7 @@ int EXPORT GB_INIT(void)
   GB.GetInterface("gb.db", DB_INTERFACE_VERSION, &DB);
   DB.Register(&_driver);
 
-  return FALSE;
+  return 0;
 }
 
 void EXPORT GB_EXIT()
