@@ -1,3 +1,26 @@
+/***************************************************************************
+
+  sqlitedataset.cpp
+
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2, or (at your option)
+  any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
+
+***************************************************************************/
+
 /**********************************************************************
  * Copyright (c) 2002, Leo Seib, Hannover
  *
@@ -71,7 +94,7 @@ int callback(void* res_ptr,int ncol, char** reslt,char** cols){
 	 }
 	 else {
              //table = strndup(cols[i], strchr(cols[i], (int)'.') - cols[i]);
-             GB.NewString(&table, cols[i], strchr(cols[i], (int)'.') - cols[i]);
+             table = GB.NewString(cols[i], strchr(cols[i], (int)'.') - cols[i]);
 //             table = strdup(reslt[0]);
              r->record_header[i].name = item + 1;
              r->record_header[i].field_table = table;
@@ -279,7 +302,6 @@ long SqliteDatabase::nextid(const char* sname) {
     if ((last_err = sqlite_exec(conn,sqlcmd,NULL,NULL,NULL)) != SQLITE_OK) return DB_UNEXPECTED_RESULT;
     return id;    
   }
-  return DB_UNEXPECTED_RESULT;
 }
 
 
@@ -648,7 +670,7 @@ fType GetFieldType(const char *Type, unsigned int *length )
 	unsigned int rTypeLen;
 	int i;
 
-	GB.NewString(&upper, Type, 0);
+	upper = GB.NewZeroString(Type);
 	for (i = 0; i < GB.StringLength(upper); i++)
 		upper[i] = toupper(upper[i]);
 	
