@@ -2,21 +2,22 @@
 
   desktop.h
 
-  (c) 2007 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -24,9 +25,22 @@
 #define __DESKTOP_H
 
 #include "main.h"
+#include "gb_list.h"
 
 #ifndef __DESKTOP_C
 extern GB_DESC CDesktopDesc[];
+extern GB_DESC CDesktopWatcherDesc[];
 #endif
 
+typedef
+	struct {
+		GB_BASE ob;
+		Atom property;
+		Window window;
+		LIST list;
+		}
+	CDESKTOPWATCHER;
+	
+#define WATCHER ((CDESKTOPWATCHER *)_object)
+
 #endif
