@@ -3,16 +3,10 @@
   gapplication.cpp
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  Gtkmae "GTK+ made easy" classes
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,12 +16,14 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #include <ctype.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "widgets.h"
 #include "widgets_private.h"
@@ -40,9 +36,11 @@
 #include "gdialog.h"
 #include "gclipboard.h"
 #include "gmouse.h"
+#include "gprinter.h"
 #include "gmainwindow.h"
 
 //#define DEBUG_IM 1
+//#define DEBUG_ENTER_LEAVE 1
 
 /*************************************************************************
 
@@ -69,8 +67,16 @@ int gKey::code()
 {
 	if (!_valid)
 		return 0;
-	else
-		return _event.keyval;
+	
+	int code = _event.keyval;
+	
+	if (code >= GDK_a && code <= GDK_z)
+		code += GDK_A - GDK_a;
+	else if (code == GDK_Alt_L || code == GDK_Alt_R || code == GDK_Control_L || code == GDK_Control_R 
+		       || code == GDK_Meta_L || code == GDK_Meta_R || code == GDK_Shift_L || code == GDK_Shift_R)
+		code = 0;
+	
+	return code;
 }
 
 int gKey::state()
@@ -83,17 +89,17 @@ int gKey::state()
 
 bool gKey::alt()
 {
-	return state() & GDK_MOD1_MASK || code() == GDK_Alt_L || code() == GDK_Alt_R;
+	return state() & GDK_MOD1_MASK || _event.keyval == GDK_Alt_L || _event.keyval == GDK_Alt_R;
 }
 
 bool gKey::control()
 {
-	return state() & GDK_CONTROL_MASK || code() == GDK_Control_L || code() == GDK_Control_R;
+	return state() & GDK_CONTROL_MASK || _event.keyval == GDK_Control_L || _event.keyval == GDK_Control_R;
 }
 
 bool gKey::meta()
 {
-	return state() & GDK_MOD2_MASK || code() == GDK_Meta_L || code() == GDK_Meta_R;
+	return state() & GDK_MOD2_MASK || _event.keyval == GDK_Meta_L || _event.keyval == GDK_Meta_R;
 }
 
 bool gKey::normal()
@@ -103,7 +109,7 @@ bool gKey::normal()
 
 bool gKey::shift()
 {
-	return state() & GDK_SHIFT_MASK || code() == GDK_Shift_L || code() == GDK_Shift_R;
+	return state() & GDK_SHIFT_MASK || _event.keyval == GDK_Shift_L || _event.keyval == GDK_Shift_R;
 }
 
 int gKey::fromString(char *str)
@@ -111,6 +117,9 @@ int gKey::fromString(char *str)
 	char *lstr;
 	int key;
 	
+	if (!str || !*str)
+		return 0;
+	
 	lstr = g_ascii_strup(str, -1);
 	key = gdk_keyval_from_name(lstr);
 	g_free(lstr);
@@ -253,11 +262,14 @@ void gKey::setActiveControl(gControl *control)
 	
 **************************************************************************/
 
+static bool _focus_change = false;
+
 static bool check_button(gControl *w)
 {
 	return w && w->isVisible() && w->enabled();
 }
 
+#if 0
 static GtkWindowGroup *get_window_group(GtkWidget *widget)
 {
   GtkWidget *toplevel = NULL;
@@ -266,48 +278,155 @@ static GtkWindowGroup *get_window_group(GtkWidget *widget)
     toplevel = gtk_widget_get_toplevel(widget);
 
   if (GTK_IS_WINDOW (toplevel))
-	  return gtk_window_get_group(GTK_WINDOW (toplevel));
-	else
+    return gtk_window_get_group(GTK_WINDOW (toplevel));
+  else
     return gtk_window_get_group(NULL);
 }
+#endif
 
-static bool is_child_of(GtkWidget *child, GtkWidget *parent)
+/*static gboolean close_dialog(GtkButton *button)
 {
-	for(;;)
-	{
-		if (!child)
-			return FALSE;
-		if (child == parent)
-			return TRUE;
+	gtk_button_clicked(button);
+	return FALSE;
+}*/
+
+static bool raise_key_event_to_parent_window(gControl *control, int type)
+{
+	gMainWindow *win;
 	
-		child = gtk_widget_get_parent(child);
+	while (control->parent())
+	{
+		win = control->parent()->window();
+		if (win->onKeyEvent && win->canRaise(win, type))
+		{
+			if (win->onKeyEvent(win, type))
+				return true;
+		}
+		
+		control = win;
 	}
+	
+	return false;
+}
+
+static bool check_crossing_event(GdkEvent *event)
+{
+	#if DEBUG_ENTER_LEAVE
+	fprintf(stderr, "check_crossing_event: %d / %d\n", event->crossing.detail, event->crossing.mode);
+	#endif
+	
+	if (event->crossing.detail != GDK_NOTIFY_INFERIOR 
+	    && (event->crossing.mode == GDK_CROSSING_NORMAL || event->crossing.mode == GDK_CROSSING_STATE_CHANGED))
+		// || event->crossing.mode == GDK_CROSSING_UNGRAB || event->crossing.mode == GDK_CROSSING_GTK_UNGRAB))
+		return true;
+	else
+		return false;
 }
 
 static void gambas_handle_event(GdkEvent *event)
 {
   GtkWidget *widget;
   GtkWidget *grab;
-	GtkWindowGroup *group;
 	gControl *control;
 	int x, y, xc, yc;
-	bool real;
+	bool cancel;
+	int type;
 	
-	if (!((event->type >= GDK_MOTION_NOTIFY && event->type <= GDK_LEAVE_NOTIFY) || event->type == GDK_SCROLL))
+	if (gApplication::_fix_printer_dialog)
+	{
+		widget = gtk_get_event_widget(event);
+		if (widget)
+		{
+			//fprintf(stderr, "type: %s\n", G_OBJECT_TYPE_NAME(widget));
+			if (!strcmp(G_OBJECT_TYPE_NAME(gtk_widget_get_toplevel(widget)), "GtkPrintUnixDialog"))
+			{
+				if (event->type == GDK_WINDOW_STATE)
+				{
+					//fprintf(stderr, "event: GDK_WINDOW_STATE!\n");
+					widget = gtk_window_get_default_widget(GTK_WINDOW(gtk_widget_get_toplevel(widget)));
+					if (widget && GTK_IS_BUTTON(widget))
+					{
+						GtkPrintUnixDialog *dialog = GTK_PRINT_UNIX_DIALOG(gtk_widget_get_toplevel(widget));
+						gPrinter::fixPrintDialog(dialog);
+						gApplication::_fix_printer_dialog = false;
+						//fprintf(stderr, "gtk_button_clicked: %s\n", gtk_button_get_label(GTK_BUTTON(widget)));
+						if (gApplication::_close_next_window)
+							gtk_button_clicked(GTK_BUTTON(widget));
+						gApplication::_close_next_window = false;
+						//return;
+						//g_timeout_add(0, (GSourceFunc)close_dialog, GTK_BUTTON(widget));
+						goto __HANDLE_EVENT;
+					}
+					//fprintf(stderr, "event: MAP! <<< end\n");
+				}
+			}
+		}
+	}
+	
+	/*if (event->type == GDK_GRAB_BROKEN)
+	{
+		if (gApplication::_in_popup)
+			fprintf(stderr, "**** GDK_GRAB_BROKEN inside popup: %s %swindow = %p grab_window = %p popup_window = %p\n", event->grab_broken.keyboard ? "keyboard" : "pointer",
+							event->grab_broken.implicit ? "implicit " : "", event->grab_broken.window, event->grab_broken.grab_window, gApplication::_popup_grab_window);
+	}*/
+	
+	if (!((event->type >= GDK_MOTION_NOTIFY && event->type <= GDK_FOCUS_CHANGE) || event->type == GDK_SCROLL))
 		goto __HANDLE_EVENT;
 	
 	widget = gtk_get_event_widget(event);
+	if (!widget)
+		goto __HANDLE_EVENT;
 	
-	real = true;
+	grab = gtk_grab_get_current();
 	
+	if (event->type == GDK_FOCUS_CHANGE)
+	{
+		control = NULL;
+		if (GTK_IS_WINDOW(widget))
+			control = (gControl *)g_object_get_data(G_OBJECT(widget), "gambas-control");
+		
+		//fprintf(stderr, "GDK_FOCUS_CHANGE: widget = %p %d : %s  grab = %p\n", widget, GTK_IS_WINDOW(widget), control ? control->name() : NULL, grab);
+		
+		if (GTK_IS_WINDOW(widget))
+		{
+			control = (gControl *)g_object_get_data(G_OBJECT(widget), "gambas-control");
+			if (control)
+				gApplication::setActiveControl(control, event->focus_change.in);
+			else if (event->focus_change.in);
+				gMainWindow::setActiveWindow(NULL);
+		}
+
+		if (event->focus_change.in && grab && widget != grab && !gtk_widget_is_ancestor(widget, grab))
+		{
+			//fprintf(stderr, "Check popup grab\n");
+			gApplication::grabPopup();
+			// Must continue, otherwise things are broken by some styles
+			//return;
+		}
+			
+		goto __HANDLE_EVENT;
+	}
+	
+	if (grab)
+	{
+		if (widget != grab && !gtk_widget_is_ancestor(widget, grab))
+			widget = grab;
+	}
+	
+	//real = true;
 	while (widget)
 	{
 		control = (gControl *)g_object_get_data(G_OBJECT(widget), "gambas-control");
 		if (control)
 			break;
 		widget = widget->parent;
-		real = false;
+		//real = false;
 	}
+
+	/*if (event->type == GDK_BUTTON_PRESS)
+		fprintf(stderr, "GDK_BUTTON_PRESS: %p %s\n", widget, control ? control->name() : NULL);
+	else if (event->type == GDK_KEY_PRESS)
+		fprintf(stderr, "GDK_KEY_PRESS: %p %s (%s)\n", widget, control ? control->name() : NULL, gApplication::activeControl() ? gApplication::activeControl()->name() : NULL);*/
 	
 	if (!widget || !control)
 		goto __HANDLE_EVENT;
@@ -322,97 +441,189 @@ static void gambas_handle_event(GdkEvent *event)
 			fprintf(stderr, "LEAVE: %p %s\n", control, control ? control->name() : 0);
 			break;
 	}*/
-	
-	grab = gtk_grab_get_current();
-	
-	if (grab && !is_child_of(widget, grab))
-		goto __HANDLE_EVENT;
-	
-	group = get_window_group(widget);
-
+		
+	//group = get_window_group(widget);
 	//if (group != gApplication::currentGroup())
 	//	goto __HANDLE_EVENT;
 	
-	if (event->type != GDK_ENTER_NOTIFY)
+	if (!gdk_events_pending() && event->type != GDK_ENTER_NOTIFY && event->type != GDK_LEAVE_NOTIFY)
 	{
 		if (gApplication::_leave)
 		{
-			if (event->type != GDK_LEAVE_NOTIFY || gApplication::_leave != control)
-				gApplication::_leave->emit(SIGNAL(gApplication::_leave->onEnterLeave), gEvent_Leave);
+			//if () // || (gApplication::_leave != control && check_crossing_event(event)))
+			#if DEBUG_ENTER_LEAVE
+			fprintf(stderr, "post leave: %s\n", gApplication::_leave->name());
+			#endif
+			
+			if (gApplication::_enter == gApplication::_leave)
+				gApplication::_enter = NULL;
+			
+			gApplication::_leave->emitLeaveEvent();
+			
 			gApplication::_leave = NULL;
 		}
 	}
 	
+	cancel = false;
+	
+	gApplication::updateLastEventTime(event);
+	
 	switch ((int)event->type)
 	{
 		case GDK_ENTER_NOTIFY:
 			
-			//gApplication::dispatchEnterLeave(control);
 			if (gApplication::_leave == control)
+			{
+				#if DEBUG_ENTER_LEAVE
+				fprintf(stderr, "enter ignored: %s\n", control->name());
+				#endif
 				gApplication::_leave = NULL;
+			}
 			else if (gApplication::_enter != control)
 			{
-				gApplication::_enter = control;
-				control->emit(SIGNAL(control->onEnterLeave), gEvent_Enter);
+				if (check_crossing_event(event))
+				{
+					#if DEBUG_ENTER_LEAVE
+					fprintf(stderr, "enter: %s\n", control->name());
+					#endif
+					gApplication::_enter = control;
+					control->emitEnterEvent();
+				}
 			}
 			break;
 		
 		case GDK_LEAVE_NOTIFY:
-			if (gdk_events_pending())
-				gApplication::_leave = control;
-			else
+			
+			if (gdk_events_pending() && gApplication::_leave == NULL)
 			{
-				if (gApplication::_enter == control)
-					gApplication::_enter = NULL;
-				control->emit(SIGNAL(control->onEnterLeave), gEvent_Leave);
+				if (check_crossing_event(event))
+				{
+					#if DEBUG_ENTER_LEAVE
+					fprintf(stderr, "leave later: %s\n", control->name());
+					#endif
+					gApplication::_leave = control;
+				}
+			}
+			else if (gApplication::_leave != control)
+			{
+				if (check_crossing_event(event))
+				{
+					if (gApplication::_enter == control)
+						gApplication::_enter = NULL;
+					
+					if (gApplication::_leave == control)
+						gApplication::_leave = NULL;
+			
+					#if DEBUG_ENTER_LEAVE
+					fprintf(stderr, "leave: %s\n", control->name());
+					#endif
+					control->emitLeaveEvent();
+				}
 			}
 			break;
 			
 		case GDK_BUTTON_PRESS:
 		case GDK_2BUTTON_PRESS:
 		case GDK_BUTTON_RELEASE:
+		{
+			bool menu = false;
+			gControl *save_control = control;
 			
-			gApplication::_event_time = event->button.time;
+			switch ((int)event->type)
+			{
+				case GDK_BUTTON_PRESS: type = gEvent_MousePress; break;
+				case GDK_2BUTTON_PRESS: type = gEvent_MouseDblClick; break;
+				default: type = gEvent_MouseRelease; break;
+			}
 			
+		__BUTTON_TRY_PROXY:
+		
+			cancel = false;
+		
 			if (control->onMouseEvent)
 			{
-				control->getScreenPos(&xc, &yc);
-				x = (int)event->button.x_root - xc;
-				y = (int)event->button.y_root - yc;
-				
-				gMouse::validate();
-				gMouse::setStart(x, y);
-				gMouse::setMouse(x, y, event->button.button, event->button.state);
-				//gMouse::setValid(1,(int)event->x,(int)event->y,event->button,event->state,data->screenX(),data->screenY());
-				switch ((int)event->type)
+				if (control->canRaise(control, type))
 				{
-					case GDK_BUTTON_PRESS: 
-						control->onMouseEvent(control, gEvent_MousePress); 
-						break;
+					control->getScreenPos(&xc, &yc);
+					x = (int)event->button.x_root - xc;
+					y = (int)event->button.y_root - yc;
 					
-					case GDK_2BUTTON_PRESS: 
-						control->onMouseEvent(control, gEvent_MouseDblClick); 
-						break;
+					gMouse::validate();
+					gMouse::setStart(x, y);
+					gMouse::setMouse(x, y, event->button.button, event->button.state);
+					//gMouse::setValid(1,(int)event->x,(int)event->y,event->button,event->state,data->screenX(),data->screenY());
+					switch ((int)event->type)
+					{
+						case GDK_BUTTON_PRESS: 
+							cancel = control->onMouseEvent(control, gEvent_MousePress);
+							break;
+						
+						case GDK_2BUTTON_PRESS: 
+							cancel = control->onMouseEvent(control, gEvent_MouseDblClick); 
+							break;
+						
+						case GDK_BUTTON_RELEASE: 
+							cancel = control->onMouseEvent(control, gEvent_MouseRelease); 
+							break;
+					}
 					
-					case GDK_BUTTON_RELEASE: 
-						control->onMouseEvent(control, gEvent_MouseRelease); 
-						/*if (control->_grab)
-							gApplication::exitLoop(control);*/
-						break;
+					gMouse::invalidate();
 				}
-				
-				gMouse::invalidate();
-				
-				if (event->button.button == 3 && event->type == GDK_BUTTON_PRESS)
-					control->onMouseEvent(control, gEvent_MouseMenu);
 			}
+				
+			if (type == gEvent_MousePress && control->isTopLevel())
+			{
+				gMainWindow *win = ((gMainWindow *)control);
+				if (win->isPopup())
+				{
+					control->getScreenPos(&xc, &yc);
+					x = (int)event->button.x_root - xc;
+					y = (int)event->button.y_root - yc;
+				
+					if (x < 0 || y < 0 || x >= win->width() || y >= win->height())
+						win->close();
+				}
+			}
+			else if (type == gEvent_MouseRelease && control->_grab)
+			{
+				gApplication::exitLoop(control);
+			}
+			
+			if (event->button.button == 3 && event->type == GDK_BUTTON_PRESS)
+				menu = true;
+			
+			if (!cancel)
+			{
+				if (control->_proxy_for)
+				{
+					control = control->_proxy_for;
+					goto __BUTTON_TRY_PROXY;
+				}
+			}
+			
+			if (menu)
+			{
+				control = save_control;
+				while (control)
+				{
+					if (control->onMouseEvent(control, gEvent_MouseMenu))
+						break;
+					control = control->_proxy_for;
+				}
+			}
+			
+			if (cancel)
+				return;
+			
 			break;
+		}
 			
 		case GDK_MOTION_NOTIFY:
 
-			gApplication::_event_time = event->motion.time;
-			
-			if (control->onMouseEvent && (control->isTracking() || (event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK))))
+		__MOTION_TRY_PROXY:
+		
+			if (control->onMouseEvent && (control->canRaise(control, gEvent_MouseMove) || control->canRaise(control, gEvent_MouseDrag))
+					&& (control->isTracking() || (event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK))))
 			{
 				control->getScreenPos(&xc, &yc);
 				x = (int)event->motion.x_root - xc;
@@ -420,7 +631,7 @@ static void gambas_handle_event(GdkEvent *event)
 				
 				gMouse::validate();
 				gMouse::setMouse(x, y, 0, event->motion.state);
-				control->onMouseEvent(control, gEvent_MouseMove);
+				cancel = control->onMouseEvent(control, gEvent_MouseMove);
 				//if (data->acceptDrops() && gDrag::checkThreshold(data, gMouse::x(), gMouse::y(), gMouse::startX(), gMouse::startY()))
 				if ((event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) 
 						//&& (abs(gMouse::x() - gMouse::y()) + abs(gMouse::startX() - gMouse::startY())) > 8)
@@ -429,14 +640,24 @@ static void gambas_handle_event(GdkEvent *event)
 					control->onMouseEvent(control, gEvent_MouseDrag);
 				}
 				gMouse::invalidate();
+				
+				if (cancel)
+					return;
 			}
+
+			if (control->_proxy_for)
+			{
+				control = control->_proxy_for;
+				goto __MOTION_TRY_PROXY;
+			}
+			
 			break;
 			
 		case GDK_SCROLL:
 			
-			gApplication::_event_time = event->scroll.time;
-			
-			if (control->onMouseEvent)
+		__SCROLL_TRY_PROXY:
+		
+			if (control->onMouseEvent && control->canRaise(control, gEvent_MouseWheel))
 			{
 				int dt, ort;
 				
@@ -455,162 +676,170 @@ static void gambas_handle_event(GdkEvent *event)
 				gMouse::validate();
 				gMouse::setMouse(x, y, 0, event->scroll.state);
 				gMouse::setWheel(dt, ort);
-				control->onMouseEvent(control, gEvent_MouseWheel);
+				cancel = control->onMouseEvent(control, gEvent_MouseWheel);
 				gMouse::invalidate();
 			}
+
+			if (cancel)
+				return;
+				
+			if (control->_proxy_for)
+			{
+				control = control->_proxy_for;
+				goto __SCROLL_TRY_PROXY;
+			}
+			
 			break;
 
 		case GDK_KEY_PRESS:
+		case GDK_KEY_RELEASE:
 		{
-			bool cancel = false;
+			gMainWindow *win;
 			
-			gApplication::_event_time = event->key.time;						
-			control = gDesktop::activeControl();
+			if (!control->_grab && gApplication::activeControl())
+				control = gApplication::activeControl();
+			
+			type =  (event->type == GDK_KEY_PRESS) ? gEvent_KeyPress : gEvent_KeyRelease;
 			
 			if (control)
 			{
+			__KEY_TRY_PROXY:
+			
+				win = control->window();
+				
 				if (!gKey::enable(control, &event->key))
 				{
 					if (gApplication::onKeyEvent)
-						cancel = gApplication::onKeyEvent(gEvent_KeyPress);
-					if (!cancel && control->onKeyEvent) 
+						cancel = gApplication::onKeyEvent(type);
+					if (!cancel)
+						cancel = raise_key_event_to_parent_window(control, type);
+					if (!cancel && control->onKeyEvent && control->canRaise(control, type)) 
 					{
 						//fprintf(stderr, "gEvent_KeyPress on %p %s\n", control, control->name());
-						cancel = control->onKeyEvent(control, gEvent_KeyPress);
+						cancel = control->onKeyEvent(control, type);
 					}
 				}
 				gKey::disable();
-			
+				
 				if (cancel)
 					return;
 				
+				if (control->_proxy_for)
+				{
+					control = control->_proxy_for;
+					goto __KEY_TRY_PROXY;
+				}
+				
 				if (event->key.keyval == GDK_Escape)
 				{
-					/*if (control->_grab)
+					if (control->_grab)
 					{
 						gApplication::exitLoop(control);
 						return;
-					}*/
+					}
 					
-					gMainWindow *win = control->window();
 					if (check_button(win->_cancel))
 					{
-						win->_cancel->animateClick(false);
+						win->_cancel->animateClick(type == gEvent_KeyRelease);
 						return;
 					}
 				}
 				else if (event->key.keyval == GDK_Return || event->key.keyval == GDK_KP_Enter)
 				{
-					gMainWindow *win = control->window();
 					if (check_button(win->_default))
 					{
-						win->_default->animateClick(false);
+						win->_default->animateClick(type == gEvent_KeyRelease);
 						return;
 					}
 				}
+				
+				if (control->_grab)
+					return;
 			}
-			
+
 			break;
 		}
-			
-		case GDK_KEY_RELEASE:
-			
-			gApplication::_event_time = event->key.time;						
-			control = gDesktop::activeControl();
-			
-			if (control)
-			{
-				if (!gKey::enable(control, &event->key))
-					control->emit(SIGNAL(control->onKeyEvent), gEvent_KeyRelease);
-				gKey::disable();
-			
-				if (event->key.keyval == GDK_Escape)
-				{
-					gMainWindow *win = control->window();
-					if (check_button(win->_cancel))
-						win->_cancel->animateClick(true);
-				}
-				else if (event->key.keyval == GDK_Return || event->key.keyval == GDK_KP_Enter)
-				{
-					gMainWindow *win = control->window();
-					if (check_button(win->_default))
-						win->_default->animateClick(true);
-				}
-			}
-			
-			break;
 	}
-
+	
 __HANDLE_EVENT:
 
 	gtk_main_do_event(event);
 }
 
 
-/********************************************************************
+
+/**************************************************************************
 
 gApplication
 
-*********************************************************************/
+**************************************************************************/
 
 int appEvents;
-GtkTooltips *app_tooltips_handle;
-bool app_tooltips=true;
-gFont *app_tooltips_font=NULL;
+
 bool gApplication::_busy = false;
 char *gApplication::_title = NULL;
+int gApplication::_in_popup = 0;
+GdkWindow *gApplication::_popup_grab_window = NULL;
 int gApplication::_loopLevel = 0;
 void *gApplication::_loop_owner = 0;
 GtkWindowGroup *gApplication::_group = NULL;
 gControl *gApplication::_enter = NULL;
 gControl *gApplication::_leave = NULL;
+gControl *gApplication::_active_control = NULL;
+gControl *gApplication::_old_active_control = NULL;
 bool (*gApplication::onKeyEvent)(int) = NULL;
 guint32 gApplication::_event_time = 0;
+bool gApplication::_close_next_window = false;
+bool gApplication::_fix_printer_dialog = false;
 
-GtkTooltips* gApplication::tipHandle()
+void gApplication::grabPopup()
 {
-	return app_tooltips_handle;
-}
-
-bool gApplication::toolTips()
-{
-	return app_tooltips;
-}
-
-void gApplication::setToolTipsFont(gFont *ft)
-{
-	GList *chd;
-	PangoFontDescription *desc;
+	if (!_popup_grab_window)
+		return;
 	
-	gFont::set(&app_tooltips_font, ft->copy());
+  int ret = gdk_pointer_grab(_popup_grab_window, TRUE,
+			 (GdkEventMask)(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+			 GDK_POINTER_MOTION_MASK),
+			 NULL, NULL, GDK_CURRENT_TIME);
 	
-	if (ft)
-    desc = pango_context_get_font_description(ft->ct);
-  else
-    desc = NULL;
-	
-	gtk_widget_modify_font(app_tooltips_handle->tip_window,desc);
-	
-	chd=gtk_container_get_children(GTK_CONTAINER(app_tooltips_handle->tip_window));
-	if (chd)
+	if (ret == GDK_GRAB_SUCCESS)
 	{
-		do { gtk_widget_modify_font(GTK_WIDGET(chd->data),desc);
-		} while (chd->next);
-		g_list_free(chd);
+		ret = gdk_keyboard_grab(_popup_grab_window, TRUE, GDK_CURRENT_TIME);
+		
+		if (ret == GDK_GRAB_SUCCESS)
+			return;
+		
+		gdk_pointer_ungrab(GDK_CURRENT_TIME);
 	}
+
+	fprintf(stderr, "gb.gtk: warning: grab failed: %d\n", ret);
 }
 
-gFont *gApplication::toolTipsFont()
+void gApplication::ungrabPopup()
 {
-	return app_tooltips_font;
+	_popup_grab_window = NULL;
+	gdk_pointer_ungrab(GDK_CURRENT_TIME);
+	gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+}
+
+bool gApplication::areTooltipsEnabled()
+{
+  gboolean enabled;
+  GtkSettings *settings;
+
+  settings = gtk_settings_get_default();
+
+  g_object_get (settings, "gtk-enable-tooltips", &enabled, (char *)NULL);
+
+  return enabled;
 }
 
 void gApplication::enableTooltips(bool vl)
 {
-	if (vl)
-		gtk_tooltips_enable(app_tooltips_handle);
-	else
-		gtk_tooltips_disable(app_tooltips_handle);
+  GtkSettings *settings;
+	gboolean enabled = vl;
+  settings = gtk_settings_get_default();
+  g_object_set (settings, "gtk-enable-tooltips", enabled, (char *)NULL);
 }
 
 void gApplication::suspendEvents(bool vl)
@@ -641,16 +870,12 @@ void gApplication::init(int *argc,char ***argv)
 	appEvents=0;
 	
 	gtk_init(argc,argv);
-	
+ 
 	gdk_event_handler_set((GdkEventFunc)gambas_handle_event, NULL, NULL);
-
-	app_tooltips_handle = gtk_tooltips_new();
-	g_object_ref(G_OBJECT(app_tooltips_handle));
-	gtk_tooltips_force_window (app_tooltips_handle);
-	app_tooltips_font = new gFont(app_tooltips_handle->tip_window);
 	
 	gClipboard::init();
 	gKey::init();
+	_loop_owner = 0;
 }
 
 void gApplication::exit()
@@ -663,7 +888,6 @@ void gApplication::exit()
   gDesktop::exit();
   gMessage::exit();
   gDialog::exit();
-  gFont::assign(&app_tooltips_font);
   gFont::exit();
   gt_exit();
 }
@@ -743,6 +967,7 @@ void gApplication::setBusy(bool b)
     iter = g_list_next(iter);
   }
   
+  MAIN_do_iteration_just_events();
 }
 
 static bool _dirty = false;
@@ -815,15 +1040,15 @@ void gApplication::enterLoop(void *owner, bool showIt)
 	GtkWindowGroup *oldGroup;
 	
 	oldGroup = enterGroup();
-
+	
 	if (showIt) ((gControl *)owner)->show();
 
 	_loopLevel++;
 	_loop_owner = owner;
-
+	
 	do
 	{
-		do_iteration(false);
+		MAIN_do_iteration(false);
 	}
 	while (_loopLevel > l);
 	
@@ -832,6 +1057,47 @@ void gApplication::enterLoop(void *owner, bool showIt)
 	exitGroup(oldGroup);
 }
 
+void gApplication::enterPopup(gMainWindow *owner)
+{
+	void *old_owner = _loop_owner;
+	int l = _loopLevel;
+	GtkWindowGroup *oldGroup;
+	GtkWindow *window = GTK_WINDOW(owner->border);
+	
+	_in_popup++;
+	
+	oldGroup = enterGroup();
+	
+	gtk_window_set_modal(window, true);
+	gdk_window_set_override_redirect(owner->border->window, true);
+	owner->show();
+	
+	if (_in_popup == 1)
+	{
+		_popup_grab_window = owner->border->window;
+		gApplication::grabPopup();
+	}
+	
+	_loopLevel++;
+	_loop_owner = owner;
+	
+	do
+	{
+		MAIN_do_iteration(false);
+	}
+	while (_loopLevel > l);
+	
+	gApplication::ungrabPopup();
+	
+	_loop_owner = old_owner;
+
+	gdk_window_set_override_redirect(owner->border->window, false);
+	gtk_window_set_modal(window, false);
+	exitGroup(oldGroup);
+	
+	_in_popup--;
+}
+
 void gApplication::exitLoop(void *owner)
 {
 	if (!hasLoop(owner))
@@ -848,3 +1114,139 @@ GtkWindowGroup *gApplication::currentGroup()
 	else
 		return gtk_window_get_group(NULL);
 }
+
+void gApplication::updateLastEventTime(GdkEvent *e)
+{
+	guint32 time;
+	
+	switch (e->type)
+	{
+		case GDK_KEY_PRESS: case GDK_KEY_RELEASE: 
+			time = e->key.time; break;
+		case GDK_BUTTON_PRESS: case GDK_2BUTTON_PRESS: case GDK_3BUTTON_PRESS: case GDK_BUTTON_RELEASE:
+			time = e->button.time; break;
+		case GDK_MOTION_NOTIFY:
+			time = e->motion.time; break;
+		case GDK_SCROLL:
+			time = e->scroll.time; break;
+		case GDK_ENTER_NOTIFY: case GDK_LEAVE_NOTIFY:
+			time = e->crossing.time; break;
+		default:
+			time = GDK_CURRENT_TIME;
+	}
+	
+	_event_time = time;
+}
+
+static void post_focus_change(void *)
+{
+	gControl *current, *control;
+	
+	//fprintf(stderr, "post_focus_change\n");
+	
+	for(;;)
+	{
+		current = gApplication::activeControl();
+		if (current == gApplication::_old_active_control)
+			break;
+
+		control = gApplication::_old_active_control;
+		while (control)
+		{
+			if (control->onFocusEvent)
+				control->onFocusEvent(control, gEvent_FocusOut);
+			control = control->_proxy_for;
+		}
+		
+		gApplication::_old_active_control = current;
+		gMainWindow::setActiveWindow(current);
+		
+		control = current;
+		while (control)
+		{
+			if (control->onFocusEvent)
+				control->onFocusEvent(control, gEvent_FocusIn);
+			control = control->_proxy_for;
+		}
+	}
+	
+	_focus_change = FALSE;
+}
+
+static void handle_focus_change()
+{
+	if (_focus_change)
+		return;
+	
+	_focus_change = TRUE;
+	GB.Post((void (*)())post_focus_change, NULL);
+}
+
+void gApplication::setActiveControl(gControl *control, bool on)
+{
+	if (on == (_active_control == control))
+		return;
+	
+	//fprintf(stderr, "setActiveControl: %s %d\n", control->name(), on);
+
+	_active_control = on ? control : NULL;
+	gKey::setActiveControl(_active_control);
+	handle_focus_change();
+}
+
+int gApplication::getScrollbarSize()
+{
+	gint focus_line_width;
+	gint focus_padding;
+	gint trough_border;
+	gint slider_width;
+	
+  gtk_style_get(gt_get_style("GtkRange", GTK_TYPE_RANGE), GTK_TYPE_RANGE,
+		"slider-width", &slider_width,
+		"trough-border", &trough_border,
+		(char *)NULL);
+	
+	gtk_style_get(gt_get_style("GtkWidget", GTK_TYPE_WIDGET), GTK_TYPE_WIDGET,
+		"focus-line-width", &focus_line_width,
+		"focus-padding", &focus_padding,
+		(char *)NULL);
+
+	return (trough_border) * 2 + slider_width;
+}
+
+int gApplication::getScrollbarSpacing()
+{
+	gint v;
+	
+	gtk_style_get(gt_get_style("GtkScrolledWindow", GTK_TYPE_SCROLLED_WINDOW), GTK_TYPE_SCROLLED_WINDOW, 
+		"scrollbar-spacing", &v, 
+		(char *)NULL);
+	
+	return v;
+}
+
+int gApplication::getFrameWidth()
+{
+	return 2;
+}
+
+int gApplication::getTextBoxFrameWidth()
+{
+	GtkStyle *style;
+	gint focus_width;
+	gboolean interior_focus;
+	int v;
+
+	style = gt_get_style("GtkEntry", GTK_TYPE_ENTRY);
+	
+	gtk_style_get(style, GTK_TYPE_ENTRY,
+		"focus-line-width", &focus_width,
+		"interior-focus", &interior_focus,
+		(char *)NULL);
+	
+	v = MAX(style->xthickness, style->ythickness);
+	if (!interior_focus)
+		v += focus_width;
+	
+	return v;
+}
