@@ -3,16 +3,10 @@
   gframe.cpp
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  Gtkmae "GTK+ made easy" classes
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -47,7 +42,7 @@ gPanel::gPanel(gContainer *parent) : gContainer(parent)
 	g_typ = Type_gPanel;
 
 	border = gtk_event_box_new();
-	widget= gtk_layout_new(0,0);
+	widget = gtk_fixed_new();
 	frame = widget;
 	realize(true);
 }
@@ -64,7 +59,7 @@ gFrame::gFrame(gContainer *parent) : gContainer(parent)
 	g_typ=Type_gFrame;
 
 	border = gtk_event_box_new();
-	widget = gtk_layout_new(0, 0);
+	widget = gtk_fixed_new();
 	
 	fr = gtk_frame_new(NULL);
 	label = gtk_frame_get_label_widget(GTK_FRAME(fr));
@@ -72,9 +67,6 @@ gFrame::gFrame(gContainer *parent) : gContainer(parent)
 	gtk_frame_set_shadow_type(GTK_FRAME(fr), GTK_SHADOW_ETCHED_IN);
 	gtk_container_add(GTK_CONTAINER(widget), fr);
 	
-	gtk_widget_add_events(widget,GDK_POINTER_MOTION_MASK);
-	gtk_widget_add_events(widget,GDK_BUTTON_RELEASE_MASK);
-	
   realize(false);
   
 	g_signal_connect(G_OBJECT(border), "size-allocate", G_CALLBACK(cb_frame_resize), (gpointer)this);
@@ -107,8 +99,7 @@ void gFrame::setText(char *vl)
 		label = gtk_label_new(vl);
 		gtk_frame_set_label_widget(GTK_FRAME(fr), label);
 		setFont(font());
-		setBackground();
-		setForeground();
+		setForeground(foreground());
 		gtk_widget_show(label);
 	}
 	else
@@ -125,6 +116,5 @@ void gFrame::setFont(gFont *ft)
 void gFrame::setRealForeground(gColor color)
 {
 	gControl::setRealForeground(color);
-	if (label)
-		set_gdk_fg_color(label, color);
+	if (label) set_gdk_fg_color(label, color);
 }
