@@ -3,16 +3,10 @@
   gmainwindow.cpp
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  Gtkmae "GTK+ made easy" classes
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,13 +16,13 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #include <ctype.h>
 #include <time.h>
-#include <unistd.h>
 
 #include "widgets.h"
 #include "widgets_private.h"
@@ -37,24 +31,26 @@
 #include <X11/extensions/shape.h>
 #endif
 
+#include "x11.h"
+
 #include "gapplication.h"
 #include "gdesktop.h"
 #include "gkey.h"
 #include "gmenu.h"
 #include "gmessage.h"
 #include "gdialog.h"
+#include "gmouse.h"
 #include "gmainwindow.h"
 
 static gboolean win_frame(GtkWidget *widget,GdkEventWindowState *event,gMainWindow *data)
 {
 	data->performArrange();
+	data->emit(SIGNAL(data->onState));
 	return false;
 }
 
 static void cb_show (GtkWidget *widget, gMainWindow *data)
 {
-	//fprintf(stderr, "cb_show: %p\n", data);
-	//data->afterShow();
 	data->emitOpen();
 	if (data->opened)
 	{
@@ -68,8 +64,8 @@ static void cb_hide (GtkWidget *widget, gMainWindow *data)
 {
 	data->emit(SIGNAL(data->onHide));
 	data->_not_spontaneous = false;
-	if (data == gDesktop::activeWindow())
-		gMainWindow::setActiveWindow(NULL);
+	//if (data == gDesktop::activeWindow())
+	//	gMainWindow::setActiveWindow(NULL);
 }
 
 static gboolean win_close(GtkWidget *widget,GdkEvent  *event,gMainWindow *data)
@@ -85,12 +81,13 @@ static gboolean resize_later(gMainWindow *data)
 	data->bufW = data->_next_w;
 	data->bufH = data->_next_h;
 	data->_next_timer = 0;
+	data->configure();
 	data->performArrange();
 
 	if (data->onResize) 
 		data->onResize(data);
 	
-	data->refresh();
+	//data->refresh();
 	
 	return false;
 }
@@ -98,7 +95,6 @@ static gboolean resize_later(gMainWindow *data)
 static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainWindow *data)
 {
 	gint x, y;
-	//fprintf(stderr, "cb_configure: %p\n", data);
 	
 	if (data->opened)
 	{
@@ -116,7 +112,7 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 			if (data->onMove) data->onMove(data);
 		}
 		
-		if ( (event->width!=data->bufW) || (event->height!=data->bufH) || (data->_resized) || !event->window )
+		if ((event->width != data->bufW) || (event->height != data->bufH) || (data->_resized) || !event->window)
 		{
 			data->_next_w = event->width;
 			data->_next_h = event->height;
@@ -125,42 +121,29 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 			//g_debug("gMainWindow: cb_configure: %d", event->send_event);
 			data->performArrange();
 			if (data->onResize) data->onResize(data);*/
-			
 			//if (data->_resized || data->parent())
-			{
+			//{
 				data->_resized = false;
 				resize_later(data);
-			}
-			//else
-			//{
-			//	//resize_later(data);
-			//	if (!data->_next_timer)
-			//		data->_next_timer = g_timeout_add(50, (GSourceFunc)resize_later, data);
-			//}
+			/*}
+			else
+			{
+				//resize_later(data);
+				if (!data->_next_timer)
+					data->_next_timer = g_timeout_add(50, (GSourceFunc)resize_later, data);
+			}*/
 		}
 	}
 	
 	return false;
 }
 
-static gboolean cb_open(GtkWidget *widget, GdkEvent *e, gMainWindow *data)
-{
-	data->emitOpen();
-	return FALSE;
-}
-
-static gboolean cb_map(GtkWidget *widget, GdkEvent *e, gMainWindow *data)
-{
-	data->afterShow();
-	return FALSE;
-}
-
 static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gMainWindow *data)
 {
 	if (data->_background)
 	{
 		gdk_window_clear(data->border->window);
-		gdk_window_clear(GTK_LAYOUT(data->widget)->bin_window);
+		//gdk_window_clear(GTK_LAYOUT(data->widget)->bin_window);
 	}
 	
 	return false;
@@ -180,6 +163,7 @@ void gMainWindow::initialize()
 	sticky = false;
 	persistent = false;
 	stack = 0;
+	_type = 0;
 	_mask = false;
 	_masked = false;
 	_resized = false;
@@ -187,13 +171,13 @@ void gMainWindow::initialize()
 	_default = NULL;
 	_cancel = NULL;
 	menuBar = NULL;
+	layout = NULL;
 	top_only = false;
 	_icon = NULL;
 	_picture = NULL;
 	focus = 0;
 	_closing = false;
 	_title = NULL;
-	_toolbox = false;
 	_not_spontaneous = false;
 	_skip_taskbar = false;
 	_current = NULL;
@@ -203,6 +187,9 @@ void gMainWindow::initialize()
 	_xembed = false;
 	_activate = false;
 	_hidden = false;
+	_hideMenuBar = false;
+	_showMenuBar = true;
+	_popup = false;
 
 	onOpen = NULL;
 	onShow = NULL;
@@ -211,6 +198,7 @@ void gMainWindow::initialize()
 	onResize = NULL;
 	onActivate = NULL;
 	onDeactivate = NULL;
+	onState = NULL;
 	
 	accel = gtk_accel_group_new();
 }
@@ -221,13 +209,12 @@ void gMainWindow::initWindow()
 	
 	if (!isTopLevel())
 	{
-		g_signal_connect(G_OBJECT(border), "show", G_CALLBACK(cb_show), (gpointer)this);
 		g_signal_connect(G_OBJECT(border), "configure-event", G_CALLBACK(cb_configure), (gpointer)this);
-		g_signal_connect_after(G_OBJECT(border), "map-event", G_CALLBACK(cb_open), (gpointer)this);
-		g_signal_connect(G_OBJECT(border),"show",G_CALLBACK(cb_show),(gpointer)this);
-		g_signal_connect(G_OBJECT(border),"hide",G_CALLBACK(cb_hide),(gpointer)this);
+		g_signal_connect_after(G_OBJECT(border), "map", G_CALLBACK(cb_show), (gpointer)this);
+		g_signal_connect(G_OBJECT(border),"unmap",G_CALLBACK(cb_hide),(gpointer)this);
 		//g_signal_connect_after(G_OBJECT(border), "size-allocate", G_CALLBACK(cb_configure), (gpointer)this);
 		g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(cb_expose), (gpointer)this);
+		gtk_widget_add_events(border, GDK_STRUCTURE_MASK);	
 	}
 	else
 	{
@@ -237,21 +224,13 @@ void gMainWindow::initWindow()
 		g_signal_connect(G_OBJECT(border), "configure-event",G_CALLBACK(cb_configure),(gpointer)this);
 		g_signal_connect(G_OBJECT(border), "delete-event",G_CALLBACK(win_close),(gpointer)this);
 		g_signal_connect(G_OBJECT(border), "window-state-event",G_CALLBACK(win_frame),(gpointer)this);
-		g_signal_connect_after(G_OBJECT(border), "map-event", G_CALLBACK(cb_map), (gpointer)this);
 		
 		gtk_widget_add_events(widget,GDK_BUTTON_MOTION_MASK);
 		g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(cb_expose), (gpointer)this);
 	}
 	
-	//GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
-	//gtk_widget_add_events(widget, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
-	
-	/*	GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-		| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
-		| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK | GDK_POINTER_MOTION_MASK);*/
-
 	gtk_window_add_accel_group(GTK_WINDOW(topLevel()->border), accel);
-
+	
 	have_cursor = true; //parent() == 0 && !_xembed;
 }
 
@@ -269,7 +248,7 @@ gMainWindow::gMainWindow(int plug) : gContainer(NULL)
 	else
 		border = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	
-	widget = gtk_layout_new(0,0);
+	widget = gtk_fixed_new(); //gtk_layout_new(0,0);
 	
 	realize(false);
 	initWindow();
@@ -286,7 +265,7 @@ gMainWindow::gMainWindow(gContainer *par) : gContainer(par)
 	g_typ = Type_gMainWindow;
 	
 	border = gtk_event_box_new();
-	widget = gtk_layout_new(0,0);
+	widget = gtk_fixed_new();
 	
 	realize(false);
 	
@@ -301,7 +280,7 @@ gMainWindow::~gMainWindow()
 	{
 		emit(SIGNAL(onClose));
 		opened = false;
-		if (GTK_IS_WINDOW(border) && modal())
+		if (GTK_IS_WINDOW(border) && isModal())
 			gApplication::exitLoop(this);
 	}
 	
@@ -315,6 +294,9 @@ gMainWindow::~gMainWindow()
 	if (_background) g_object_unref(_background);
 	if (_style) g_object_unref(_style);
 	
+	if (_active == this)
+		_active = NULL;
+	
 	windows = g_list_remove(windows, (gpointer)this);
 }
 
@@ -362,7 +344,16 @@ void gMainWindow::setStacking(int vl)
 void gMainWindow::setRealBackground(gColor color)
 {
 	if (!_picture)
+	{
 		gControl::setRealBackground(color);
+		gMenu::updateColor(this);
+	}
+}
+
+void gMainWindow::setRealForeground(gColor color)
+{
+	gControl::setRealForeground(color);
+	gMenu::updateColor(this);
 }
 
 void gMainWindow::move(int x, int y)
@@ -385,21 +376,35 @@ void gMainWindow::move(int x, int y)
 
 void gMainWindow::resize(int w, int h)
 {
+	if (w == bufW && h == bufH)
+		return;
+			
+	_resized = true;
+		
 	if (isTopLevel())
 	{
-		if (w == bufW && h == bufH)
-			return;
-			
-		_resized=true;
-		bufW = w;
-		bufH = h;
-		
 		//gdk_window_enable_synchronized_configure (border->window);
 		
-		if (gtk_window_get_resizable(GTK_WINDOW(border)))
-			gtk_window_resize(GTK_WINDOW(border),w,h);
+		bufW = w < 0 ? 0 : w;
+		bufH = h < 0 ? 0 : h;
+		
+		if (w < 1 || h < 1)
+		{
+			if (visible)
+				gtk_widget_hide(border);			
+		}
 		else
-			gtk_widget_set_size_request(border, w, h);
+		{
+			//fprintf(stderr, "resize: %d %d (%d)\n", w, h, gtk_window_get_resizable(GTK_WINDOW(border)));
+			if (isResizable())
+				gtk_window_resize(GTK_WINDOW(border),w, h);
+			else
+				gtk_widget_set_size_request(border, w, h);
+				
+			if (visible)
+				gtk_widget_show(border);
+			
+		}
 	}
 	else
 	{
@@ -412,9 +417,25 @@ void gMainWindow::emitOpen()
 {
 	if (!opened)
 	{
-		//fprintf(stderr, "emit Open: %p (%d %d)\n", this, width(), height());
+		//fprintf(stderr, "emit Open: %p (%d %d) resizable = %d\n", this, width(), height(), isResizable());
 		opened = true;
-		gtk_widget_realize(border);
+		
+		if (isTopLevel()) 
+		{
+			gtk_widget_realize(border);
+			/*if (modal())
+			{
+				GdkGeometry geometry;
+				
+				geometry.min_width = bufW;
+				geometry.min_height = bufH;
+				
+				fprintf(stderr, "gtk_window_set_geometry_hints: %d %d\n", bufW, bufH);
+				
+				gtk_window_set_geometry_hints(GTK_WINDOW(border), border, &geometry, GDK_HINT_MIN_SIZE);
+			}*/
+		}
+		
 		performArrange();
 		emit(SIGNAL(onOpen));
 		if (opened)
@@ -453,18 +474,32 @@ void gMainWindow::setVisible(bool vl)
 		{
 			if (!_title || !*_title)
 				gtk_window_set_title(GTK_WINDOW(border), gApplication::defaultTitle());
-
+			
 			if (!_xembed)
 			{
 				gtk_window_group_add_window(gApplication::currentGroup(), GTK_WINDOW(border));
-
 				active = gDesktop::activeWindow();
-				if (active && active != this)
-					gtk_window_set_transient_for(GTK_WINDOW(border), GTK_WINDOW(active->border));
+				if (isModal() && active && active != this)
+					gtk_window_set_transient_for(GTK_WINDOW(border), GTK_WINDOW(active->topLevel()->border));
 			}
 			
+			// Thanks for Ubuntu's GTK+ patching :-(
+			#if GTK_CHECK_VERSION(3,0,0)
+			gtk_window_set_has_resize_grip(GTK_WINDOW(border), false);
+			#else
+			if (g_object_class_find_property(G_OBJECT_GET_CLASS(border), "has-resize-grip"))
+				g_object_set(G_OBJECT(border), "has-resize-grip", false, (char *)NULL);
+			#endif
+				
 			gtk_window_move(GTK_WINDOW(border), bufX, bufY);
-			gtk_window_present(GTK_WINDOW(border));
+			if (isPopup())
+			{
+				gtk_widget_show_now(border);
+				gtk_widget_grab_focus(border);
+			}
+			else
+				gtk_window_present(GTK_WINDOW(border));
+			//gtk_window_move(GTK_WINDOW(border), bufX, bufY);
 		}
 		else 
 		{
@@ -476,6 +511,7 @@ void gMainWindow::setVisible(bool vl)
 		
 		if (focus)
 		{
+			//fprintf(stderr, "focus = %s\n", focus->name());
 			focus->setFocus();
 			focus = 0;
 		}
@@ -486,46 +522,61 @@ void gMainWindow::setVisible(bool vl)
 	else
 	{
 		if (this == _active)
-			focus = gDesktop::activeControl();
+			focus = gApplication::activeControl();
 			
 		_not_spontaneous = visible;
 		_hidden = true;
 		gControl::setVisible(false);
+		
+		if (_popup)
+			gApplication::exitLoop(this);
 	}
 }
 
 
 void gMainWindow::setMinimized(bool vl)
 {
+	if (!isTopLevel()) return;
 	if (vl) gtk_window_iconify(GTK_WINDOW(border));
 	else    gtk_window_deiconify(GTK_WINDOW(border));
 }
 
 void gMainWindow::setMaximized(bool vl)
 {
+	if (!isTopLevel()) return;
 	if (vl) gtk_window_maximize(GTK_WINDOW(border));
 	else    gtk_window_unmaximize(GTK_WINDOW(border));
 }
 
 void gMainWindow::setFullscreen(bool vl)
 {
+	if (!isTopLevel()) return;
 	if (vl) gtk_window_fullscreen(GTK_WINDOW(border));
 	else    gtk_window_unfullscreen(GTK_WINDOW(border));
 }
 
 bool gMainWindow::minimized()
 {
-	return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_ICONIFIED);
+	if (isTopLevel())
+		return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_ICONIFIED);
+	else
+		return false;
 }
 
 bool gMainWindow::maximized()
 {
-	return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_MAXIMIZED);
+	if (isTopLevel())
+		return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_MAXIMIZED);
+	else
+		return false;
 }
 
 bool gMainWindow::fullscreen()
 {
-	return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_FULLSCREEN);
+	if (isTopLevel())
+		return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_FULLSCREEN);
+	else
+		return false;
 }
 
 void gMainWindow::center()
@@ -534,17 +585,17 @@ void gMainWindow::center()
 	
 	if (!isTopLevel()) return;
 	
-	myx=(gDesktop::width()/2)-(width()/2);
-	myy=(gDesktop::height()/2)-(height()/2);
+	myx = (gDesktop::width() - width()) / 2;
+	myy = (gDesktop::height() - height()) / 2;
 	
-	move(myx,myy);
+	move(myx, myy);
 }
 
-bool gMainWindow::modal()
+bool gMainWindow::isModal() const
 {
 	if (!isTopLevel()) return false;
 
-	return gtk_window_get_modal (GTK_WINDOW(border));
+	return gtk_window_get_modal(GTK_WINDOW(border));
 }
 
 void gMainWindow::showModal()
@@ -552,7 +603,7 @@ void gMainWindow::showModal()
   gMainWindow *save;
 	
 	if (!isTopLevel()) return;
-	if (modal()) return;
+	if (isModal()) return;
 	
 	save = _current;
 	_current = this;
@@ -578,6 +629,68 @@ void gMainWindow::showModal()
 		hide();
 }
 
+void gMainWindow::showPopup(int x, int y)
+{
+  gMainWindow *save;
+	bool has_border;
+	int oldx, oldy;
+	//int type;
+	
+	if (!isTopLevel()) return;
+	if (isModal()) return;
+	
+	//gtk_widget_unrealize(border);
+	//((GtkWindow *)border)->type = GTK_WINDOW_POPUP;
+	//gtk_widget_realize(border);
+
+	oldx = left();
+	oldy = top();
+	
+	has_border = gtk_window_get_decorated(GTK_WINDOW(border));
+	//type = getType();
+	
+	//setType(_NET_WM_WINDOW_TYPE_COMBO);
+	gtk_window_set_decorated(GTK_WINDOW(border), false);
+	//gtk_window_set_type_hint(GTK_WINDOW(border), GDK_WINDOW_TYPE_HINT_POPUP_MENU);
+	
+  move(x, y);
+	gtk_window_resize(GTK_WINDOW(border), bufW, bufH);
+	
+	//reparent(NULL, x, y, GTK_WINDOW_POPUP);
+
+	_popup = true;
+	save = _current;
+	_current = this;
+	
+	gApplication::enterPopup(this);
+	
+	_current = save;
+	_popup = false;
+	
+	if (!persistent)
+	{
+		destroyNow();
+	}
+	else
+	{
+		hide();
+		
+		//gdk_window_set_override_redirect(gtk_widget_get_window(GTK_WINDOW(border)), false);
+		gtk_window_set_decorated(GTK_WINDOW(border), has_border);
+		//setType(type);
+		//gtk_window_set_type_hint(GTK_WINDOW(border), type);
+		
+		move(oldx, oldy);
+	}
+}
+
+void gMainWindow::showPopup()
+{
+	int x, y;
+	gMouse::getScreenPos(&x, &y);
+	showPopup(x, y);
+}
+
 void gMainWindow::raise()
 {
 	if (!isTopLevel()) { gControl::raise(); return; }
@@ -607,43 +720,50 @@ void gMainWindow::setText(const char *txt)
 		gtk_window_set_title(GTK_WINDOW(border), txt);
 }
 
-int gMainWindow::getBorder()
+bool gMainWindow::hasBorder()
 {
-	if (!isTopLevel()) return 0;
-	
-	if (!gtk_window_get_decorated(GTK_WINDOW(border))) return 0;
-	if (!gtk_window_get_resizable(GTK_WINDOW(border))) return 1;
-	return 2;
+	if (isTopLevel())
+		return gtk_window_get_decorated(GTK_WINDOW(border));
+	else
+		return false;
 }
 
-void gMainWindow::setBorder(int b)
+bool gMainWindow::isResizable()
 {
-  if (!isTopLevel()) return;
-	 
-	switch (b)
-	{
-		case 0: // None
-			gtk_window_set_decorated(GTK_WINDOW(border),false);
-			break;
-		case 1: // Fixed
-			gtk_window_set_decorated(GTK_WINDOW(border),true);
-			if (gtk_window_get_resizable(GTK_WINDOW(border)))
-			{
-				gtk_window_set_resizable(GTK_WINDOW(border),false);
-				gtk_widget_set_size_request(border,bufW,bufH);
-			}
-			break;
-		case 2: // Resizable
-			gtk_window_set_decorated(GTK_WINDOW(border),true);
-			if (!gtk_window_get_resizable(GTK_WINDOW(border)))
-			{
-				gtk_window_set_resizable(GTK_WINDOW(border),true);
-				gtk_widget_set_size_request(border,1,1);
-			}
-			break;
-	}
+	if (isTopLevel())
+		return gtk_window_get_resizable(GTK_WINDOW(border));
+	else
+		return false;
 }
 
+void gMainWindow::setBorder(bool b)
+{
+  if (!isTopLevel()) 
+  	return;
+	
+	gtk_window_set_decorated(GTK_WINDOW(border), b);
+}
+
+void gMainWindow::setResizable(bool b)
+{
+  if (!isTopLevel()) 
+  	return;
+	
+	if (b == isResizable())
+		return;
+	
+	//fprintf(stderr, "setResizable: %d %d %d\n", b, bufW, bufH);
+	
+	if (b)
+		gtk_widget_set_size_request(border, 1, 1);
+	else
+		gtk_widget_set_size_request(border, bufW, bufH);
+
+	gtk_window_set_resizable(GTK_WINDOW(border), b);
+}
+
+
+
 void gMainWindow::setSkipTaskBar(bool b)
 {
 	_skip_taskbar = b;
@@ -657,7 +777,7 @@ void gMainWindow::setSkipTaskBar(bool b)
 	GdkPixbuf *buf;
 	gPicture *pic;
 	
-	if (parent()) return NULL;
+	if (!isTopLevel()) return NULL;
 	
 	buf=gtk_window_get_icon(GTK_WINDOW(border));
 	if (!buf) return NULL;
@@ -718,6 +838,7 @@ void gMainWindow::remap()
 	if (top_only) { setTopOnly(false); setTopOnly(true); }
 	if (sticky) { setSticky(false); setSticky(true); }
 	if (stack) { setStacking(0); setStacking(stack); }
+	X11_set_window_type(handle(), _type);
 }
 
 void gMainWindow::drawMask()
@@ -771,9 +892,9 @@ void gMainWindow::drawMask()
 		gtk_widget_realize(border);
 		gtk_widget_realize(widget);
 		gdk_window_set_back_pixmap(border->window, back, FALSE);
-		gdk_window_set_back_pixmap(GTK_LAYOUT(widget)->bin_window, back, FALSE);
+		//gdk_window_set_back_pixmap(GTK_LAYOUT(widget)->bin_window, back, FALSE);
 		gdk_window_clear(border->window);
-		gdk_window_clear(GTK_LAYOUT(widget)->bin_window);
+		//gdk_window_clear(GTK_LAYOUT(widget)->bin_window);
 		gtk_widget_set_app_paintable(border, true);
 		gtk_widget_set_app_paintable(widget, true);
 	}
@@ -871,12 +992,12 @@ bool gMainWindow::doClose()
 {
 	if (_closing)
 		return false;
-
+	
 	if (opened)
 	{
-		if (modal() && !gApplication::hasLoop(this))
+		if (isModal() && !gApplication::hasLoop(this))
 			return true;
-
+		
 		_closing = true;
 		if (onClose) 
 		{
@@ -887,13 +1008,16 @@ bool gMainWindow::doClose()
 			opened = false;
 		_closing = false;
 		
-		if (modal())
+		if (!opened && isModal())
 			gApplication::exitLoop(this);
   }
   
   if (!opened) // && !modal())
   {
-  	if (!modal())
+		if (_active == this)
+			setActiveWindow(NULL);
+		
+  	if (!isModal())
   	{
 			if (persistent)
 				hide();
@@ -912,6 +1036,99 @@ bool gMainWindow::close()
 	return doClose();
 }
 
+void gMainWindow::reparent(gContainer *newpr, int x, int y)
+{
+	GtkWidget *new_border;
+	int w, h;
+	gColor fg, bg;
+	int i;
+	gControl *child;
+	
+	if (_xembed)
+		return;
+	
+	bg = background();
+	fg = foreground();
+	
+	if (isTopLevel() && newpr)
+	{
+		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
+		
+		new_border = gtk_event_box_new();
+		gtk_widget_reparent(widget, new_border);
+		embedMenuBar(new_border);
+		_no_delete = true;
+		gtk_widget_destroy(border);
+		_no_delete = false;
+		
+		border = new_border;
+		registerControl();
+		
+		setParent(newpr);
+		connectParent();
+		borderSignals();
+		initWindow();	
+		
+		setBackground(bg);
+		setForeground(fg);
+		setFont(font());
+		
+		checkMenuBar();
+		
+		move(x, y);
+		gtk_widget_set_size_request(border, width(), height());
+		
+		// Hidden children are incorrectly shown. Fix that!
+		for (i = 0;; i++)
+		{
+			child = getControl(i);
+			if (!child)
+				break;
+			if (!child->isVisible())
+				child->setVisible(false);
+		}
+	}
+	else if ((!isTopLevel() && !newpr)
+	         || (isTopLevel() && isPopup()))
+	         //|| (isTopLevel() && (isPopup() ^ (type == GTK_WINDOW_POPUP))))
+	{
+		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
+		// TODO: test that
+		new_border = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_widget_reparent(widget, new_border);
+		embedMenuBar(new_border);
+		_no_delete = true;
+		gtk_widget_destroy(border);
+		_no_delete = false;
+
+		border = new_border;
+		registerControl();
+		
+		if (parent())
+		{
+			parent()->remove(this);
+			parent()->arrange();
+			setParent(NULL);
+		}
+		initWindow();	
+		borderSignals();
+		setBackground(bg);
+		setForeground(fg);
+		setFont(font());
+		
+		move(x, y);
+		w = width();
+		h = height();
+		bufW = bufH = -1;
+		resize(w, h);
+		
+		_popup = false; //type == GTK_WINDOW_POPUP;
+	}
+	else
+		gControl::reparent(newpr, x, y);	
+}
+
+
 int gMainWindow::controlCount()
 {
 	GList *list = gControl::controlList();
@@ -972,6 +1189,11 @@ int gMainWindow::clientX()
 	return 0;
 }
 
+int gMainWindow::containerX()
+{
+	return 0;
+}
+
 int gMainWindow::clientY()
 {
 	GtkRequisition req;
@@ -985,23 +1207,39 @@ int gMainWindow::clientY()
 	return 0;
 }
 
+int gMainWindow::containerY()
+{
+	return 0;
+}
+
+
 int gMainWindow::clientWidth()
 {
 	return width();
 }
 
-int gMainWindow::clientHeight()
+
+int gMainWindow::menuBarHeight()
 {
-	GtkRequisition req;
+	GtkRequisition req = { 0, 0 };
 	int h = 0;
 
-	if (menuBar && GTK_WIDGET_VISIBLE(GTK_WIDGET(menuBar)))
+	if (menuBar)
 	{
+		gtk_widget_show(GTK_WIDGET(menuBar));
 		gtk_widget_size_request(GTK_WIDGET(menuBar), &req);
 		h = req.height;
 	}
 	
-	return height() - h;
+	return h;
+}
+
+int gMainWindow::clientHeight()
+{
+	if (isMenuBarVisible())
+		return height() - menuBarHeight();
+	else
+		return height();
 }
 
 void gMainWindow::setActiveWindow(gControl *control)
@@ -1014,6 +1252,8 @@ void gMainWindow::setActiveWindow(gControl *control)
 		
 	_active = window;
 	
+	//fprintf(stderr, "setActiveWindow: %p %s\n", _active, _active ? _active->name() : "");
+	
 	if (old)
 		old->emit(SIGNAL(old->onDeactivate));
 		
@@ -1021,87 +1261,153 @@ void gMainWindow::setActiveWindow(gControl *control)
 		window->emit(SIGNAL(window->onActivate));
 }
 
-void gMainWindow::setToolBox(bool vl)
+#ifdef GDK_WINDOWING_X11
+int gMainWindow::getType()
 {
-	_toolbox = vl;
-	
-	// Should be replaced by a "Type" property that tells which is the type of the window
-	// see gdk_window_set_type_hint()
-	
-	stub("gMainWindow::setToolBox");
+	if (!isTopLevel())
+		return 0;
+	return _type; //X11_get_window_type(handle());
 }
 
-void gMainWindow::reparent(gContainer *newpr, int x, int y)
+void gMainWindow::setType(int type)
 {
-	GtkWidget *new_border;
-	int w, h;
-	gColor fg, bg;
+	if (!isTopLevel())
+		return;
+	X11_set_window_type(handle(), type);
+	_type = type;
+}
+#else
+int gMainWindow::getType()
+{
+	return 0;
+}
+
+void gMainWindow::setType()
+{
+}
+#endif
+
+void gMainWindow::configure()
+{
+	int h;
 	
-	if (_xembed)
+	if (bufW < 1 || bufH < 1)
 		return;
 	
-	bg = background();
-	fg = foreground();
+	h = menuBarHeight();
 	
-	if (isTopLevel() && newpr)
+	if (isMenuBarVisible())
 	{
-		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
-		
-		new_border = gtk_event_box_new();
-		gtk_widget_reparent(widget, new_border);
-		gMenu::embedMenuBar(this, new_border);
-		_no_delete = true;
-		gtk_widget_destroy(border);
-		_no_delete = false;
-		
-		border = new_border;
-		registerControl();
-		
-		setParent(newpr);
-		connectParent();
-		borderSignals();
-		initWindow();	
-		
-		setBackground(bg);
-		setForeground(fg);
-		setFont(font());
-		
-		gMenu::checkMenuBar(this);
-		
-		move(x, y);
-		gtk_widget_set_size_request(border, width(), height());
-	}
-	else if (!isTopLevel() && !newpr)
-	{
-		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
-		// TODO: test that
-		new_border = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-		gtk_widget_reparent(widget, new_border);
-		gMenu::embedMenuBar(this, new_border);
-		_no_delete = true;
-		gtk_widget_destroy(border);
-		_no_delete = false;
-		
-		border = new_border;
-		registerControl();
-		
-		parent()->remove(this);
-		parent()->arrange();
-
-		setParent(NULL);
-		borderSignals();
-		initWindow();	
-		
-		setBackground(bg);
-		setForeground(fg);
-		setFont(font());
-		
-		move(x, y);
-		w = width();
-		h = height();
-		bufW = bufH = -1;
-		resize(w, h);
+		gtk_fixed_move(layout, GTK_WIDGET(menuBar), 0, 0);
+		if (h > 1)
+			gtk_widget_set_size_request(GTK_WIDGET(menuBar), width(), h);
+		gtk_fixed_move(layout, widget, 0, h);
+		gtk_widget_set_size_request(widget, width(), height() - h);
 	}
 	else
-		gControl::reparent(newpr, x, y);	
+	{
+		if (layout)
+		{
+			if (menuBar)
+				gtk_fixed_move(layout, GTK_WIDGET(menuBar), 0, -h);
+			gtk_fixed_move(layout, widget, 0, 0);
+			gtk_widget_set_size_request(widget, width(), height());
+		}
+	}
+}
+
+void gMainWindow::setMenuBarVisible(bool v)
+{
+	_showMenuBar = v;
+	
+	if (!menuBar)
+		return;
+	
+	configure();
+	performArrange();
+}
+
+bool gMainWindow::isMenuBarVisible()
+{
+	return menuBar && !_hideMenuBar && _showMenuBar; //|| (menuBar && GTK_WIDGET_MAPPED(GTK_WIDGET(menuBar)));
+}
+
+void gMainWindow::setFont(gFont *ft)
+{
+	gContainer::setFont(ft);
+	gMenu::updateFont(this);
+}
+
+void gMainWindow::checkMenuBar()
+{
+	int i;
+	gMenu *menu;
+
+	if (menuBar)
+	{
+		_hideMenuBar = true;
+		for (i = 0;; i++)
+		{
+			menu = gMenu::winChildMenu(this, i);
+			if (!menu)
+				break;
+			if (menu->isVisible() && !menu->isSeparator())
+			{
+				_hideMenuBar = false;
+				break;
+			}
+		}
+	}
+		
+	configure();
+}
+
+void gMainWindow::embedMenuBar(GtkWidget *border)
+{
+	if (menuBar)
+	{
+		// layout is automatically destructed ?
+		layout = GTK_FIXED(gtk_fixed_new());
+
+		g_object_ref(G_OBJECT(menuBar));
+		
+		if (gtk_widget_get_parent(GTK_WIDGET(menuBar)))
+			gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(GTK_WIDGET(menuBar))), GTK_WIDGET(menuBar));
+	
+		gtk_fixed_put(layout, GTK_WIDGET(menuBar), 0, 0);
+
+		g_object_unref(G_OBJECT(menuBar));
+	
+		gtk_widget_reparent(widget, GTK_WIDGET(layout));
+		gtk_container_add(GTK_CONTAINER(border), GTK_WIDGET(layout));
+	
+		gtk_widget_show(GTK_WIDGET(menuBar));
+		gtk_widget_show(GTK_WIDGET(layout));
+		gtk_widget_show(GTK_WIDGET(widget));
+		
+		gMenu::updateFont(this);
+		gMenu::updateColor(this);
+		
+		checkMenuBar();
+	}
+}
+
+double gMainWindow::opacity()
+{
+	if (isTopLevel())
+		return gtk_window_get_opacity(GTK_WINDOW(border));
+	else
+		return 1.0;
+}
+
+void gMainWindow::setOpacity(double v)
+{
+	if (isTopLevel())
+		gtk_window_set_opacity(GTK_WINDOW(border), v);
+}
+
+int gMainWindow::screen()
+{
+	gMainWindow *tl = topLevel();
+	return gdk_screen_get_monitor_at_window(gtk_window_get_screen(GTK_WINDOW(tl->border)), tl->border->window);
 }
