@@ -4,15 +4,9 @@
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
 
-  GTK+ component
-
-  Realizado para la Junta de Extremadura.
-  Consejería de Educación Ciencia y Tecnología.
-  Proyecto gnuLinEx
-
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -32,9 +27,11 @@
 
 #include "gmemory.h"
 #include "main.h"
+#include "gb.image.h"
 #include "gb.gtk.h"
 #include "watcher.h"
 
+#include "x11.h"
 #include "CScreen.h"
 #include "CDraw.h"
 #include "CConst.h"
@@ -42,6 +39,7 @@
 #include "CFont.h"
 #include "CKey.h"
 #include "CPicture.h"
+#include "CImage.h"
 #include "CClipboard.h"
 #include "CMouse.h"
 #include "CMessage.h"
@@ -68,15 +66,21 @@
 #include "CSpinBox.h"
 #include "CSplitter.h"
 #include "CStock.h"
-#include "CIconView.h"
-#include "CGridView.h"
 #include "CSeparator.h"
+#include "cprinter.h"
+#include "csvgimage.h"
 
 #include <gtk/gtk.h>
 #include <string.h>
 
-static int hook_image(CIMAGE **pimage, GB_IMAGE_INFO *info);
-static int hook_picture(CPICTURE **ppicture, GB_PICTURE_INFO *info);
+GB_CLASS CLASS_Picture;
+GB_CLASS CLASS_Image;
+GB_CLASS CLASS_DrawingArea;
+GB_CLASS CLASS_Menu;
+GB_CLASS CLASS_Window;
+GB_CLASS CLASS_Printer;
+GB_CLASS CLASS_SvgImage;
+
 static void my_lang(char *lang,int rtl1);
 static void my_error(int code,char *error,char *where);
 static void my_quit (void);
@@ -86,7 +90,9 @@ static void my_wait(int duration);
 static void my_post(void);
 static int my_loop();
 static void my_watch(int fd, int type, void *callback, intptr_t param);
-static bool post_Check=false;
+
+static bool _post_check = false;
+static bool _must_check_quit = false;
 
 static bool _application_keypress = false;
 static GB_FUNCTION _application_keypress_func;
@@ -96,12 +102,15 @@ int MAIN_scale = 0;
 extern "C"
 {
 	GB_INTERFACE GB EXPORT;
+	IMAGE_INTERFACE IMAGE EXPORT;
 
 	GB_DESC *GB_CLASSES[] EXPORT =
 	{
-		CDesktopDesc,
-		CApplicationTooltipDesc,
-		CApplicationDesc,
+		ScreenDesc,
+		ScreensDesc,
+		DesktopDesc,
+		ApplicationDesc,
+		StyleDesc,
 		CSelectDesc,
 		CAlignDesc,
 		CArrangeDesc,
@@ -110,7 +119,6 @@ extern "C"
 		CLineDesc,
 		CFillDesc,
 		CColorDesc,
-		CColorInfoDesc,
 		CFontsDesc,
 		CFontDesc,
 		CKeyDesc,
@@ -170,8 +178,6 @@ extern "C"
 		CListViewDesc,
 		CTreeViewItemDesc,
 		CTreeViewDesc,
-		CIconViewItemDesc,
-		CIconViewDesc,
 		CColumnViewItemDesc,
 		CColumnViewColumnDesc,
 		CColumnViewColumnsDesc,
@@ -183,26 +189,18 @@ extern "C"
 		CScrollViewDesc,
 		CSpinBoxDesc,
 		CSeparatorDesc,
-		CGridViewItemDesc,
-		CGridViewDataDesc,
-		CGridViewColumnDesc,
-		CGridViewRowDesc,
-		CGridViewColumnsDesc,
-		CGridViewRowsDesc,
-		CGridViewDesc,
 		CStockDesc,
-
+		PrinterDesc,
+		SvgImageDesc,
 		NULL
 	};
 
 	void *GB_GTK_1[] EXPORT =
 	{
 		(void *)GTK_INTERFACE_VERSION,
+		(void *)my_main,
 		(void *)GTK_GetPicture,
 		(void *)GTK_GetImage,
-		(void *)CGRIDVIEW_footer,
-		(void *)CGRIDVIEW_column_footer_text,
-		(void *)CGRIDVIEW_columns_get,
 		(void *)DRAW_get_drawable,
 		(void *)DRAW_get_style,
 		(void *)DRAW_get_state,
@@ -225,19 +223,33 @@ extern "C"
 		GB.Hook(GB_HOOK_POST,(void*)my_post);
 		GB.Hook(GB_HOOK_ERROR,(void*)my_error);
 		GB.Hook(GB_HOOK_LANG,(void*)my_lang);
-		GB.Hook(GB_HOOK_IMAGE, (void *)hook_image);
-    GB.Hook(GB_HOOK_PICTURE, (void *)hook_picture);
 
-		GB.LoadComponent("gb.draw");
-		
 		// Thanks again to GTK+ 2.18 :-(
 		#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 18
-		putenv((char *)"GDK_NATIVE_WINDOWS=1");
+		//putenv((char *)"GDK_NATIVE_WINDOWS=1");
 		#endif
 		
+		GB.LoadComponent("gb.draw");
+		GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
+		IMAGE.SetDefaultFormat(GB_IMAGE_RGBA);
+		DRAW_init();
+		
 		CWatcher::init();
 
-		return TRUE;
+		//CLASS_Control = GB.FindClass("Control");
+		//CLASS_Container = GB.FindClass("Container");
+		//CLASS_UserControl = GB.FindClass("UserControl");
+		//CLASS_UserContainer = GB.FindClass("UserContainer");
+		CLASS_Window = GB.FindClass("Window");
+		CLASS_Menu = GB.FindClass("Menu");
+		CLASS_Picture = GB.FindClass("Picture");
+		//CLASS_Drawing = GB.FindClass("Drawing");
+		CLASS_DrawingArea = GB.FindClass("DrawingArea");
+		CLASS_Printer = GB.FindClass("Printer");
+		CLASS_Image = GB.FindClass("Image");
+		CLASS_SvgImage = GB.FindClass("SvgImage");
+		
+		return -1;
 	}
 
 	void EXPORT GB_EXIT()
@@ -288,7 +300,7 @@ extern "C"
 
 void my_quit (void)
 {
-	CWINDOW *win=WINDOW_get_main();
+	CWINDOW *win = CWINDOW_Main;
 
 	gApplication::suspendEvents(false);
 	if (win)
@@ -308,16 +320,26 @@ static bool global_key_event_handler(int type)
 
 static void my_main(int *argc, char **argv)
 {
+	static bool init = false;
+	
+	if (init)
+		return;
+	
 	gApplication::init(argc, &argv);
 	gApplication::setDefaultTitle(GB.Application.Title());
 	gDesktop::init();
 	MAIN_scale = gDesktop::scale();
+	#ifdef GDK_WINDOWING_X11
+  	X11_init(gdk_x11_display_get_xdisplay(gdk_display_get_default()), gdk_x11_get_default_root_xwindow());
+  #endif
 
-	if (GB.GetFunction(&_application_keypress_func, (void *)GB.FindClass(GB.Application.Startup()), "Application_KeyPress", "", "") == 0)
+	if (GB.GetFunction(&_application_keypress_func, (void *)GB.Application.StartupClass(), "Application_KeyPress", "", "") == 0)
 	{
 		_application_keypress = true;
 		gApplication::onKeyEvent = global_key_event_handler;
 	}
+
+	init = true;
 }
 
 /*static void raise_timer(GB_TIMER *timer)
@@ -381,24 +403,36 @@ static void my_timer(GB_TIMER *timer,bool on)
 
 static void my_post(void)
 {
-	post_Check=true;
+	_post_check = true;
 }
 
-static bool must_quit(void)
+void MAIN_check_quit()
 {
-	//fprintf(stderr, "must_quit: %p %d\n", WINDOW_get_main(), CWatcher::count());
-  return !WINDOW_get_main() && CWatcher::count() == 0; // && in_event_loop;
+	_must_check_quit = true;
 }
 
 static int my_loop()
 {
 	gControl::cleanRemovedControls();
+	_must_check_quit = true;
+	
+	for(;;)
+	{
+		if (_must_check_quit)
+		{
+			if (CWINDOW_must_quit() && CWatcher::count() == 0)
+				break;
+			_must_check_quit = false;
+		}
+		MAIN_do_iteration(false);
+	}
 
-	while (!must_quit())
-		do_iteration(false);
-
-	while (gtk_events_pending ()) 	gtk_main_iteration();
+	while (gtk_events_pending())
+		gtk_main_iteration();
   
+	CWINDOW_delete_all();
+	gControl::cleanRemovedControls();
+
 	CWatcher::Clear();
 	gApplication::exit();
 
@@ -407,7 +441,7 @@ static int my_loop()
 
 static void my_wait(int duration)
 {
-	do_iteration(true, true);
+	MAIN_do_iteration(true, true);
 }
 
 static void my_watch(int fd, int type, void *callback, intptr_t param)
@@ -449,7 +483,13 @@ static void my_lang(char *lang,int rtl)
 	}
 }
 
-void do_iteration(bool do_not_block, bool do_not_sleep)
+void MAIN_do_iteration_just_events()
+{
+	if (gtk_events_pending ())
+		gtk_main_iteration_do (false);
+}
+
+void MAIN_do_iteration(bool do_not_block, bool do_not_sleep)
 {
 	struct timespec mywait;
 
@@ -471,67 +511,11 @@ void do_iteration(bool do_not_block, bool do_not_sleep)
 	
 	gApplication::_loopLevel--;
 
-	if (post_Check) { post_Check=false; GB.CheckPost(); }
+	if (_post_check)
+	{
+		_post_check = false;
+		GB.CheckPost();
+	}
+	
 	gControl::cleanRemovedControls();
 }
-
-// BM: New image & picture hook syntax
-// BM: Conversion functions are now in the interpreter API
-
-static int hook_image(CIMAGE **pimage, GB_IMAGE_INFO *info)
-{
-	CIMAGE *image = *pimage;
-	gPicture *img;
-	const guchar *buf;
-
-	if (!image)
-	{
-		img = new gPicture(gPicture::MEMORY, info->width, info->height, GB_IMAGE_TRANSPARENT(info->format));
-		buf = gdk_pixbuf_get_pixels(img->getPixbuf());
-
-		if (info->data) 
-		  GB.Image.Convert((void *)buf, GB_IMAGE_RGBA, info->data, info->format, info->width, info->height);
-
-    image = CIMAGE_create(img);
-		*pimage = image;
-	}
-	else
-	{
-		info->data = gdk_pixbuf_get_pixels(image->picture->getPixbuf());
-		info->width = image->picture->width();
-		info->height = image->picture->height();
-		info->format = image->picture->isTransparent() ? GB_IMAGE_RGBA : GB_IMAGE_RGBX;
-	}
-
-	return 0;
-}
-
-static int hook_picture(CPICTURE **ppicture, GB_PICTURE_INFO *info)
-{
-	CPICTURE *picture = *ppicture;
-	gPicture *img;
-	void *buf;
-
-	if (!picture)
-	{
-		img = new gPicture(gPicture::MEMORY, info->width, info->height, GB_IMAGE_TRANSPARENT(info->format));
-		buf = gdk_pixbuf_get_pixels(img->getPixbuf());
-
-		if (info->data) 
-		  GB.Image.Convert((void *)buf, GB_IMAGE_RGBA, info->data, info->format, info->width, info->height);
-
-    picture = CPICTURE_create(img);
-		*ppicture = picture;		
-	}
-	else
-	{
-		info->data = NULL;
-		info->width = picture->picture->width();
-		info->height = picture->picture->height();
-		info->format = picture->picture->isTransparent() ? GB_IMAGE_RGBA : GB_IMAGE_RGBX;
-	}
-
-	return 0;
-}
-
-
