@@ -2,14 +2,12 @@
 
   main.c
 
-  The PCRE component
-
   (c) 2004 Rob Kudla <pcre-component@kudla.org>
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,7 +17,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -34,7 +33,6 @@
 
 #include "main.h"
 
-
 GB_INTERFACE GB EXPORT;
 
 GB_DESC *GB_CLASSES[] EXPORT =
@@ -45,17 +43,13 @@ GB_DESC *GB_CLASSES[] EXPORT =
   NULL
 };
 
-
 int EXPORT GB_INIT(void)
 {
-  REGEXP_init();
-
   return 0;
 }
 
 
 void EXPORT GB_EXIT()
 {
-  REGEXP_exit();
 }
 
