@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  output.h
+  gbc_output.h
 
-  The object file creator
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -44,9 +43,9 @@ typedef
 
 #define OUTPUT_BUFFER_SIZE 16384
 
-PUBLIC void OUTPUT_do(bool swap);
-PUBLIC const char *OUTPUT_get_file(const char *file);
-PUBLIC const char *OUTPUT_get_trans_file(const char *file);
+void OUTPUT_do(bool swap);
+char *OUTPUT_get_file(const char *file);
+char *OUTPUT_get_trans_file(const char *file);
 
 #endif
 
