@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  read.h
+  gbc_read.h
 
-  Lexical parser
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,17 +16,32 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
-#ifndef _READ_H
-#define _READ_H
+#ifndef __GBC_READ_H
+#define __GBC_READ_H
 
 #include "gbc_read_common.h"
 
+#include <ctype.h>
+
+#undef isdigit
+#define isdigit(_c) (READ_digit_car[_c])
+#undef isspace
+#define isspace(_c) (((uchar)_c) <= ' ')
+
+#ifndef __GBC_READ_C
+extern char READ_digit_car[];
+#endif
+
 void READ_do(void);
-PUBLIC void READ_dump_pattern(PATTERN *pattern);
-PUBLIC char *READ_get_pattern(PATTERN *pattern);
+void READ_dump_pattern(PATTERN *pattern);
+char *READ_get_pattern(PATTERN *pattern);
+int READ_get_column();
+
+void THROW_UNEXPECTED(PATTERN *pattern);
 
 #endif
