@@ -1,12 +1,12 @@
 /***************************************************************************
 
-  CGambas.h
+  gbx_c_gambas.h
 
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -16,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -25,26 +26,10 @@
 
 #include "gambas.h"
 #include "gbx_object.h"
-#include "gb_list.h"
 
 #ifndef __GBX_C_GAMBAS_C
 extern GB_DESC NATIVE_Gambas[];
 extern GB_DESC NATIVE_Param[];
-extern GB_DESC NATIVE_Observer[];
 #endif
 
-typedef
-	struct {
-		OBJECT ob;
-		LIST list;
-		ushort *event;
-		void *object;
-		bool after;
-		char _reserved[3];
-	}
-	COBSERVER;
-
-void COBSERVER_attach(COBSERVER *this, void *parent, const char *name);
-void COBSERVER_detach(COBSERVER *this);
-
 #endif
