@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  class_load.c
+  gbx_class_load.c
 
-  Gambas class loader
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -41,20 +40,20 @@
 #include "gbx_debug.h"
 #include "gb_magic.h"
 #include "gbx_stream.h"
-
+#include "gbx_c_array.h"
 #include "gbx_string.h"
 #include "gbx_object.h"
 #include "gbx_variant.h"
 #include "gbx_number.h"
+#include "gbx_struct.h"
 
 #include "gambas.h"
 
 #include "gbx_class.h"
 
-//#define DEBUG
+//#define DEBUG DEBUG
 //#define DEBUG_LOAD 1
-
-//long total = 0;
+//#define DEBUG_STRUCT 1
 
 static const char *ClassName;
 static bool _swap;
@@ -76,16 +75,102 @@ static void SWAP_type(CTYPE *p)
 	SWAP_short(&p->value);
 }
 
+
 static int align_pos(int pos, int size)
 {
-	if (size >= 4)
-		return (pos + 3) & ~3;
-	else if (size == 2)
-		return (pos + 1) & ~1;
-	else
-		return pos;
+	switch (size)
+	{
+		case 1:
+			return pos;
+		case 2:
+			return (pos + 1) & ~1;
+		#ifndef OS_64BITS
+		case 4:
+		default:
+			return (pos + 3) & ~3;
+		#else
+		case 4:
+			return (pos + 3) & ~3;
+		case 8:
+		default:
+			return (pos + 7) & ~7;
+		#endif
+	}
 }
 
+
+int CLASS_sizeof_ctype(CLASS *class, CTYPE ctype)
+{
+	size_t size;
+	
+	if (ctype.id == TC_ARRAY)
+	{
+		CLASS_ARRAY *array = class->load->array[ctype.value];
+		size = CARRAY_get_static_size(class, array);
+		return (size + 3) & ~3;
+	}
+	else if (ctype.id == TC_STRUCT)
+	{
+		return CSTRUCT_get_size(class->load->class_ref[ctype.value]);
+	}
+	else
+		return TYPE_sizeof_memory(ctype.id);
+}
+
+TYPE CLASS_ctype_to_type(CLASS *class, CTYPE ctype)
+{
+  if (ctype.id == T_OBJECT && ctype.value >= 0)
+    return (TYPE)(class->load->class_ref[ctype.value]);
+	else if (ctype.id == TC_ARRAY)
+		return (TYPE)CARRAY_get_array_class(class, class->load->array[ctype.value]->ctype);
+  else if (ctype.id == TC_STRUCT)
+    return (TYPE)(class->load->class_ref[ctype.value]);
+  else
+    return (TYPE)(ctype.id);	
+}
+
+
+static void conv_ctype(CTYPE *ctype)
+{
+	//if (ctype->id == TC_POINTER)
+	//	ctype->id = T_POINTER;
+}
+
+
+static void conv_type(CLASS *class, void *ptype)
+{
+  CTYPE ctype = *(CTYPE *)ptype;
+
+	if (_swap)
+	{
+		SWAP_int((int *)&ctype);
+		SWAP_type(&ctype);
+	}
+
+  *((TYPE *)ptype) = CLASS_ctype_to_type(class, ctype);
+}
+
+
+static void conv_type_simple(CLASS *class, int *ptype)
+{
+  CTYPE ctype = *(CTYPE *)ptype;
+
+	if (_swap)
+		SWAP_int((int *)&ctype);
+
+  *ptype = ctype.id;
+}
+
+
+static void check_version(CLASS *class, int loaded)
+{
+  if (loaded > GAMBAS_PCODE_VERSION)
+    THROW(E_CLASS, ClassName, "Bytecode too recent. Please upgrade Gambas.", "");
+  if (loaded < GAMBAS_PCODE_VERSION_MIN)
+    THROW(E_CLASS, ClassName, "Bytecode too old. Please recompile the project.", "");
+}
+
+
 static char *get_section(char *sec_name, char **section, short *pcount, const char *desc)
 {
 	static void *jump_swap[] = { &&__SWAP_END, &&__SWAP_BYTE, &&__SWAP_SHORT, &&__SWAP_INT, &&__SWAP_POINTER, &&__SWAP_CTYPE, &&__SWAP_TYPE };
@@ -113,7 +198,7 @@ static char *get_section(char *sec_name, char **section, short *pcount, const ch
 
   #ifdef DEBUG
   NSection++;
-  printf("Section #%d %s %08lX %d %d\n", NSection + 1, sec_name, (int)(current - (char *)Class->data), one, section_size);
+  fprintf(stderr, "Section #%d %s %08lX %d %d\n", NSection + 1, sec_name, (int)(current - (char *)Class->data), (int)size_one, (int)section_size);
   #endif
 
   *section += section_size + sizeof(int);
@@ -178,7 +263,7 @@ static char *get_section(char *sec_name, char **section, short *pcount, const ch
 		
 		if (size_one_64 != size_one)
 		{
-			ALLOC(POINTER(&alloc), size_one_64 * size, "get_section");
+			ALLOC(&alloc, size_one_64 * size, "get_section");
 				
 			for (i = 0; i < size; i++)
 			{
@@ -232,161 +317,164 @@ static char *get_section(char *sec_name, char **section, short *pcount, const ch
 	}
 	
 	return current;
-	
-		#if 0
-		for (i = 0; i < size; i++)
-		{
-			p = current + i * one;
-
-			ps = swapcode;
-			while (*ps)
-			{
-				if (*ps == 's')
-				{
-					func = SWAP_short;
-					ss = sizeof(short);
-				}
-				else if (*ps == 'i')
-				{
-					func = SWAP_int;
-					ss = sizeof(int);
-				}
-				else if (*ps == 't')
-				{
-					func = SWAP_type;
-					ss = sizeof(int);
-				}
-				else
-				{
-					func = NULL;
-					ss = 1;
-				}
-
-				ps++;
-
-				if (isdigit(*ps))
-				{
-					n = *ps - '0';
-					ps++;
-				}
-				else if (*ps == '*')
-				{
-					n = one / ss;
-					ps++;
-				}
-				else
-					n = 1;
-
-				if (func)
-				{
-					while (n)
-					{
-						(*func)(p);
-						p += ss;
-						n--;
-					}
-				}
-				else
-					p += n;
-			}
-    }
-    #endif
-
-}
-
-TYPE CLASS_ctype_to_type(CLASS *class, CTYPE ctype)
-{
-  if (ctype.id == T_OBJECT && ctype.value >= 0)
-    return (TYPE)(class->load->class_ref[ctype.value]);
-  else if (ctype.id == T_ARRAY)
-    ERROR_panic("conv_type: bad type");
-  else
-    return (TYPE)(ctype.id);	
-}
-
-static void conv_type(CLASS *class, void *ptype)
-{
-  CTYPE ctype = *(CTYPE *)ptype;
-  TYPE type;
-
-	if (_swap)
-	{
-		SWAP_int((int *)&ctype);
-		SWAP_type(&ctype);
-	}
-
-  if (ctype.id == T_OBJECT && ctype.value >= 0)
-    type = (TYPE)(class->load->class_ref[ctype.value]);
-  else if (ctype.id == T_ARRAY)
-    ERROR_panic("conv_type: bad type");
-  else
-    type = (TYPE)(ctype.id);
-
-  *((TYPE *)ptype) = type;
-}
-
-static void conv_type_simple(CLASS *class, int *ptype)
-{
-  CTYPE ctype = *(CTYPE *)ptype;
-
-	if (_swap)
-		SWAP_int((int *)&ctype);
-
-  *ptype = ctype.id;
-}
-
-
-static void check_version(int loaded)
-{
-	static bool warning = FALSE;
-	
-  int current = GAMBAS_PCODE_VERSION;
-
-  if (GAMBAS_FULL_VERSION > 0x01090000 && GAMBAS_FULL_VERSION <= 0x0200FFFF)
-  {
-    if (loaded == current)
-      return;
-    
-    if ((COMPONENT_current && COMPONENT_current->warning) || warning)
-    	return;
-    
-    fprintf(stderr, "WARNING: ");
-    if (COMPONENT_current)
-    	fprintf(stderr, "%s: ", COMPONENT_current->name);
-    fprintf(stderr, "current bytecode version is ");
-    fprintf(stderr, "%X.%X.%X ", current >> 24, (current >> 16) & 0xFF, current & 0xFFFF);
-    fprintf(stderr, "and project bytecode version is ");
-    fprintf(stderr, "%X.%X.%X. You should recompile your project.\n", loaded >> 24, (loaded >> 16) & 0xFF, loaded & 0xFFFF);
-    
-    if (COMPONENT_current)
-    	COMPONENT_current->warning = TRUE;
-    else
-    	warning = TRUE;
-    	
-    return;
-  }
-  
-  if (loaded > current)
-    THROW(E_CLASS, ClassName, "Bytecode version too recent. Please upgrade Gambas.", "");
-  if (loaded < current)
-    THROW(E_CLASS, ClassName, "Bytecode version too old. Please recompile the project.", "");
 }
 
 #define RELOCATE(_ptr) (_ptr = (char *)&class->string[(int)(intptr_t)(_ptr)])
 
-
-static int sizeof_ctype(CLASS *class, CTYPE ctype)
+static void load_structure(CLASS *class, int *structure, int nfield)
 {
-	size_t size;
+	char *name;
+	char *field;
+	CLASS *sclass;
+	int i, pos, size, len;
+	CTYPE ctype;
+	CLASS_DESC *desc;
+	CLASS_VAR *var;
+	GLOBAL_SYMBOL *global = NULL;
 	
-	if (ctype.id != T_ARRAY)
-		return TYPE_sizeof_memory(ctype.id);
+	name = (char *)(intptr_t)(*structure++);
+	RELOCATE(name);
+	#if DEBUG_STRUCT
+	fprintf(stderr, "Loading structure %s\n", name);
+	#endif
+	
+	if (class->global)
+		sclass = CLASS_find_global(name);
+	else
+		sclass = CLASS_find(name);
+	
+	if (sclass->state)
+	{
+		if (!sclass->is_struct)
+			THROW(E_CLASS, ClassName, "Class already exists: ", name);
 		
-	size = ARRAY_get_size((ARRAY_DESC *)class->load->array[ctype.value]);
-  return (size + 3) & ~3;
+		// Check compatibility with previous declaration
+		
+		if (sclass->load->n_dyn != nfield)
+			goto __MISMATCH;
+		
+		desc = (CLASS_DESC *)sclass->data;
+		
+		for (i = 0; i < nfield; i++)
+		{
+			field = (char *)(intptr_t)(*structure++);
+			RELOCATE(field);
+			len = strlen(field);
+			ctype = *((CTYPE *)structure);
+			structure++;
+			
+			if (CLASS_ctype_to_type(class, ctype) != desc[i].variable.type)
+				goto __MISMATCH;
+			
+			if (TABLE_compare_ignore_case_len(field, strlen(field), sclass->table[i].name, sclass->table[i].len))
+				goto __MISMATCH;
+		}
+		
+		// OK, they are the same!
+		return;
+	}
+	
+	sclass->swap = class->swap;
+	sclass->component = class->component;
+	sclass->debug = class->debug;
+
+	ALLOC_ZERO(&sclass->load, sizeof(CLASS_LOAD), "load_structure");
+	
+	ALLOC(&var, sizeof(CLASS_VAR) * nfield, "load_structure");
+	sclass->load->dyn = var;
+	sclass->load->n_dyn = nfield;
+	sclass->load->class_ref = class->load->class_ref;
+	sclass->load->array = class->load->array;
+	
+	if (sclass->debug)
+	{
+		ALLOC(&global, sizeof(GLOBAL_SYMBOL) * nfield, "load_structure");
+		sclass->load->global = global;
+		sclass->load->n_global = nfield;
+	}
+	
+	sclass->n_desc = nfield;
+	ALLOC(&sclass->table, sizeof(CLASS_DESC_SYMBOL) * nfield, "load_structure");
+	ALLOC(&desc, sizeof(CLASS_DESC) *nfield, "load_structure");
+	sclass->data = (char *)desc;
+
+	pos = 0; //sizeof(CSTRUCT);
+	
+	for (i = 0; i < nfield; i++)
+	{
+		field = (char *)(intptr_t)(*structure++);
+		RELOCATE(field);
+		len = strlen(field);
+		ctype = *((CTYPE *)structure);
+		structure++;
+		
+		desc[i].variable.name = "f";
+		desc[i].variable.type = CLASS_ctype_to_type(class, ctype);
+		desc[i].variable.ctype = ctype;
+		desc[i].variable.offset = pos; // This the position relative to the data, NOT the object!
+		desc[i].variable.class = sclass;
+		
+		var[i].type = ctype;
+
+		size = CLASS_sizeof_ctype(class, ctype);
+		pos = align_pos(pos, size);
+		var[i].pos = pos;
+		
+		if (sclass->debug)
+		{
+			global[i].sym.name = field;
+			global[i].sym.len = len;
+			global[i].ctype = ctype;
+			global[i].value = i;
+		}
+		
+		#if DEBUG_STRUCT
+		fprintf(stderr, "  %d: %s As %s (%d)\n", i, field, TYPE_get_name(desc[i].variable.type), CLASS_sizeof_ctype(class, ctype));
+		#endif
+
+		sclass->table[i].desc = &desc[i];
+		sclass->table[i].name = field;
+		sclass->table[i].len = len;
+
+		pos += size; //sizeof_ctype(class, var->type);
+	}
+	
+	#ifdef OS_64BITS
+	size = align_pos(pos, 8);
+	#else
+	size = align_pos(pos, 4);
+	#endif
+
+	size += sizeof(CSTRUCT);
+
+	#if DEBUG_STRUCT
+	fprintf(stderr, "  --> size = %d\n", size);
+	#endif
+	
+  CLASS_calc_info(sclass, 0, size, TRUE, 0);
+
+	CLASS_sort(sclass);
+	
+	if (sclass->debug)
+		sclass->load->sort = sclass->sort;
+	
+  CLASS_search_special(sclass);
+	/*for (i = 0; i < MAX_SPEC; i++)
+		sclass->special[i] = NO_SYMBOL;*/
+	
+	sclass->is_struct = TRUE;
+	
+  sclass->state = CS_READY;
+	sclass->ready = TRUE;
+	return;
+	
+__MISMATCH:
+
+  THROW(E_CLASS, ClassName, "Structure is declared elsewhere differently: ", sclass->name);
 }
 
-static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfirst)
+
+static void load_and_relocate(CLASS *class, int len_data, CLASS_DESC **pstart, int *pndesc)
 {
   char *section;
 	CLASS_INFO *info;
@@ -395,11 +483,16 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
   CLASS_PARAM *local;
   CLASS_EVENT *event;
   CLASS_EXTERN *ext;
+  CLASS_VAR *var;
   FUNCTION *func;
   FUNC_DEBUG *debug;
-  int i, j;
+  int i, j, pos;
   int offset;
-  short n_desc, n_class_ref, n_unknown, n_array;
+  short n_desc, n_class_ref, n_unknown, n_array, n_struct;
+	CLASS_STRUCT *structure = NULL;
+	int size;
+	char *name;
+	int len;
 	
   ALLOC_ZERO(&class->load, sizeof(CLASS_LOAD), "CLASS_load");
 
@@ -426,18 +519,24 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
   {
   	int fd;
   	
-  	fprintf(stderr, "Dumping class file at /tmp/gambas-bad-header.out: %d bytes\n", len_data);
-  	fd = open("/tmp/gambas-bad-header.out", O_CREAT | O_WRONLY, 0666);
-  	write(fd, class->data, len_data);
-  	close(fd);
+  	fd = open("/tmp/gambas-bad-header.dump", O_CREAT | O_WRONLY, 0666);
+		if (fd >= 0)
+		{
+			if (write(fd, class->data, len_data) != len_data)
+				fprintf(stderr, "Cannot dump bad class file.\n");
+			else
+				fprintf(stderr, "Bad class file dumped at /tmp/gambas-bad-header.dump\n");
+			close(fd);
+		}
+		
     THROW(E_CLASS, ClassName, "Bad header", "");
 	}
 	
-  check_version(header->version);
+  check_version(class, header->version);
 
   class->debug = header->flag & CF_DEBUG;
   
-  info = (CLASS_INFO *)get_section("info", &section, NULL, _s _s _i _i );
+  info = (CLASS_INFO *)get_section("info", &section, NULL, _s _s _i _i _s _s );
   #ifdef OS_64BITS
   class->load->desc =
   #endif
@@ -456,7 +555,19 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
   local = (CLASS_PARAM *)get_section("local", &section, NULL, _t);
   class->load->array = (CLASS_ARRAY **)get_section("array", &section, &n_array, _i);  // A special process is needed later
 
-  /* Loading code */
+	// Structure descriptions
+	
+	if (info->nstruct)
+	{
+		ALLOC(&structure, sizeof(CLASS_STRUCT) * info->nstruct, "load_and_relocate");
+		for (i = 0; i < info->nstruct; i++)
+		{
+			structure[i].desc = (int *)get_section("structure", &section, &n_struct, _i);
+			structure[i].nfield = (n_struct - 1) / 2;
+		}
+	}
+	
+  // Loading code
 
   for (i = 0; i < class->load->n_func; i++)
   {
@@ -474,7 +585,8 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
 
   if (class->debug)
   {
-    class->load->global = (GLOBAL_SYMBOL *)get_section("debug global", &section, &class->load->n_global, _s _s _p _c _i );
+    class->load->global = (GLOBAL_SYMBOL *)get_section("debug global", &section, &class->load->n_global, _p _i _c _i );
+    class->load->sort = (ushort *)get_section("debug global sort", &section, NULL, _s);
     #ifdef OS_64BITS
     class->load->debug =
     #endif
@@ -495,14 +607,14 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
     for (i = 0; i < class->load->n_func; i++)
     {
       func = &class->load->func[i];
-      func->debug->local = (LOCAL_SYMBOL *)get_section("debug local", &section, &func->debug->n_local, _s _s _p _i );
+      func->debug->local = (LOCAL_SYMBOL *)get_section("debug local", &section, &func->debug->n_local, _p _i _i );
     }
   }
 
-  /* Source file path */
+  /* Source file path, ingored now! */
 
   if (class->debug)
-    class->path = (char *)get_section("debug file name", &section, NULL, NULL);
+    get_section("debug file name", &section, NULL, NULL);
 
   /* Strings */
 
@@ -513,8 +625,31 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
   for (i = 0; i < n_class_ref; i++)
   {
   	offset = (int)(intptr_t)class->load->class_ref[i];
+		
+		// The compiler does not know if an array class is global or not, we must check now.
+		
+		if (offset >= 0)
+		{
+			name = &class->string[offset];
+			len = strlen(name);
+			
+			if (len >= 3 && name[len - 2] == '[')
+			{
+				do
+				{
+					len -= 2;
+				}
+				while (len >= 3 && name[len - 2] == '[');
+			
+				if (CLASS_look_global(name, len))
+					offset = (- offset);
+			}
+		}
+		
   	if (offset >= 0)
     	class->load->class_ref[i] = CLASS_find(&class->string[offset]);
+		else if (offset < -1)
+    	class->load->class_ref[i] = CLASS_find_global(&class->string[-offset]);
 		else
 			class->load->class_ref[i] = 0; //0x31415926; //CLASS_find(&class->string[-offset]);
 	}
@@ -561,6 +696,9 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
 				}
       }
 
+      for (j = 0; j < func->n_local; j++)
+      	conv_ctype(&func->local[j].type);
+
       local += func->n_local;
     }
   }
@@ -631,47 +769,63 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
     	#else
       class->load->array[i] = (CLASS_ARRAY *)((char *)class->load->array + ((int *)class->load->array)[i]);
       #endif
+      //conv_type(class, &class->load->array[i]->type);
     }
   }
 
-  /* Computes variable position again */
-  
+	// Create structures (we may need the structure size to compute the variable sizes)
+	
+	if (info->nstruct)
 	{
-		int pos;
-		int size;
-		CLASS_VAR *var;
-		
-		pos = 0;
-		for (i = 0; i < class->load->n_stat; i++)
-		{
-			var = &class->load->stat[i];
-			size = sizeof_ctype(class, var->type);
-			pos = align_pos(pos, size);
-			var->pos = pos;
-			pos += size;
-		}
-		#ifdef OS_64BITS
-		info->s_static = (pos + 7) & ~7;
-		#else
-		info->s_static = align_pos(pos, 4);
-		#endif
-  
-		pos = 0;
-		for (i = 0; i < class->load->n_dyn; i++)
-		{
-			var = &class->load->dyn[i];
-			size = sizeof_ctype(class, var->type);
-			pos = align_pos(pos, size);
-			var->pos = pos;
-			pos += size;
-		}
-		#ifdef OS_64BITS
-		info->s_dynamic = (pos + 7) & ~7;
-		#else
-		info->s_dynamic = align_pos(pos, 4);
-		#endif
+		for (i = 0; i < info->nstruct; i++)
+			load_structure(class, structure[i].desc, structure[i].nfield);
+		FREE(&structure, "load_and_relocate");
 	}
 
+  // Computes and align the position of each static and dynamic variables.
+	// Computes the total size needed accordingly.
+  
+  #ifdef DEBUG
+  fprintf(stderr, "Compute variable position for %s\n", class->name);
+  #endif
+  
+	pos = 0;
+	for (i = 0; i < class->load->n_stat; i++)
+	{
+		var = &class->load->stat[i];
+		conv_ctype(&var->type);
+		size = CLASS_sizeof_ctype(class, var->type);
+		pos = align_pos(pos, size);
+		var->pos = pos;
+		#ifdef DEBUG
+		fprintf(stderr, "Static #%d: %d\n", i, var->pos);
+		#endif
+		pos += size;
+	}
+	#ifdef OS_64BITS
+	info->s_static = align_pos(pos, 8);
+	#else
+	info->s_static = align_pos(pos, 4);
+	#endif
+
+	pos = 0;
+	for (i = 0; i < class->load->n_dyn; i++)
+	{
+		var = &class->load->dyn[i];
+		conv_ctype(&var->type);
+		size = CLASS_sizeof_ctype(class, var->type);
+		pos = align_pos(pos, size);
+		var->pos = pos;
+		#ifdef DEBUG
+		fprintf(stderr, "Dynamic #%d: %d\n", i, var->pos);
+		#endif
+		pos += size; //sizeof_ctype(class, var->type);
+	}
+	#ifdef OS_64BITS
+	info->s_dynamic = align_pos(pos, 8);
+	#else
+	info->s_dynamic = align_pos(pos, 4);
+	#endif
 
   /* String relocation */
 
@@ -720,17 +874,15 @@ static void load_and_relocate(CLASS *class, int len_data, int *pndesc, int *pfir
   //if (info->s_dynamic == 0)
   //  class->no_create = TRUE;
   
-  /* Descriptions */
-
-  CLASS_make_description(class, start, n_desc, pfirst);
-
   /* Class size and offsets */
 
   CLASS_calc_info(class, class->n_event, info->s_dynamic, FALSE, info->s_static);
-
+	
+	*pstart = start;
   *pndesc = n_desc;
 }
 
+
 void CLASS_load_without_init(CLASS *class)
 {
   int i;
@@ -748,14 +900,21 @@ void CLASS_load_without_init(CLASS *class)
   int first;
   int first_event;
   COMPONENT *save;
-
+	CLASS_DESC *start;
+	char kind;
+	
   //size_t alloc = MEMORY_size;
+	
+	if (class->state >= CS_LOADED)
+		return;
 
-  if (class->state >= CS_LOADED)
-    return;
+	if (class->error)
+		THROW(E_CLASS, class->name, "Loading has already failed", "");
+    
+	class->error = TRUE;
 
   #if DEBUG_LOAD
-    printf("Loading class %s (%p)...\n", class->name, class);
+    fprintf(stderr, "Loading class %s (%p)...\n", class->name, class);
   #endif
 
   #ifdef DEBUG
@@ -768,15 +927,26 @@ void CLASS_load_without_init(CLASS *class)
   if (class->in_load)
     THROW(E_CLASS, ClassName, "Circular reference", "");
 
-  if (COMPONENT_current)
-    class->component = COMPONENT_current;
-  else if (CP)
-    class->component = CP->component;
+	if (!class->component)
+	{
+		if (CP)
+			class->component = CP->component;
+		else
+			class->component = NULL;
+	}
+
+  #if DEBUG_COMP
+  if (class->component)
+    fprintf(stderr, "class %s -> component %s\n", class->name, class->component->name);
   else
-    class->component = NULL;
+    fprintf(stderr, "class %s -> no component\n", class->name);
+  #endif
 
   save = COMPONENT_current;
   COMPONENT_current = class->component;
+	#if DEBUG_LOAD
+		fprintf(stderr, "COMPONENT_current = %s\n", COMPONENT_current ? COMPONENT_current->name : "NULL");
+	#endif
 
   len = strlen(class->name);
 
@@ -807,17 +977,15 @@ void CLASS_load_without_init(CLASS *class)
   }
 
   COMPONENT_current = save;
+	#if DEBUG_LOAD
+		fprintf(stderr, "COMPONENT_current = %s\n", COMPONENT_current ? COMPONENT_current->name : "NULL");
+	#endif
 
   class->in_load = TRUE;
 
-  #if DEBUG_COMP
-  if (class->component)
-    fprintf(stderr, "class %s -> component %s\n", class->name, class->component->name);
-  else
-    fprintf(stderr, "class %s -> no component\n", class->name);
-  #endif
+	class->init_dynamic = TRUE;
 
-	load_and_relocate(class, len_data, &n_desc, &first);
+	load_and_relocate(class, len_data, &start, &n_desc);
 
   /* Information on static and dynamic variables */
 
@@ -832,19 +1000,6 @@ void CLASS_load_without_init(CLASS *class)
     var->pos += offset;
   }
 
-	#if 0
-	if (class->parent)
-	{
-		offset = class->parent->size_stat;
-
-		for (i = 0; i < class->load->n_stat; i++)
-		{
-			var = &class->load->stat[i];
-			var->pos += offset;
-		}
-	}
-	#endif
-
   /* Constant conversion & relocation */
 
   for (i = 0; i < class->load->n_cst; i++)
@@ -865,7 +1020,8 @@ void CLASS_load_without_init(CLASS *class)
 				#ifdef OS_64BITS
 				// Special process for long constants: the first 32 bits part of the LONG constant
 				// has been extended to 64 bits
-				cc->_swap.val[0] = (int)(*((int64_t *)(void *)&cc->_string.addr));
+				//cc->_swap.val[0] = (int)(*((int64_t *)(void *)&cc->_string.addr));
+				cc->_swap.val[0] = (int)(intptr_t)cc->_string.addr;
 				cc->_swap.val[1] = cc->_string.len;
 				#endif
 				/*
@@ -890,58 +1046,30 @@ void CLASS_load_without_init(CLASS *class)
 	      cc->_string.addr += (intptr_t)class->string;
 				if (NUMBER_from_string(NB_READ_FLOAT, cc->_string.addr, strlen(cc->_string.addr), &value))
 					THROW(E_CLASS, ClassName, "Bad constant", "");
-	      cc->_float.value = value._float.value;
+				if (cc->type == T_SINGLE)
+					cc->_single.value = (float)value._float.value;
+				else
+					cc->_float.value = value._float.value;
 				break;
     }
-    
-    #if 0
-    if (!TYPE_is_integer_long(cc->type))
-      cc->_string.addr += (int)class->string;
-
-    if (cc->type == T_FLOAT || cc->type == T_SINGLE)
-    {
-      if (NUMBER_from_string(NB_READ_FLOAT, cc->_string.addr, strlen(cc->_string.addr), &value))
-        THROW(E_CLASS, ClassName, "Bad constant", "");
-
-      cc->_float.value = value._float.value;
-    }
-    else if (cc->type == T_LONG)
-    {
-      /*
-      Each long value has been already swapped independently.
-      In the case of a LONG constant, we just have to swap the two 32 bits parts of the
-      constant value.
-      */
-      if (_swap)
-      {
-        int val;
-
-        val = cc->_swap.val[0];
-        cc->_swap.val[0] = cc->_swap.val[1];
-        cc->_swap.val[1] = val;
-      }
-    }
-    #ifdef OS_64BITS
-    // Special process for integer constants
-    else if (cc->type <= T_INTEGER)
-    {
-    	cc->_integer.value = (int)cc->_string.addr;
-    }
-    #endif
-    #endif
   }
 
   /* Class public description */
 
-  for (i = first; i < class->n_desc; i++)
+  for (i = 0; i < n_desc; i++)
   {
-    desc = class->table[i].desc;
+    desc = &start[i]; //class->table[i].desc;
 
-		desc->gambas.name = (char *)CLASS_DESC_get_type_name(desc);
+		//desc->gambas.name = (char *)CLASS_DESC_get_type_name(desc);
 
     conv_type(class, &desc->gambas.type);
+		
+		kind = *CLASS_DESC_get_type_name(desc);
 
-    switch (CLASS_DESC_get_type(desc))
+		if (!desc->gambas.val1 && index(CD_CALL_SOMETHING_LIST, kind) != NULL)
+			fprintf(stderr, "CLASS_load_without_init: '%s.%s' gambas.val1 == 0\n", class->name, desc->gambas.name);
+		
+    switch (kind)
     {
       case CD_METHOD:
       case CD_STATIC_METHOD:
@@ -958,11 +1086,13 @@ void CLASS_load_without_init(CLASS *class)
 
       case CD_PROPERTY:
       case CD_STATIC_PROPERTY:
+      case CD_PROPERTY_READ:
+      case CD_STATIC_PROPERTY_READ:
 
         desc->property.read = (void (*)())desc->gambas.val1;
         desc->property.write = (void (*)())desc->gambas.val2;
-        if ((intptr_t)desc->property.write == -1L)
-          desc->gambas.name = *desc->gambas.name == 'p' ? "r" : "R";
+        //if ((intptr_t)desc->property.write == -1L)
+        //  desc->gambas.name = *desc->gambas.name == 'p' ? "r" : "R";
         desc->property.native = FALSE;
 
         break;
@@ -970,11 +1100,12 @@ void CLASS_load_without_init(CLASS *class)
       case CD_VARIABLE:
       case CD_STATIC_VARIABLE:
 
-        if (CLASS_DESC_get_type(desc) == CD_STATIC_VARIABLE)
+        if (kind == CD_STATIC_VARIABLE)
           var = &class->load->stat[desc->gambas.val1];
         else
           var = &class->load->dyn[desc->gambas.val1];
 
+        desc->variable.ctype = var->type;
         desc->variable.offset = var->pos;
 
         break;
@@ -989,10 +1120,13 @@ void CLASS_load_without_init(CLASS *class)
           desc->constant.value._float = cc->_float.value;
         else if (desc->constant.type == T_LONG)
           desc->constant.value._long = cc->_long.value;
+        else if (desc->constant.type == T_SINGLE)
+          desc->constant.value._single = cc->_single.value;
         else
         {
           desc->constant.type = T_CSTRING;
           desc->constant.value._string = cc->_string.addr;
+					desc->constant.translate = (cc->type == T_CSTRING);
         }
 
         break;
@@ -1020,10 +1154,21 @@ void CLASS_load_without_init(CLASS *class)
 
         THROW(E_CLASS, ClassName, "Bad description", "");
     }
-
-    desc->method.class = class;
   }
 
+  /* Inheritance */
+
+  CLASS_make_description(class, start, n_desc, &first);
+
+	/* Transfer symbol kind into symbol name (which is stored in the symbol table now), like native classes */
+	
+  for (i = 0; i < n_desc; i++)
+  {
+    desc = &start[i];
+		desc->gambas.name = (char *)CLASS_DESC_get_type_name(desc);
+    desc->method.class = class;
+	}
+
   /* Event description */
 
   CLASS_make_event(class, &first_event);
@@ -1046,6 +1191,7 @@ void CLASS_load_without_init(CLASS *class)
   /* ...and usable ! */
 
   class->state = CS_LOADED;
+	class->error = FALSE;
 
 	/* Init breakpoints */
 
@@ -1062,7 +1208,7 @@ void CLASS_load_without_init(CLASS *class)
   //EXEC.func = &class->load->func[FUNC_INIT_STATIC];
 
   EXEC_function();
-
+	
   //total += MEMORY_size - alloc;
   //printf("%s: %d  TOTAL = %d\n", class->name, MEMORY_size - alloc, total);
 }
@@ -1070,7 +1216,6 @@ void CLASS_load_without_init(CLASS *class)
 
 void CLASS_load_real(CLASS *class)
 {
-#if 0
 	char *name = class->name;
 	int len = strlen(name);
 
@@ -1079,10 +1224,10 @@ void CLASS_load_real(CLASS *class)
 		CLASS_create_array_class(class);
 		return;
 	}
-#endif
 
-  CLASS_load_without_init(class);
+	CLASS_load_without_init(class);
   class->state = CS_READY;
+	class->ready = TRUE;
 
   EXEC_public(class, NULL, "_init", 0);
 }
