@@ -2,11 +2,11 @@
 
   CConnection.c
 
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -16,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -41,7 +42,7 @@ static CCONNECTION *_current = NULL;
 
 static GB_SUBCOLLECTION_DESC _databases_desc =
 {
-  ".ConnectionDatabases",
+  ".Connection.Databases",
   (void *)CDATABASE_get,
   (void *)CDATABASE_exist,
   (void *)CDATABASE_list,
@@ -50,7 +51,7 @@ static GB_SUBCOLLECTION_DESC _databases_desc =
 
 static GB_SUBCOLLECTION_DESC _users_desc =
 {
-  ".ConnectionUsers",
+  ".Connection.Users",
   (void *)CUSER_get,
   (void *)CUSER_exist,
   (void *)CUSER_list,
@@ -59,7 +60,7 @@ static GB_SUBCOLLECTION_DESC _users_desc =
 
 static GB_SUBCOLLECTION_DESC _tables_desc =
 {
-  ".ConnectionTables",
+  ".Connection.Tables",
   (void *)CTABLE_get,
   (void *)CTABLE_exist,
   (void *)CTABLE_list,
@@ -85,6 +86,8 @@ static void open_connection(CCONNECTION *_object)
 
 static bool check_opened(CCONNECTION *_object)
 {
+	DB_CurrentDatabase = &THIS->db;
+
 	if (!THIS->db.handle)
 		open_connection(THIS);
 
@@ -140,88 +143,61 @@ static void close_connection(CCONNECTION *_object)
 }
 
 
-// BEGIN_METHOD(CCONNECTION_new, GB_STRING url)
-BEGIN_METHOD_VOID(CCONNECTION_new)
+BEGIN_METHOD(CCONNECTION_new, GB_STRING url)
 
-  /*int i, state, p;
-  char c;*/
+	char *url, *name, *p;
 
   THIS->db.handle = NULL;
-  //THIS->db.flags.ignore_case = FALSE; // Now case is sensitive by default!
+  THIS->db.ignore_case = FALSE; // Now case is sensitive by default!
+  THIS->db.timeout = 20; // Connection timeout is 20 seconds by default
 
   if (_current == NULL)
     _current = THIS;
 
-#if 0
-  if (!MISSING(url))
-  {
-    char *url = GB.ToZeroString(ARG(url));
+	if (MISSING(url))
+		return;
+	
+	url = GB.ToZeroString(ARG(url));
+	
+	p = index(url, ':');
+	if (!p || p == url) goto __BAD_URL;
+	*p++ = 0;
+	if (p[0] != '/' || p[1] != '/') goto __BAD_URL;
+	p += 2;
+	
+	THIS->desc.type = GB.NewZeroString(url);
+	url = p;
+	
+	p = rindex(url, '/');
+	if (!p || p == url) goto __BAD_URL;
+	*p++ = 0;
+	
+	name = p;
+	
+	p = index(url, '@');
+	if (p)
+	{
+		if (p == url)
+			goto __BAD_URL;
+		*p = 0;
+		THIS->desc.user = GB.NewZeroString(url);
+		url = p + 1;
+	}
+	
+	p = index(url, ':');
+	if (p)
+	{
+		*p = 0;
+		THIS->desc.port = GB.NewZeroString(p + 1);
+	}
 
-    state = 0;
-    p = 0;
-    for (i = 0; i <= LENGTH(url); i++)
-    {
-      c = url[i];
+	THIS->desc.host = GB.NewZeroString(url);
+	THIS->desc.name = GB.NewZeroString(name);
 
-      switch (state)
-      {
-        case 0: /* type */
-          if (c == ':' || c == 0)
-          {
-            GB.NewString(&THIS->desc.type, &url[p], i - p);
-            state++;
-            p = i + 3; /* On saute '://' */
-          }
-          break;
-
-        case 1: /* // */
-        case 2:
-          if (c && c != '/')
-            goto BAD_URL;
-
-          state++;
-          break;
-
-        case 3: /* host */
-          if (c == ':' || c == '/' || c == 0)
-          {
-            if (i > p)
-              GB.NewString(&THIS->desc.host, &url[p], i - p);
-            p = i + 1;
-            state++;
-            if (c != ':')
-              state++;
-          }
-          break;
-
-        case 4: /* port */
-          if (c == '/' || c == 0)
-          {
-            if (i == p)
-              goto BAD_URL;
-
-            GB.NewString(&THIS->desc.port, &url[p], i - p);
-            p = i + 1;
-            state++;
-          }
-          break;
-
-        case 5: /* name */
-          if (c == 0 && i > p)
-            GB.NewString(&THIS->desc.name, &url[p], i - p);
-          break;
-      }
-    }
-  }
-
-  return;
-
-BAD_URL:
+__BAD_URL:
 
   GB.Error("Malformed URL");
-  return;
-#endif
-
+	
 END_METHOD
 
 
@@ -271,6 +247,16 @@ BEGIN_PROPERTY(CCONNECTION_version)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Connection_Timeout)
+
+  if (READ_PROPERTY)
+		GB.ReturnInteger(THIS->db.timeout);
+	else
+		THIS->db.timeout = VPROP(GB_INTEGER);
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(CCONNECTION_opened)
 
   CHECK_DB();
@@ -336,6 +322,17 @@ BEGIN_PROPERTY(CCONNECTION_ignore_case)
 END_PROPERTY
 #endif
 
+BEGIN_PROPERTY(CCONNECTION_ignore_charset)
+
+  CHECK_DB();
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(THIS->ignore_charset);
+	else
+		THIS->ignore_charset = VPROP(GB_BOOLEAN);
+
+END_PROPERTY
+
 BEGIN_METHOD_VOID(CCONNECTION_begin)
 
   CHECK_DB();
@@ -503,7 +500,7 @@ static char *get_query(char *prefix, CCONNECTION *_object, char *table, int len_
   if (query && len_query > 0)
   {
   	q_add(" ");
-  	if (strncmp(query, "WHERE ", 6))
+  	if (strncasecmp(query, "WHERE ", 6) && strncasecmp(query, "ORDER BY ", 9))
     	q_add("WHERE ");
     q_add_length(query, len_query);
 	}
@@ -568,6 +565,9 @@ BEGIN_METHOD(CCONNECTION_edit, GB_STRING table; GB_STRING query; GB_VALUE param[
 
   CHECK_OPEN();
 
+  /*if (check_table(THIS, table, TRUE))
+    return;*/
+
   query = get_query("SELECT * FROM", THIS, STRING(table), LENGTH(table),
     MISSING(query) ? NULL : STRING(query),
     MISSING(query) ? 0 : LENGTH(query),
@@ -584,17 +584,41 @@ BEGIN_METHOD(CCONNECTION_edit, GB_STRING table; GB_STRING query; GB_VALUE param[
 END_METHOD
 
 
-BEGIN_METHOD(CCONNECTION_quote, GB_STRING name)
+BEGIN_METHOD(CCONNECTION_quote, GB_STRING name; GB_BOOLEAN is_table)
+
+	char *name = STRING(name);
+	int len = LENGTH(name);
 
   CHECK_DB();
   CHECK_OPEN();
 
-  q_init();
-  q_add(THIS->driver->GetQuote());
-  q_add_length(STRING(name), LENGTH(name));
-  q_add(THIS->driver->GetQuote());
+	if (VARGOPT(is_table, FALSE) && THIS->db.flags.schema)
+		GB.ReturnNewZeroString(DB_GetQuotedTable(THIS->driver, &THIS->db, GB.ToZeroString(ARG(name))));
+	else
+	{
+		q_init();
+		q_add(THIS->driver->GetQuote());
+		q_add_length(name, len);
+		q_add(THIS->driver->GetQuote());
+		GB.ReturnString(q_get());
+	}
 
-  /* q_get() returns a gambas string */
+END_METHOD
+
+
+BEGIN_METHOD(CCONNECTION_format_blob, GB_STRING data)
+
+	DB_BLOB blob;
+
+  CHECK_DB();
+  CHECK_OPEN();
+
+	blob.data = STRING(data);
+	blob.length = LENGTH(data);
+	
+  q_init();
+	DB_CurrentDatabase = &THIS->db;
+	(*THIS->driver->FormatBlob)(&blob, q_add_length);
   GB.ReturnString(q_get());
 
 END_METHOD
@@ -694,11 +718,20 @@ BEGIN_PROPERTY(CCONNECTION_debug)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Connection_Handle)
+
+  CHECK_DB();
+  CHECK_OPEN();
+	GB.ReturnPointer(THIS->db.handle);
+
+END_PROPERTY
+
+
 GB_DESC CConnectionDesc[] =
 {
   GB_DECLARE("Connection", sizeof(CCONNECTION)),
 
-  GB_METHOD("_new", NULL, CCONNECTION_new, NULL), //"[(URL)s]"),
+  GB_METHOD("_new", NULL, CCONNECTION_new, "[(DatabaseURL)s]"),
   GB_METHOD("_free", NULL, CCONNECTION_free, NULL),
 
   GB_PROPERTY("Type", "s", CCONNECTION_type),
@@ -708,11 +741,13 @@ GB_DESC CConnectionDesc[] =
   GB_PROPERTY("Password", "s", CCONNECTION_password),
   GB_PROPERTY("Name", "s", CCONNECTION_name),
   GB_PROPERTY("Port", "s", CCONNECTION_port),
+  GB_PROPERTY("Timeout", "i", Connection_Timeout),
   GB_PROPERTY_READ("Charset", "s", CCONNECTION_charset),
   GB_PROPERTY_READ("Version", "i", CCONNECTION_version),
   GB_PROPERTY_READ("Opened", "b", CCONNECTION_opened),
   GB_PROPERTY_READ("Error", "i", CCONNECTION_error),
-  //GB_PROPERTY("IgnoreCase", "b", CCONNECTION_ignore_case),
+  GB_PROPERTY("IgnoreCharset", "b", CCONNECTION_ignore_charset),
+  GB_STATIC_PROPERTY_READ("Handle", "p", Connection_Handle),
 
   GB_METHOD("Open", NULL, CCONNECTION_open, NULL),
   GB_METHOD("Close", NULL, CCONNECTION_close, NULL),
@@ -729,11 +764,12 @@ GB_DESC CConnectionDesc[] =
   GB_METHOD("Commit", NULL, CCONNECTION_commit, NULL),
   GB_METHOD("Rollback", NULL, CCONNECTION_rollback, NULL),
 
-  GB_METHOD("Quote", "s", CCONNECTION_quote, "(Name)s"),
+  GB_METHOD("Quote", "s", CCONNECTION_quote, "(Name)s[(Table)b]"),
+  GB_METHOD("FormatBlob", "s", CCONNECTION_format_blob, "(Data)s"),
 
-  GB_PROPERTY("Tables", ".ConnectionTables", CCONNECTION_tables),
-  GB_PROPERTY("Databases", ".ConnectionDatabases", CCONNECTION_databases),
-  GB_PROPERTY("Users", ".ConnectionUsers", CCONNECTION_users),
+  GB_PROPERTY("Tables", ".Connection.Tables", CCONNECTION_tables),
+  GB_PROPERTY("Databases", ".Connection.Databases", CCONNECTION_databases),
+  GB_PROPERTY("Users", ".Connection.Users", CCONNECTION_users),
   //GB_PROPERTY("Views", ".ConnectionViews", CCONNECTION_views),
 
   GB_CONSTANT("_Properties", "s", "Type,Host,Login,Password,Name,Port"),
@@ -764,7 +800,8 @@ GB_DESC CDBDesc[] =
   GB_STATIC_PROPERTY_READ("Version", "i", CCONNECTION_version),
   GB_STATIC_PROPERTY_READ("Opened", "b", CCONNECTION_opened),
   GB_STATIC_PROPERTY_READ("Error", "i", CCONNECTION_error),
-  //GB_STATIC_PROPERTY("IgnoreCase", "b", CCONNECTION_ignore_case),
+  GB_STATIC_PROPERTY("IgnoreCharset", "b", CCONNECTION_ignore_charset),
+  GB_STATIC_PROPERTY_READ("Handle", "p", Connection_Handle),
 
   GB_STATIC_PROPERTY("Debug", "b", CCONNECTION_debug),
 
@@ -780,12 +817,13 @@ GB_DESC CDBDesc[] =
   GB_STATIC_METHOD("Commit", NULL, CCONNECTION_commit, NULL),
   GB_STATIC_METHOD("Rollback", NULL, CCONNECTION_rollback, NULL),
 
-  GB_STATIC_METHOD("Quote", "s", CCONNECTION_quote, "(Name)s"),
+  GB_STATIC_METHOD("Quote", "s", CCONNECTION_quote, "(Name)s[(Table)b]"),
+  GB_STATIC_METHOD("FormatBlob", "s", CCONNECTION_format_blob, "(Data)s"),
 
-  GB_STATIC_PROPERTY("Tables", ".ConnectionTables", CCONNECTION_tables),
+  GB_STATIC_PROPERTY("Tables", ".Connection.Tables", CCONNECTION_tables),
   //GB_STATIC_PROPERTY("Views", ".ConnectionViews", CCONNECTION_views),
-  GB_STATIC_PROPERTY("Databases", ".ConnectionDatabases", CCONNECTION_databases),
-  GB_STATIC_PROPERTY("Users", ".ConnectionUsers", CCONNECTION_users),
+  GB_STATIC_PROPERTY("Databases", ".Connection.Databases", CCONNECTION_databases),
+  GB_STATIC_PROPERTY("Users", ".Connection.Users", CCONNECTION_users),
 
   GB_END_DECLARE
 };
