@@ -2,11 +2,11 @@
 
   CDraw.h
 
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -16,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -30,11 +31,15 @@
 
 extern GB_DESC CDrawDesc[];
 extern GB_DESC CDrawClipDesc[];
+extern GB_DESC CDrawStyleDesc[];
 
 #endif
 
 GB_DRAW *DRAW_get_current();
+GB_DRAW *DRAW_from_device(void *device);
 bool DRAW_begin(void *device);
 void DRAW_end();
+bool DRAW_open(GB_DRAW *draw);
+void DRAW_close(GB_DRAW *draw);
 
 #endif
