@@ -2,13 +2,11 @@
 
   gb_common_swap.h
 
-  Swapping routines for endianess management
-
-  Copyright (c) 2000-2007 Benoit Minisini <gambas@freesurf.fr>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,20 +16,27 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #ifndef __GB_COMMON_SWAP_H
 #define __GB_COMMON_SWAP_H
 
-PUBLIC void SWAP_int(int *val);
-PUBLIC void SWAP_ints(int *val, int n);
+void SWAP_int(int *val);
+void SWAP_ints(int *val, int n);
 
-PUBLIC void SWAP_short(short *val);
-PUBLIC void SWAP_double(double *val);
+void SWAP_short(short *val);
+void SWAP_double(double *val);
 
 #define SWAP_float(_val) SWAP_int((int *)_val)
-#define SWAP_int64(_val) SWAP_double((double *)_val)
+#define SWAP_int64(_val) SWAP_double((double *)(void *)_val)
+
+#if OS_64BITS
+#define SWAP_pointer(_val) SWAP_int64(_val)
+#else
+#define SWAP_pointer(_val) SWAP_int(((int *)(void *)_val))
+#endif
 
 #endif
