@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  replace.h
+  gb_replace.h
 
-  Replacement functions
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -27,6 +26,7 @@
 
 #include <config.h>
 #include <gb_common.h>
+#include <termios.h>
 
 #ifndef HAVE_SETENV
 int setenv(const char *name, const char *value, int overwrite);
@@ -44,4 +44,8 @@ int getdomainname(char *name, size_t len);
 int getpt(void);
 #endif
 
+#ifndef HAVE_CFMAKERAW
+void cfmakeraw(struct termios *termios_p);
+#endif
+
 #endif
